/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * KumbangProperties.java
 *
 * Created on 3. kes�kuuta 2004, 15:20
 */

package kumbang.configurator.client.core;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

/**
 * 
 * @author vmyllarn
 */
public class ClientProperties {

	public static final int EXPORT_FORMAT_TEXT = 1;
	
	public static final int EXPORT_FORMAT_XML = 2;
	
	public static final int EXPORT_FORMAT_GLUE = 3;
	public static final int EXPORT_FORMAT_TUEXML = 4;
	public static final int EXPORT_FORMAT_ACCESS = 5;
	
	public static final String EXTENSION_MODEL = ".kbm";

	public static final String EXTENSION_CONFIGURATION = ".kbc";

	public static final String EXTENSION_EXPORT = ".kbe";

	public static final String EXTENSION_EXPORT_XML = ".xml";

	public static final String PROPERTIES_COMMUNICATION_METHOD = "communication_method";

	public static final String PROPERTIES_CLIENT = "client";

	public static final String PROPERTIES_ROOTDIR = "rootdir";

	public static final String COMMUNICATION_METHOD_RMI = "rmi";

	public static final String COMMUNICATION_METHOD_LOCAL = "local";

	public static final String PROPERTIES_FILE_NAME = "kumbang.properties";

	public static final String CLIENT_ECLIPSE = "eclipse";

	public static final String CLIENT_SWING = "swing";

	public static final String CLIENT_FITNESSE = "fitnesse";

	protected Properties properties;

	private static ClientProperties clientProperties;

	/** Creates a new instance of KumbangProperties */
	private ClientProperties() {
		properties = new Properties();
		// Read properties from .properties file
		try {
			properties.load(ClientProperties.class.getClassLoader()
					.getResourceAsStream(PROPERTIES_FILE_NAME));

		} catch (IOException e) {
			e.getStackTrace().toString();
			System.out
					.println("Properties file (kumbang.properties) not found.");
		}
	}

	private ClientProperties(String path) {
		properties = new Properties();
		// Read properties from .properties file
		try {
			properties.load(new FileInputStream(path));
		} catch (IOException e) {
			e.getStackTrace().toString();
			// properties.setProperty(PROPERTIES_COMMUNICATION_METHOD,
			// COMMUNICATION_METHOD_RMI);
			System.out
					.println("Properties file (kumbang.properties) not found.");
			System.out.println("Exiting...");
			System.exit(0);
		}
	}

	public static ClientProperties getInstance() {
		if (clientProperties == null) {
			clientProperties = new ClientProperties();
		}
		return clientProperties;
	}

	public static ClientProperties getInstance(String path) {
		if (clientProperties == null) {
			clientProperties = new ClientProperties(path);
		}
		return clientProperties;
	}

	public String getCommunicationMethod() {
		return properties.getProperty(PROPERTIES_COMMUNICATION_METHOD);
	}

	public void setCommunicationMethod(String method) {
		method = method.toLowerCase();
		properties.setProperty(PROPERTIES_COMMUNICATION_METHOD, method);
		if (!(method.equals(COMMUNICATION_METHOD_LOCAL))
				|| method.equals(COMMUNICATION_METHOD_RMI)) {
			throw new IllegalArgumentException("Illegal Communication method.");
		}
	}

	public void setClient(String client) {

		if (!(client.equals(CLIENT_ECLIPSE) || (client.equals(CLIENT_SWING) || (client
				.equals(CLIENT_FITNESSE))))) {
			throw new IllegalArgumentException("Illegal Client");
		} else {
			properties.setProperty(PROPERTIES_CLIENT, client);
		}
	}

	public String getClient() {
		return properties.getProperty(PROPERTIES_CLIENT);
	}

	public void setRootDir(String dir) {
		properties.setProperty(PROPERTIES_ROOTDIR, dir);
	}

	public String getRootDir() {
		return properties.getProperty(PROPERTIES_ROOTDIR);
	}

	public void setAbsolutePath(String path) {
		try {
			properties.load(new FileInputStream(path));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
