/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * ClientServices.java
 *
 * Created on 29. joulukuuta 2004, 15:26
 */

package kumbang.configurator.client.core;

import kumbang.core.configuration.core.ConfigurationException;

import kumbang.core.model.exception.ModelException;

import kumbang.core.configuration.instance.ComponentInstance;
import kumbang.core.configuration.instance.FeatureInstance;
import kumbang.core.configuration.instance.Instance;
import kumbang.core.configuration.task.ClientTask;

import kumbang.configurator.client.core.exception.*;


import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * 
 * @author vmyllarn
 */
public class ClientServices {

	private ConfigurationClient client;

	/** Creates a new instance of ClientServices */
	public ClientServices(ConfigurationClient client) {
		this.client = client;
	}

	public ComponentInstance getComponentRoot() {
		return client.getConfigurationHandler().getComponentRoot();
	}

	public FeatureInstance getFeatureRoot() {
		return client.getConfigurationHandler().getFeatureRoot();
	}

	public TaskHandler getTaskHandler() {
		return client.getTaskHandler();
	}

	public File getSaveFile() {
		return client.getSaveHandler().getSaveFile();
	}

	/**
	 * Return name of the current configuration
	 * 
	 * @return String name of the configuration
	 */
	public String getConfigurationName() {
		return client.getConfigurationHandler().getConfigurationName();
	}

	/**
	 * Return name of the current model
	 * 
	 * @return String name of the model
	 */
	public String getModelName() {
		return client.getConfigurationHandler().getModelName();
	}

	public boolean isModelInitialized() {
		if (client.getConfigurationHandler().getState() != 0)
			return true;
		else
			return false;
	}

	public boolean isConfigurationModified() {
		return client.getConfigurationHandler().isModified();
	}

	public boolean isConfigurationStateReseted() {
		return client.getConfigurationHandler().getState() == ConfigurationHandler.STATE_RESET;
	}

	public boolean isConfigurationStateComplete() {
		return client.getConfigurationHandler().getState() == ConfigurationHandler.STATE_COMPLETE;
	}

	public boolean isConfigurationStateConsistent() {
		return client.getConfigurationHandler().getState() == ConfigurationHandler.STATE_INCOMPLETE;
	}

	public void exit() {
		client.exit();
	}

	public String[] getModelNames() {
		return client.getModelNames();
	}

	public void openModel(String modelName, String configurationName) throws ClientException,
			ModelException {
		client.openModel(modelName, configurationName);
	}

	public void closeConfiguration() {
		client.closeConfiguration();
	}

	public void importModel(File file, String name) throws ClientException, ModelException {
		client.importModel(file, name);
	}

	public void exportConfiguration(File exportFile, int format) throws ExportException {
    	List<Instance> roots = new ArrayList<Instance>();
    	roots.add(client.getConfigurationHandler().getFeatureRoot());
    	roots.add(client.getConfigurationHandler().getComponentRoot());
		client.getExportHandler().export(
				exportFile,
				client.getConfigurationHandler().getConfigurationName(),
				format, 
				roots);
	}

	public void saveConfiguration() throws SaveException {
		client.getSaveHandler().saveConfiguration();
	}

	public void saveConfiguration(File saveFile) throws SaveException {
		client.getSaveHandler().saveConfiguration(saveFile);
	}

	public void openConfiguration(File openFile) throws OpenException, ClientException {
		client.openConfiguration(openFile);
	}

	public void findCompleteConfiguration() throws ClientException {
		client.findCompleteConfiguration();
	}

	public ClientTask handleTask(ClientTask task) throws ClientException, ConfigurationException {
		return client.applyTask(task);
	}

	public void removeModel(String modelName) throws ClientException, ModelException {
		client.removeModel(modelName);
		client.getModelNames();
	}

	public int numberOfSessions() throws ClientException {
		return client.numberOfSessions();
	}
}
