/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.core;

import kumbang.configurator.client.gui.core.MainGUI;

import kumbang.util.gui.KumbangMetalTheme;

import kumbang.util.KumbangLogger;

import java.io.File;
import java.io.IOException;

import java.rmi.RemoteException;

import java.util.logging.FileHandler;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import javax.swing.plaf.metal.MetalLookAndFeel;

public class KumbangConfigurator {
    
    public static Logger logger;
    
    public static void main(String[] args) {
        ClientProperties.getInstance().setClient(ClientProperties.CLIENT_SWING);
    	MetalLookAndFeel.setCurrentTheme(new KumbangMetalTheme());
        try {              
            UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
        } catch (Exception e) {}
        
        try {
            if (args.length > 0) {
                new KumbangConfigurator(args[0]);
            }
            else {
                new KumbangConfigurator(null);
            }
        } catch (IOException io) {
            System.out.println("IO Exception when initializing logging: "+io);
        }
    }
    
    public static void logConfig(String str) {
    	if (logger != null) {
    		logger.config(str);
    	}
    }
    
    public KumbangConfigurator(String logLevel) throws IOException{
        System.out.println("KumbangConfigurator: Initializing logging");
        initLogger(logLevel);
            
        try {
        	ClientProperties.getInstance();
        	System.out.println("Communication method: " + ClientProperties.getInstance().getCommunicationMethod());
            System.out.println("KumbangConfigurator: Initializing configuration client");
            ConfigurationClient client = new ConfigurationClient(ClientProperties.getInstance().getCommunicationMethod());
            ClientServices services = new ClientServices(client);
            System.out.println("KumbangConfigurator: Initializing graphical user interface");
            MainGUI mainGUI = new MainGUI(services);

            SwingUtilities.updateComponentTreeUI(mainGUI);
            mainGUI.repaint();
            
            mainGUI.setVisible(true);
            
        } catch (RemoteException re) {
            logger.severe("Cannot initialise connection to configuration server:\n"+re);
        }
    }
    
    public static void initLogger(String logLevel) throws IOException {
        Level level = Level.CONFIG;
        if (logLevel != null) {
            if (logLevel.equalsIgnoreCase("fine") && logLevel.equalsIgnoreCase("-fine")) {
                level = Level.FINE;
            }
            else if (logLevel.equalsIgnoreCase("finer") && logLevel.equalsIgnoreCase("-finer")) {
                level = Level.FINER;
            }
            else if (logLevel.equalsIgnoreCase("finest") && logLevel.equalsIgnoreCase("-finest")) {
                level = Level.FINEST;
            }
            else if (logLevel.equalsIgnoreCase("all") && logLevel.equalsIgnoreCase("-all")) {
                level = Level.ALL;
            }
        }
        KumbangLogger.initLogger("client");
        
        if (logger == null) {
            logger = Logger.getLogger("client");
            logger.setLevel(Level.ALL);

            File logDir = new File("logs");
            if (!logDir.isDirectory()) {
                if (!logDir.mkdir()) {
                    throw new IOException("Cannot create log directory \"logs\"!");
                }
            }
            Handler fHandler = new FileHandler("logs"+File.separator+"clientlog.txt");
            fHandler.setLevel(level);
            fHandler.setFormatter(new SimpleFormatter());
            logger.addHandler(fHandler);
        }
    }
    
  
}

