/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * SaveHandler.java
 *
 * Created on 27. joulukuuta 2004, 15:35
 */

package kumbang.configurator.client.core;

import kumbang.core.configuration.core.Configuration;

import kumbang.configurator.client.core.exception.SaveException;
import kumbang.configurator.client.core.exception.OpenException;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 *
 * @author  vmyllarn
 */
public class SaveHandler {
    
    private ConfigurationHandler cHandler;
    
    private File saveFile;
    
    
    /** Creates a new instance of SaveHandler */
    public SaveHandler(ConfigurationHandler cHandler) {
        this.cHandler = cHandler;
    }
    
    public void reset() {
        this.saveFile = null;
    }
    
    public boolean hasBeenSaved() {
        return this.saveFile != null;
    }
    
    public File getSaveFile() {
        return this.saveFile;
    }
    
    public void saveConfiguration(File saveFile) throws SaveException {
        try {
            FileOutputStream fos = new FileOutputStream(saveFile);
            ObjectOutputStream oos = new ObjectOutputStream(fos);
            oos.writeUTF(cHandler.getModelName());
            oos.writeUTF(cHandler.getConfigurationName());
            oos.writeObject(cHandler.getConfiguration());
            this.saveFile = saveFile;
            oos.close();
        } catch (IOException ioe) {
            throw new SaveException(saveFile, ioe.toString());
        }
    }
    
    public void saveConfiguration() throws SaveException {
        if (saveFile != null) {
            saveConfiguration(saveFile);
        }
    }
    
    public void openConfiguration(File openFile) throws OpenException {
        try {
            FileInputStream fis = new FileInputStream(openFile);
            ObjectInputStream ois = new ObjectInputStream(fis);
            String modelName = ois.readUTF();
            String configurationName = ois.readUTF();
            Configuration config = (Configuration)ois.readObject();
            ois.close();
            if (config != null) {
                cHandler.initConfiguration(modelName, configurationName, config);
                // Set the opened file as the current saveFile..
                this.saveFile = openFile;
            }
            else {
                throw new OpenException(openFile, "File does not contain a saved configuration.");
            }
        } catch (ClassNotFoundException cnfe) {
            throw new OpenException(openFile, "File does not contain a saved configuration.");
        } catch (IOException ioe) {
            throw new OpenException(openFile, ioe.toString());
        }
    }
    
}
