/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * TaskHandler.java
 *
 * Created on 26. lokakuuta 2004, 10:31
 */

package kumbang.configurator.client.core;

import java.util.Iterator;

import kumbang.core.configuration.instance.AttributedInstance;
import kumbang.core.configuration.instance.ComponentInstance;
import kumbang.core.configuration.instance.ComposableInstance;
import kumbang.core.configuration.instance.Instance;
import kumbang.core.configuration.instance.InterfaceInstance;
import kumbang.core.configuration.instance.PartInstance;
import kumbang.core.configuration.task.AddBindingTask;
import kumbang.core.configuration.task.AddComposableTask;
import kumbang.core.configuration.task.EditAttributesTask;
import kumbang.core.configuration.task.EditInterfacesTask;
import kumbang.core.configuration.task.RemoveInstanceTask;
import kumbang.core.model.type.ComponentType;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.InterfaceDefinition;
import kumbang.core.model.type.InterfaceType;
import kumbang.core.model.type.PartDefinition;

/**
 * This class seems to be a bit thin. Is there really need for this, or should the functionality
 * be merged into ConfigurationHandler. Also, the logic is in most cases kind of simple, although
 * not in all cases. Also, ClientTask objects are manipulated directly instead of using this class.
 * -toasikai
 * 
 * TODO The instances of this class have no state. The methods could be made static.
 * 
 * @author vmyllarn
 */
public class TaskHandler {

	/** Creates a new instance of TaskManager */
	public TaskHandler() {}

	public AddComposableTask createAddComposableTask(PartInstance partInstance, String typeString) {
		PartDefinition def = partInstance.getDefinition();
		for (Iterator it = def.getAllPossibleTypesIterator(false); it.hasNext(); ) {
			ComposableType comp = (ComposableType) it.next();
			if (typeString.equals(comp.toString()))
				return createAddComposableTask(partInstance, comp);
		}
		
		// A type maching the argument typeString was not found.
		return null;
	}
	
	public AddComposableTask createAddComposableTask(PartInstance partInstance, ComposableType type) {
		AddComposableTask task = new AddComposableTask(partInstance, type, true);
		return task;
	}

	public AddBindingTask createAddBindingTask(InterfaceInstance i1, InterfaceInstance i2) {
		AddBindingTask task = null;

		if (i1.isRequired() && i2.isProvided()) {
			task = new AddBindingTask(i1, i2, true);
		} else if (i1.isProvided() && i2.isRequired()) {
			task = new AddBindingTask(i2, i1, true);

		} else if (i1.isRequired() && i2.isRequired()) {
			ComponentInstance parent = (ComponentInstance) i1.getParent().getParent();
			if (parent != null && parent.containsInterface(i2)) {
				task = new AddBindingTask(i1, i2, true);
			}

		} else if (i1.isProvided() && i2.isProvided()) {
			ComponentInstance parent = (ComponentInstance) i2.getParent().getParent();
			if (parent != null && parent.containsInterface(i1)) {
				task = new AddBindingTask(i2, i1, true);
			}
		}
		
		if (task != null) {
			if (task.getSource().isBoundTo(task.getTarget()))
				return null;
		}

		if (task == null) {
			task = new AddBindingTask(null, null, true);
			task.markError("Interfaces " + i1 + " and " + i2 + " cannot be bound together.");
		}

		return task;
	}

	public RemoveInstanceTask createRemoveInstanceTask(Instance instance) {
		RemoveInstanceTask task = new RemoveInstanceTask(instance);
		return task;
	}
    
	public void addInterfaceToAddComposableTask(AddComposableTask task,
			InterfaceDefinition definition, InterfaceType type) {
		task.addInterface(definition, type);
	}

//	private InterfaceDefType getDefinitionAndType(ComponentType compType, String defString,
//			String typeString) {
//		InterfaceDefinition intDef = compType.getInterface(defString);
//		if (intDef == null)
//			return null;
//		
//		InterfaceType intType = intDef.getType(typeString);
//		if (intType == null)
//			return null;
//		
//		return new InterfaceDefType(intDef, intType);
//	}

//	/*
//	 * TODO EditComposableTask and AddComposableTask could share a supertype.
//	 */
//	public void addInterfaceToEditComposableTask(AddComposableTask task,
//			String defString, String typeString) {
//		ComponentType compType = (ComponentType) task.getType();
//		InterfaceDefType idt = getDefinitionAndType(compType, defString, typeString);
//		
//		if (idt == null)
//			return;
//		
//		task.addInterface(idt.def, idt.type);
//	}
    
//    /*
//     * TODO Added this (plahti2) - Is the method above unnecessary (it takes a
//     * funny argument) ?
//     */
//    public void addInterfaceToEditComposableTask(EditComposableTask task,
//            String defString, String typeString) {
//        
//        ComponentType compType = (ComponentType) task.getInstance().getComposableType();
//        InterfaceDefType idt = getDefinitionAndType(compType, defString, typeString);
//        
//        if (idt == null)
//            return;
//        
//        task.addInterface(idt.def, idt.type);
//    }
//	
//	public void addInterfaceToEditComposableTask(EditComposableTask task,
//			InterfaceDefinition definition, InterfaceType type) {
//		task.addInterface(definition, type);
//	}
//
//	private class InterfaceDefType {
//		public InterfaceDefType(InterfaceDefinition intDef, InterfaceType intType) {
//			this.def = intDef;
//			this.type = intType;
//		}
//		public InterfaceDefinition def;
//		public InterfaceType type;
//	}
    
    
}
