/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * ClientMenu.java
 *
 * Created on 6. huhtikuuta 2004, 13:50
 */

package kumbang.configurator.client.gui.core;

import kumbang.util.gui.Icons;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

/**
 *
 * @author  vmyllarn
 */
public class ClientMenuBar extends JMenuBar implements ActionListener {
    
    private MainGUI mainGUI;
    
    private JMenuItem openModelItem;
    private JMenuItem openConfigItem;
    private JMenuItem closeItem;
    private JMenuItem saveItem;
    
    private JMenuItem exitItem;
    private JMenuItem importItem;
    private JMenuItem exportItem;
    
    private JMenuItem newFeatureItem;
    private JMenuItem newComponentItem;
    private JMenuItem findConfigurationItem;
    private JMenuItem confPreferencesItem;
    
    private JCheckBoxMenuItem viewNameItem;
    private JCheckBoxMenuItem viewTypeItem;
    private JCheckBoxMenuItem viewConnectionItem;
    private JCheckBoxMenuItem viewPotentialItem;
    private JCheckBoxMenuItem largeTreeIcons;

    
    /** Creates a new instance of ClientMenu */
    public ClientMenuBar(MainGUI mainGUI) {
        this.mainGUI = mainGUI;
        
        add(initFileMenu());
        add(initViewMenu());
        add(initConfigurationMenu());
    }
    
    private JMenu initFileMenu() {
        JMenu fileMenu = new JMenu("File");
        
        openModelItem = new JMenuItem("Init configuration for model", Icons.OPEN);
        openConfigItem = new JMenuItem("Open saved configuration", Icons.OPEN);
        closeItem = new JMenuItem("Close configuration", Icons.CLOSE);
        saveItem = new JMenuItem("Save configuration", Icons.SAVE);
        importItem = new JMenuItem("Import configuration model", Icons.IMPORT);
        exportItem = new JMenuItem("Export configuration description", Icons.EXPORT);
        exitItem = new JMenuItem("Exit");
        

        fileMenu.add(openModelItem);
        fileMenu.add(saveItem);
        fileMenu.add(openConfigItem);
        fileMenu.add(closeItem);
        
        fileMenu.addSeparator();
        fileMenu.add(importItem);
        fileMenu.add(exportItem);
        
        fileMenu.addSeparator();        
        fileMenu.add(exitItem);

        openModelItem.addActionListener(this);
        openConfigItem.addActionListener(this);
        closeItem.addActionListener(this);
        saveItem.addActionListener(this);
        importItem.addActionListener(this);
        exportItem.addActionListener(this);
        exitItem.addActionListener(this);
        
        openModelItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_N, ActionEvent.CTRL_MASK));
        openConfigItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_O, ActionEvent.CTRL_MASK));
        closeItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_C, ActionEvent.CTRL_MASK));
        saveItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_S, ActionEvent.CTRL_MASK));
        importItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_I, ActionEvent.CTRL_MASK));
        exportItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_E, ActionEvent.CTRL_MASK));
        exitItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
        
        fileMenu.setMnemonic(KeyEvent.VK_F);
        return fileMenu;
    }
    
    private JMenu initConfigurationMenu() {
        JMenu confMenu = new JMenu("Configuration");
                
        newFeatureItem 
                = new JMenuItem("Add new feature", Icons.NEW_FEATURE);
        newComponentItem 
                = new JMenuItem("Add new component", Icons.NEW_COMPONENT);
        findConfigurationItem 
                = new JMenuItem("Find complete configuration", Icons.COMPUTE);
        confPreferencesItem 
                = new JMenuItem("Configuration preferences", Icons.COMPUTE);
        
        confMenu.add(newFeatureItem);
        confMenu.add(newComponentItem);        
        confMenu.addSeparator();
        confMenu.add(findConfigurationItem);
        confMenu.add(confPreferencesItem);
      
        newComponentItem.addActionListener(this);
        newFeatureItem.addActionListener(this);
        findConfigurationItem.addActionListener(this);
        confPreferencesItem.addActionListener(this);
        confPreferencesItem.setEnabled(false);
        
        newFeatureItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));
        newComponentItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F2, 0));
        findConfigurationItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F3, 0));
        
        confMenu.setMnemonic(KeyEvent.VK_C);
        return confMenu;
    }
    
    private JMenu initViewMenu() {
        JMenu viewMenu = new JMenu("View");
        
        largeTreeIcons = new JCheckBoxMenuItem("Show large icons in trees", false);
        viewNameItem = new JCheckBoxMenuItem("Show names", true);
        viewTypeItem = new JCheckBoxMenuItem("Show types", true);
        viewConnectionItem = new JCheckBoxMenuItem("Show bindings in diagram", true);
        viewPotentialItem = new JCheckBoxMenuItem("Show parts in diagram", true);
        
        viewMenu.add(largeTreeIcons);
        viewMenu.addSeparator();
        viewMenu.add(viewNameItem);
        viewMenu.add(viewTypeItem);
        viewMenu.add(viewConnectionItem);
        viewMenu.add(viewPotentialItem);
        
        largeTreeIcons.addActionListener(this);
        viewNameItem.addActionListener(this);
        viewTypeItem.addActionListener(this);
        viewConnectionItem.addActionListener(this);
        viewPotentialItem.addActionListener(this);
        
        largeTreeIcons.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_L, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        viewNameItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_N, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        viewTypeItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_T, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        viewConnectionItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_B, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        viewPotentialItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_P, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        
        viewMenu.setMnemonic(KeyEvent.VK_V);
        return viewMenu;
    }
    
    public void actionPerformed(ActionEvent e) {
        Object o = e.getSource();
        if (o == exitItem) {
            mainGUI.exit();
        }
        else if (o == openModelItem) {
            mainGUI.openModel();
        }
        else if (o == openConfigItem) {
            mainGUI.openConfiguration();
        }
        else if (o == closeItem) {
            mainGUI.closeConfiguration();
        }
        else if (o == saveItem) {
            mainGUI.saveConfiguration();
        }
        else if (o == importItem) {
            mainGUI.importModel();
        }
        else if (o == exportItem) {
            mainGUI.exportConfiguration();
        }
        else if (o == newComponentItem) {
            mainGUI.addComponent();
        }
        else if (o == newFeatureItem) {
            mainGUI.addFeature();
        }
        else if (o == findConfigurationItem) {
            mainGUI.findCompleteConfiguration();
        }
        else if (o == confPreferencesItem) {
            mainGUI.editConfigurationPreferences();
        }
        else if (o == viewNameItem) {
            mainGUI.showNames(viewNameItem.isSelected());
        }
        else if (o == viewTypeItem) {
            mainGUI.showTypes(viewTypeItem.isSelected());
        }
        else if (o == viewConnectionItem) {
            mainGUI.showConnections(viewConnectionItem.isSelected());
        }
        else if (o == viewPotentialItem) {
            mainGUI.showPotentialElements(viewPotentialItem.isSelected());
        }
        else if (o == largeTreeIcons) {
            mainGUI.setLargeTreeIcons(largeTreeIcons.isSelected());
        }
    }
    

}
