/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * ConfigurationInfoPanel.java
 *
 * Created on 9. syyskuuta 2004, 9:20
 */

package kumbang.configurator.client.gui.core;

import kumbang.configurator.client.core.ClientServices;

import kumbang.configurator.client.gui.event.ConfigurationModificationListener;
import kumbang.configurator.client.gui.event.ConfigurationModifiedEvent;

import kumbang.util.gui.Icons;
import kumbang.util.gui.Margins;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 *
 * @author  vmyllarn
 */
public class ConfigurationInfoPanel extends JPanel implements ActionListener, ConfigurationModificationListener {
    
    private MainGUI mainGUI;
    
    private ClientServices client;
    
    private JLabel confName;
    
    private JLabel modelName;
    
    private JLabel state;
    
    private JLabel stateIcon;
    
    private JLabel infoIcon;
    
    private JPanel propertyPanel;
    private JPanel statePanel;
    private JPanel textPanel;
    
    private JButton exportButton;
    
    private JButton saveButton;
    
    /** Creates a new instance of ConfigurationInfoPanel */
    public ConfigurationInfoPanel(MainGUI mainGUI, ClientServices client) {
        this.mainGUI = mainGUI;
        this.client = client;
        initComponents();
        updateInformation();
        mainGUI.addConfigurationModificationListener(this);
    }
    
    
    public void init() {
        updateInformation();
    }
    
    public void reset() {
        updateInformation();
    }
    
    private void initComponents() {
        setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

        // create panel for configuration properties (name etc)
        propertyPanel = createPropertyPanel();
        propertyPanel.setAlignmentY(Component.TOP_ALIGNMENT);
        propertyPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        propertyPanel.setMaximumSize(propertyPanel.getPreferredSize());
        
        // create panel for configuration state
        statePanel = createStatePanel();
        statePanel.setAlignmentY(Component.TOP_ALIGNMENT);
        statePanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        statePanel.setMaximumSize(statePanel.getPreferredSize());
        
        add(Margins.createVerticalMarginM());
        add(propertyPanel);
        add(Margins.createVerticalMarginL());
        add(statePanel);
        add(Box.createVerticalGlue());
    }
    
    private JPanel createPropertyPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.LINE_AXIS));
        
        panel.add(Margins.createHorizontalMarginM());
        
        infoIcon = new JLabel();
        infoIcon.setIcon(Icons.INFO_L);
        infoIcon.setAlignmentY(Component.TOP_ALIGNMENT);
        infoIcon.setAlignmentX(Component.LEFT_ALIGNMENT);
        infoIcon.setMaximumSize(infoIcon.getPreferredSize());
        panel.add(infoIcon);
        
        panel.add(Margins.createHorizontalMarginM());
        
        textPanel = createPropertyTextPanel();
        textPanel.setAlignmentY(Component.TOP_ALIGNMENT);
        textPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        //textPanel.setMaximumSize(textPanel.getPreferredSize());
        panel.add(textPanel);
        
        //panel.add(Box.createGlue());
        
        return panel;
    }
    
    private JPanel createPropertyTextPanel() {
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        
        JPanel panel = new JPanel(gridbag);
        
        c.anchor = GridBagConstraints.FIRST_LINE_START;
               
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 2;
        c.insets = new Insets(0, 0, Margins.M, 0);
        JLabel tempLabel = new JLabel("Properties");
        panel.add(tempLabel, c);
        
        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth = 1;
        c.insets = new Insets(0, 0, Margins.S, Margins.L);
        tempLabel = new JLabel("Configuration name");
        panel.add(tempLabel, c);
              
        c.gridx = 0;
        c.gridy = 2;
        tempLabel = new JLabel("Model name");
        panel.add(tempLabel, c);
        
        c.gridx = 1;
        c.gridy = 1;
        confName = new JLabel("");
        panel.add(confName, c);
        
        c.gridx = 1;
        c.gridy = 2;
        modelName = new JLabel("");
        panel.add(modelName, c);
        
        return panel;
    }
    
    private JPanel createStatePanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.LINE_AXIS));
        
        panel.add(Margins.createHorizontalMarginM());
        
        // state icon
        stateIcon = new JLabel();
        stateIcon.setIcon(Icons.STATUS_INVALID_L);
        stateIcon.setAlignmentY(Component.TOP_ALIGNMENT);
        stateIcon.setAlignmentX(Component.LEFT_ALIGNMENT);
        stateIcon.setMaximumSize(stateIcon.getPreferredSize());
        panel.add(stateIcon);
        
        panel.add(Margins.createHorizontalMarginM());
        
        // state text and stuff
        textPanel = createStateTextPanel();
        textPanel.setAlignmentY(Component.TOP_ALIGNMENT);
        textPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        //textPanel.setMaximumSize(textPanel.getPreferredSize());
        panel.add(textPanel);
        
        return panel;
    }
    
    private JPanel createStateTextPanel() {
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        
        JPanel panel = new JPanel(gridbag);
        
        c.anchor = GridBagConstraints.FIRST_LINE_START;
               
        // state label
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 2;
        c.insets = new Insets(0, 0, Margins.M, 0);
        state = new JLabel("Configuration is incomplete.");
        panel.add(state, c);
        
        
        // export button
        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth = 1;
        c.insets = new Insets(0, 0, Margins.S, Margins.M);
        exportButton = new JButton(Icons.EXPORT);
        exportButton.addActionListener(this);
        exportButton.setEnabled(false);
        exportButton.setMargin(new Insets(0,0,0,0));
        exportButton.setToolTipText("Export configuration description");
        panel.add(exportButton, c);
              
        // save button
        c.gridx = 0;
        c.gridy = 2;
        saveButton = new JButton(Icons.SAVE);
        saveButton.addActionListener(this);
        saveButton.setMargin(new Insets(0,0,0,0));
        saveButton.setToolTipText("Save configuration");
        panel.add(saveButton, c);
        
        c.gridx = 1;
        c.gridy = 1;
        JLabel tempLabel = new JLabel("Export");
        panel.add(tempLabel, c);
        
        c.gridx = 1;
        c.gridy = 2;
        tempLabel = new JLabel("Save");
        panel.add(tempLabel, c);
        
        return panel;
    }
    
    private void updateInformation() {
        if (client.isConfigurationStateReseted()) {
            confName.setText("");
            modelName.setText("");
            //state.setText("No configuration");
            //stateIcon.setIcon(Icons.STATUS_INVALID);
            statePanel.setVisible(false);
            return;
        }
        confName.setText(client.getConfigurationName());
        modelName.setText(client.getModelName());
        if (client.isConfigurationStateComplete()) {
            state.setText("Configuration is complete.");
            stateIcon.setIcon(Icons.STATUS_COMPLETE_L);
            exportButton.setEnabled(true);
        }
        else {
            state.setText("Configuration is incomplete.");
            stateIcon.setIcon(Icons.STATUS_INCOMPLETE_L);
            exportButton.setEnabled(false);
        }
        statePanel.setVisible(true);
        
        //textPanel.setPreferredSize(textPanel.getMinimumSize());
        //propertyPanel.setPreferredSize(propertyPanel.getMinimumSize());
        textPanel.setMaximumSize(textPanel.getPreferredSize());
        propertyPanel.setMaximumSize(propertyPanel.getPreferredSize());
    }

    
    public void configurationModified(ConfigurationModifiedEvent e) {
        updateInformation();
    }
    
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == exportButton) {
            mainGUI.exportConfiguration();
        }
        else if (e.getSource() == saveButton) {
            mainGUI.saveConfiguration();
        }
    }
    

}
