/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * DiagramInfoPanel.java
 *
 * Created on 7. syyskuuta 2004, 16:23
 */

package kumbang.configurator.client.gui.core;

import kumbang.core.configuration.instance.ComponentInstance;
import kumbang.core.configuration.instance.Instance;
import kumbang.core.configuration.instance.InterfaceInstance;

import kumbang.configurator.client.gui.event.ConfigurationModificationListener;
import kumbang.configurator.client.gui.event.ConfigurationModifiedEvent;
import kumbang.configurator.client.gui.event.ComponentSelectionListener;
import kumbang.configurator.client.gui.event.ComponentSelectedEvent;

import java.awt.CardLayout;

import javax.swing.JPanel;
import javax.swing.JScrollPane;


/**
 *
 * @author  vmyllarn
 */
public class DiagramInfoPanel extends JPanel 
            implements ConfigurationModificationListener, ComponentSelectionListener {
    
    private static final String EMPTY = "";
    private static final String COMPONENT = "Component info";
    //private static final String FEATURE = "Feature info";
    private static final String INTERFACE = "Interface info";
    
    
    /** Reference to main GUI */
    private MainGUI mainGUI;
    
    /** Card layout holding different panels */
    private CardLayout cardLayout;
    
    /** Panel for component info */
    private ComponentInfoPanel componentInfo;
    
    /** Panel for feature info */
    //private FeatureInfoPanel featureInfo;
    
    /** Panel for interface info */
    private InterfaceInfoPanel interfaceInfo;
    
    /** Empty panel for showing no information */
    private JPanel emptyInfo;
    
    /** Current component instance that is showing or null */
    private ComponentInstance compInstance;
    
    /** Current feature instance that is showing, or null */
    //private FeatureInstance featInstance;
    
    /** Current interface instance that is showing, or null */
    private InterfaceInstance intfInstance;
 

    //---------- PUBLIC METHODS -----------------------------------------------
    
    /** Creates a new instance of DiagramInfoPanel */
    public DiagramInfoPanel(MainGUI mainGUI) {
        this.mainGUI = mainGUI;
        mainGUI.addConfigurationModificationListener(this);
        mainGUI.addComponentSelectionListener(this);
        initComponents();
    }
    
    public void init() {
        componentInfo.reset();
        interfaceInfo.reset();
    }
    
    public void reset() {
        componentInfo.reset();
        interfaceInfo.reset();
    }
    
    public void componentSelected(ComponentSelectedEvent e) {
        Instance instance = e.getSelectedInstance();
        if (instance == compInstance) {
            cardLayout.show(this, COMPONENT);
            return;
        }
        else if (instance == intfInstance) {
            cardLayout.show(this, INTERFACE);
            return;
        }
        if (instance instanceof ComponentInstance) {
            componentInfo.reset();
            compInstance = (ComponentInstance)instance;
            componentInfo.selectInstance(compInstance);
            cardLayout.show(this, COMPONENT);
        }
        else if (instance instanceof InterfaceInstance) {
            interfaceInfo.reset();
            intfInstance = (InterfaceInstance)instance;
            interfaceInfo.selectInstance(intfInstance);
            cardLayout.show(this, INTERFACE);
        }
        else {
            cardLayout.show(this, EMPTY);
        }
    }
    
    public void configurationModified(ConfigurationModifiedEvent e) {
        componentInfo.updateInformation();
        interfaceInfo.updateInformation();
    }
    
    private void initComponents() {
        this.cardLayout = new CardLayout();
        setLayout(cardLayout);
        
        this.emptyInfo = new JPanel();
        this.componentInfo = new ComponentInfoPanel(mainGUI);
        //this.featureInfo = new FeatureInfoPanel(mainGUI);
        this.interfaceInfo = new InterfaceInfoPanel(mainGUI);
        add(emptyInfo, EMPTY);
        add(new JScrollPane(componentInfo), COMPONENT);
        //add(new JScrollPane(featureInfo), FEATURE);
        add(new JScrollPane(interfaceInfo), INTERFACE);
     
    }
}
