/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * FeaturePanel.java
 *
 * Created on 28. tammikuuta 2005, 11:14
 */

package kumbang.configurator.client.gui.core;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.Box;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;

import kumbang.core.configuration.instance.AttributeInstance;
import kumbang.core.configuration.instance.FeatureInstance;
import kumbang.core.configuration.instance.PartInstance;
import kumbang.util.gui.Icons;
import kumbang.util.gui.Margins;

/**
 * 
 * @author vmyllarn
 */
public class FeaturePanel extends JScrollPane implements ActionListener {

	private static final Dimension LIST_SIZE = new Dimension(200, 200);

	private FeatureInstance instance;

	private MainGUI mainGUI;

	private FeatureInfoPanel parent;

//	private ConsequenceHandler cnsqHandler;

	// private List attributeCombos;
	private List<AttributeValueLabel> attributeLabels;

	/** button for navigating into parent feature */
	private JButton gotoParent;

	/** button for editing attributes */
	private JButton editAttributes;

	/** button for adding subfeature under given role */
	private JButton addSubfeature;

	/** button for removing subfeatures */
	private JButton removeSubfeature;

	/** button for navigating into given subfeature */
	private JButton gotoSubfeature;

	private JList roleList;

	private JList subfeatureList;

	private DefaultListModel roleListModel;

	private DefaultListModel subfeatureListModel;

	private List<JComponent> focusable;

	/** Creates a new instance of FeaturePanel */
	public FeaturePanel(FeatureInfoPanel parent, MainGUI mainGUI, FeatureInstance instance) {
		this.mainGUI = mainGUI;
		this.parent = parent;
//		this.cnsqHandler = mainGUI.getConsequenceHandler();
		this.instance = instance;
		initComponents();
		initFocusable();
	}

	public void update() {
		// update attribute value
		if (attributeLabels != null) {
			for (AttributeValueLabel avl : attributeLabels) {
				avl.update();
			}
		}

		// update roles
		if (roleListModel != null) {
			roleListModel.clear();
			Iterator it = instance.getParts();
			while (it.hasNext()) {
				roleListModel.addElement(it.next());
			}
		}

		// update subfeatures
		if (subfeatureListModel != null) {
			subfeatureListModel.clear();
			Iterator it = instance.getPartInstances();
			while (it.hasNext()) {
				subfeatureListModel.addElement(it.next());
			}
		}

		repaint();
	}

	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if (source == editAttributes) {
			mainGUI.editAttributes(instance);
		}
		/* add subfeatures */
		else if (source == addSubfeature) {
			PartInstance part = (PartInstance) roleList.getSelectedValue();
			if (part != null) {
				mainGUI.addComposable(part);
			}
		}
		/* remove subfeatures */
		else if (source == removeSubfeature) {
			FeatureInstance f = (FeatureInstance) subfeatureList.getSelectedValue();
			if (f != null) {
				mainGUI.removeComposable(f);
			}
		}
		/* goto subfeature */
		else if (source == gotoSubfeature) {
			FeatureInstance f = (FeatureInstance) subfeatureList.getSelectedValue();
			if (f != null) {
				parent.showPanelForFeature(f);
			}
		}
		/* goto parent feature */
		else if (source == gotoParent) {
			FeatureInstance parentFeature = (FeatureInstance) instance.getParent();
			parent.showPanelForFeature(parentFeature);
		}
	}

	public Component getFirstFocusableComponent() {
		return (Component) focusable.get(0);
	}

	public Component getLastFocusableComponent() {
		return (Component) focusable.get(focusable.size() - 1);
	}

	public Component getNextFocusableComponent(Component aComponent) {
		int index = focusable.indexOf(aComponent);
		if (index < 0 || index == focusable.size() - 1) {
			return null;
		} else {
			return (Component) focusable.get(index + 1);
		}
	}

	public Component getPreviousFocusableComponent(Component aComponent) {
		int index = focusable.indexOf(aComponent);
		if (index < 1) {
			return null;
		} else {
			return (Component) focusable.get(index - 1);
		}
	}

	private void initFocusable() {
		focusable = new ArrayList<JComponent>();
		if (gotoParent != null && gotoParent.isEnabled()) {
			focusable.add(gotoParent);
		}
		if (editAttributes != null) {
			focusable.add(editAttributes);
		}
		if (roleList != null) {
			focusable.add(roleList);
		}
		if (addSubfeature != null) {
			focusable.add(addSubfeature);
		}
		if (subfeatureList != null) {
			focusable.add(subfeatureList);
		}
		if (gotoSubfeature != null) {
			focusable.add(gotoSubfeature);
		}
		if (removeSubfeature != null) {
			focusable.add(removeSubfeature);
		}
	}

	private void initComponents() {
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		JPanel gridPanel = new JPanel(gridbag);

		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		c.insets = new Insets(0, 0, Margins.XL, Margins.M);

		c.gridwidth = 2;
		JComponent comp = createHeaderLabel();
		gridPanel.add(comp, c);

		c.gridy++;
		c.gridwidth = 1;
		comp = createPropertyLabel();
		gridPanel.add(comp, c);

		c.gridx++;
		comp = initPropertyArea();
		gridPanel.add(comp, c);

		if (instance.hasAttributes()) {
			c.gridx = 0;
			c.gridy++;
			comp = createAttributeLabel();
			gridPanel.add(comp, c);

			c.gridx++;
			comp = initAttributeArea();
			gridPanel.add(comp, c);
		}

		if (instance.hasParts()) {
			c.gridx = 0;
			c.gridy++;
			comp = createRoleLabel();
			gridPanel.add(comp, c);

			c.gridx++;
			comp = initRoleArea();
			gridPanel.add(comp, c);

			c.gridx = 0;
			c.gridy++;
			comp = createSubfeatureLabel();
			gridPanel.add(comp, c);

			c.gridx++;
			comp = initSubfeatureArea();
			gridPanel.add(comp, c);
		}

		gridPanel.setMaximumSize(gridPanel.getPreferredSize());

		Box vBox = Box.createVerticalBox();
		vBox.add(Margins.createVerticalMarginXL());
		vBox.add(gridPanel);
		vBox.add(Box.createVerticalGlue());

		Box hBox = Box.createHorizontalBox();
		hBox.add(Margins.createHorizontalMarginXL());
		hBox.add(vBox);
		hBox.add(Box.createHorizontalGlue());

		setViewportView(hBox);
	}

	private JLabel createHeaderLabel() {
		JLabel label = new JLabel(instance.getDisplayName() + ":"
				+ instance.getType().getDisplayName(), Icons.FEATURE_L, SwingConstants.LEADING);
		label.setFont(label.getFont().deriveFont(Font.BOLD));
		return label;
	}

	private JLabel createPropertyLabel() {
		JLabel label = new JLabel("Feature properties", Icons.INFO_L, SwingConstants.LEADING);
		return label;
	}

	private JLabel createAttributeLabel() {
		JLabel label = new JLabel("Attributes", Icons.ATTRIBUTE_L, SwingConstants.LEADING);
		return label;
	}

	private JLabel createRoleLabel() {
		JLabel label = new JLabel("Subfeature roles", Icons.SUBFEATURE_ROLE_L,
				SwingConstants.LEADING);
		return label;
	}

	private JLabel createSubfeatureLabel() {
		JLabel label = new JLabel("Subfeatures", Icons.FEATURE_L, SwingConstants.LEADING);
		return label;
	}

	private JComponent initPropertyArea() {
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();

		JPanel propertyPanel = new JPanel(gridbag);
		// propertyPanel.setBackground(Color.WHITE);
		// propertyPanel.setBorder(BorderFactory.createEtchedBorder());
		// propertyPanel.setMinimumSize(MIN_SIZE);

		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		c.insets = new Insets(0, 0, Margins.S, Margins.M);

		JLabel tempLabel = new JLabel("Feature type ");
		// tempLabel.setFont(tempLabel.getFont().deriveFont(Font.ITALIC));
		propertyPanel.add(tempLabel, c);

		c.gridy++;
		tempLabel = new JLabel("Feature name ");
		// tempLabel.setFont(tempLabel.getFont().deriveFont(Font.ITALIC));
		propertyPanel.add(tempLabel, c);

		c.gridy++;
		tempLabel = new JLabel("Parent type ");
		// tempLabel.setFont(tempLabel.getFont().deriveFont(Font.ITALIC));
		propertyPanel.add(tempLabel, c);

		c.gridy++;
		tempLabel = new JLabel("Parent name ");
		// tempLabel.setFont(tempLabel.getFont().deriveFont(Font.ITALIC));
		propertyPanel.add(tempLabel, c);

		c.gridy = 0;
		c.gridx++;
		c.insets = new Insets(0, 0, Margins.S, 0);
		tempLabel = new JLabel(instance.getType().getDisplayName());
		propertyPanel.add(tempLabel, c);

		c.gridy++;
		tempLabel = new JLabel(instance.getDisplayName());
		propertyPanel.add(tempLabel, c);

		c.gridy++;
		if (instance.getParent() != null) {
			tempLabel = new JLabel(instance.getParent().getType().getDisplayName());
		} else {
			tempLabel = new JLabel("");
		}
		propertyPanel.add(tempLabel, c);

		c.gridy++;
		if (instance.getParent() != null) {
			tempLabel = new JLabel(instance.getParent().getDisplayName());
		} else {
			tempLabel = new JLabel("");
		}
		propertyPanel.add(tempLabel, c);

		c.gridx = 0;
		c.gridy++;
		gotoParent = new JButton(Icons.UP);
		gotoParent.setToolTipText("View parent feature information");
		gotoParent.setMargin(new Insets(0, 0, 0, 0));
		if (instance.getParent() != null) {
			gotoParent.addActionListener(this);
		} else {
			gotoParent.setEnabled(false);
		}
		propertyPanel.add(gotoParent, c);

		return propertyPanel;

	}

	private JComponent initAttributeArea() {
		if (!instance.hasAttributes()) {
			return null;
		}

		this.attributeLabels = new ArrayList<AttributeValueLabel>();
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();

		JPanel valuePanel = new JPanel(gridbag);

		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		c.insets = new Insets(0, 0, Margins.S, Margins.M);

		int numberOfAttributes = 0;

		Iterator it = instance.getAttributes();
		while (it.hasNext()) {
			numberOfAttributes++;
			AttributeInstance attr = (AttributeInstance) it.next();
			AttributeValueLabel label = new AttributeValueLabel(attr);
			attributeLabels.add(label);
			// put name and attribute value label into grid
			JLabel nameLabel = new JLabel(attr.getName() + " ");
			// nameLabel.setFont(nameLabel.getFont().deriveFont(Font.ITALIC));
			c.gridx = 0;
			valuePanel.add(nameLabel, c);
			c.gridx++;
			valuePanel.add(label, c);
			c.gridy++;
		}

		c.gridx = 0;
		editAttributes = new JButton(Icons.EDIT);
		editAttributes.addActionListener(this);
		editAttributes.setMargin(new Insets(0, 0, 0, 0));
		editAttributes.setToolTipText("Edit attribute values");
		valuePanel.add(editAttributes, c);

		return valuePanel;
	}

	private JComponent initRoleArea() {
		if (!instance.hasParts()) {
			return null;
		}

		// initialise list for roles
		roleListModel = new DefaultListModel();
		Iterator it = instance.getParts();
		while (it.hasNext()) {
			roleListModel.addElement(it.next());
		}

		roleList = new JList(roleListModel);
		roleList.setMinimumSize(LIST_SIZE);
		roleList.setPreferredSize(LIST_SIZE);
		roleList.setMaximumSize(LIST_SIZE);
		if (!roleListModel.isEmpty()) {
			roleList.setSelectedIndex(0);
		}

		// initilise add button
		addSubfeature = new JButton(Icons.NEW_FEATURE);
		addSubfeature.addActionListener(this);
		addSubfeature.setMargin(new Insets(0, 0, 0, 0));
		addSubfeature.setToolTipText("Add new subfeature to selected role");
		addSubfeature.setAlignmentY(Component.TOP_ALIGNMENT);

		// lay role buttons out in a box vertically (top to bottom)
		Box roleButtonBox = Box.createVerticalBox();
		roleButtonBox.add(addSubfeature);
		roleButtonBox.add(Box.createVerticalGlue());

		Box box = Box.createHorizontalBox();
		box.add(roleList);
		box.add(Margins.createHorizontalMarginXS());
		box.add(roleButtonBox);

		return box;
	}

	private JComponent initSubfeatureArea() {
		if (!instance.hasParts()) {
			return null;
		}
		// initialise list for subfeatures
		subfeatureListModel = new DefaultListModel();
		Iterator it = instance.getPartInstances();
		while (it.hasNext()) {
			subfeatureListModel.addElement(it.next());
		}

		subfeatureList = new JList(subfeatureListModel);
		subfeatureList.setMinimumSize(LIST_SIZE);
		subfeatureList.setPreferredSize(LIST_SIZE);
		subfeatureList.setMaximumSize(LIST_SIZE);
		if (!subfeatureListModel.isEmpty()) {
			subfeatureList.setSelectedIndex(0);
		}

		// initialise navigate button
		gotoSubfeature = new JButton(Icons.DOWN);
		gotoSubfeature.addActionListener(this);
		gotoSubfeature.setMargin(new Insets(0, 0, 0, 0));
		gotoSubfeature.setToolTipText("View selected subfeature information");

		// initialise remove button
		removeSubfeature = new JButton(Icons.DELETE);
		removeSubfeature.addActionListener(this);
		removeSubfeature.setMargin(new Insets(0, 0, 0, 0));
		removeSubfeature.setToolTipText("Remove selected subfeature");

		// lay subfeature buttons out in a box vertically (top to bottom)
		Box subfButtonBox = Box.createVerticalBox();
		subfButtonBox.add(gotoSubfeature);
		subfButtonBox.add(Margins.createVerticalMarginXS());
		subfButtonBox.add(removeSubfeature);
		subfButtonBox.add(Box.createVerticalGlue());

		Box box = Box.createHorizontalBox();
		box.add(subfeatureList);
		box.add(Margins.createHorizontalMarginXS());
		box.add(subfButtonBox);

		return box;
	}

	class AttributeValueLabel extends JLabel {
		private AttributeInstance attribute;

		private int prefWidth = 0;

		public AttributeValueLabel(AttributeInstance attribute) {
			super("= ");
			this.attribute = attribute;
			prefWidth = new JLabel("= ").getPreferredSize().width;
			Iterator iter = attribute.getPossibleValues();
			while (iter.hasNext()) {
				JLabel label = new JLabel("= " + iter.next().toString());
				if (label.getPreferredSize().width > prefWidth) {
					prefWidth = label.getPreferredSize().width;
				}
			}
			setPreferredSize(new Dimension(prefWidth, getPreferredSize().height));
			update();
		}

		public void setPreferredSize(Dimension size) {
			super.setPreferredSize(new Dimension(prefWidth, size.height));
		}

		public void update() {
			if (attribute.getValue() != null) {
				setText("= " + attribute.getDisplayValue());
			} else {
				setText("= ");
			}
		}

		public String toString() {
			return "AttributeValueLabel[" + super.getText() + "]";
		}
	}

}
