/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * BoxedElement.java
 *
 * Created on 26. helmikuuta 2004, 15:39
 */

package kumbang.configurator.client.gui.diagram;

import kumbang.util.gui.Colors;

import java.awt.Color;
import java.awt.Graphics;

/**
 *
 * @author  vmyllarn
 */
public abstract class BoxedElement extends DiagramElement {
    
    /** Upper left Y coordinate of this diagram element. */    
    private int upperLeftX;

    /** Upper left Y coordinate of this diagram element. */
    private int upperLeftY;

    /** Width of this diagram element. */
    private int width;

    /** Height of this diagram element. */
    private int height;
    
    /** Background color of this diagram element */
    private Color backgroundColor;
    
    private boolean opaque;
    
    /** Creates a new instance of BoxedElement */
    public BoxedElement(String name, int width, int height) {
        this(name, width, height, 
            Colors.EMPTY, Colors.FOREGROUND);
    }
    
    /** Creates a new instance of BoxedElement */
    public BoxedElement(String name, int width, int height, Color backgroundColor) {
        this(name, width, height, 
            backgroundColor, Colors.FOREGROUND);
    }
         
    /** Creates a new instance of BoxedElement */
    public BoxedElement(String name, int width, int height, 
            Color backgroundColor, Color foregroundColor) {
        this(name, width, height, foregroundColor, backgroundColor != null);
        this.backgroundColor = backgroundColor;
    }
    
    /** Creates a new instance of BoxedElement */
    public BoxedElement(String name, int width, int height, Color foregroundColor, boolean opaque) {
        super(name, foregroundColor);
        this.width = width;
        this.height = height;
        this.opaque = opaque;
    }
    
    /** Moves this element to a new location in coordinates */
    protected void setLocation(int centerX, int centerY) {
        this.upperLeftX = centerX - width/2;
        this.upperLeftY = centerY - height/2;        
    }
    
    protected void addToLocation(int deltaX, int deltaY) {
        // implemented by calling setLocation(), so that subclasses
        // that have overriden that method can use this directly!
        setLocation(getLocationX()+deltaX, getLocationY()+deltaY);
    }
    
    public int getLocationX() {
        return this.upperLeftX + width/2;
    }
    
    public int getLocationY() {
        return this.upperLeftY + height/2;
    }  
    
    public int getWidth() {
        return width;
    }
    
    public int getHeight() {
        return height;
    }
    
    protected void setWidth(int width) {
        this.width = width;
    }
    
    protected void setHeight(int height) {
        this.height = height;
    }
     
    protected abstract void removeFromDiagram();
    
    public void paint(Graphics g) {
        if (opaque) {
            g.translate(upperLeftX, upperLeftY);
            g.setColor(backgroundColor);
            g.fillRect(0, 0, width, height);
        }
        if (isInAction()) {
            g.setColor(getActionColor());
        }
        else if (isSelected()) {
            g.setColor(getSelectionColor());
        }
        else {
            g.setColor(getForegroundColor());
        }
        if (opaque) {
            g.drawRect(0, 0, width, height);
            g.translate(-upperLeftX, -upperLeftY);
        }
    }
    
    public boolean isInside(int xCoordinate, int yCoordinate) {
        return 
            xCoordinate >= upperLeftX && 
            xCoordinate <= upperLeftX + width &&
            yCoordinate >= upperLeftY &&
            yCoordinate <= upperLeftY + height;
    }  
    
}
