/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * DiagramElement.java
 *
 * Created on 25. helmikuuta 2004, 13:13
 */

package kumbang.configurator.client.gui.diagram;

import kumbang.util.gui.Colors;

import java.awt.Graphics;
import java.awt.Color;

/**
 *
 * @author  vmyllarn
 */
public abstract class DiagramElement {
    
    /** Diagram level that this element belongs to */
    private DiagramLevel level;
    
    /** Name of this diagram element */
    private String name;

    /** Tells whether this diagram element is selected or not. */
    private boolean isSelected;
    
    /** Tells whether this diagram element is action (moved etc) or not. */
    private boolean isInAction;    
      
    /** Tells whether this diagram element is visible or not. */
    private boolean isVisible;
    
    /** Foreground color of this diagram element */
    private Color foregroundColor = Colors.FOREGROUND;
    
    /** Selection color of this diagram element */
    private Color selectionColor = Colors.SELECTION;
    
    /** Action color of this diagram element */
    private Color actionColor = Colors.ACTION;    
        
    /** Creates a new instance of DiagramElement */
    public DiagramElement(String name) {
        this(name, Colors.FOREGROUND);
    }

    public DiagramElement(String name, Color foregroundColor) {
        this.name = name;
        this.isSelected = false;
        this.isInAction = false;
        this.foregroundColor = foregroundColor;
        this.isVisible = true;
    }
    
    public abstract void paint(Graphics g);
    
    public abstract boolean isInside(int xCoordinate, int yCoordinate);
    
    protected abstract void removeFromDiagram();
    
    public DiagramLevel getLevel() {
        return this.level;
    }
    
    protected void setLevel(DiagramLevel level) {
        this.level = level;
    }
    
    public Color getForegroundColor() {
        return this.foregroundColor;
    }
    
    public Color getSelectionColor() {
        return this.selectionColor;
    }
    
    public Color getActionColor() {
        return this.actionColor;
    }
    
    public void setSelected(boolean selected) {
        this.isSelected = selected;
    }
    
    public boolean isSelected() {
        return this.isSelected;
    }
    
    public void setInAction(boolean inAction) {
        this.isInAction = inAction;
    }
    
    public boolean isInAction() {
        return this.isInAction;
    }
    
    public boolean isVisible() {
        return this.isVisible;
    }
    
    public void setVisible(boolean visible) {
        this.isVisible = visible;
    }
    
    public String getName() {
        return this.name;
    }
    
    public String toString() {
        return name;
    }
    
    public abstract boolean equals(Object o);
}
