/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * AddComposableDialog.java
 *
 * Created on 11. maaliskuuta 2004, 15:23
 */

package kumbang.configurator.client.gui.dialog;

import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import kumbang.configurator.client.core.TaskHandler;
import kumbang.configurator.client.gui.core.MainGUI;
import kumbang.core.configuration.instance.PartInstance;
import kumbang.core.configuration.task.AddComposableTask;
import kumbang.core.model.type.AttributeDefinition;
import kumbang.core.model.type.ComponentType;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.InterfaceDefinition;
import kumbang.core.model.type.InterfaceType;
import kumbang.core.model.type.Name;
import kumbang.util.gui.Margins;

/**
 * 
 * @author vmyllarn
 */
public class AddComposableDialog extends TemplateDialog implements ActionListener {

	public static final int MIN_AREA_WIDTH = 300;
	public static final int MIN_AREA_HEIGHT = 150;

	private PartInstance partInstance;

//	private ConsequenceHandler cnsqHandler;

	private JButton ok;
	private JButton cancel;

	private List possibleTypes;

	private JComboBox typeCombo;

	private JPanel interfacePanel;

	private JPanel attributePanel;

	private boolean hasAttributes = false;

	private boolean hasInterfaces = false;

	// -------------- METHODS TO INITIALIZE DIALOG GUI --------------------------

	/**
	 * Creates a new instance of AddComposableDialog.
	 * 
	 * @param mainGUI mainGUI
	 * @param instance potential instance that is used as a basis for adding
	 */
	public AddComposableDialog(MainGUI mainGUI, PartInstance partInstance) {
		super(mainGUI, "Add a new " + partInstance.getComposedKind());
		this.partInstance = partInstance;
//		this.cnsqHandler = mainGUI.getConsequenceHandler();
		this.possibleTypes = new LinkedList();
		Iterator iter = partInstance.getPossibleTypes();
		while (iter.hasNext()) {
			PossibleTypeItem item = new PossibleTypeItem((ComposableType) iter.next());
			if (!hasAttributes && item.getNumberOfAttributes() > 0) {
				this.hasAttributes = true;
			}
			if (!hasInterfaces && item.getNumberOfInterfaces() > 0) {
				this.hasInterfaces = true;
			}
			this.possibleTypes.add(item);
		}
		initComponents();
	}

	// -------------- METHODS TO INITIALIZE DIALOG GUI --------------------------

	/**
	 * Creates the components of this dialog. Overriden from TemplateDialog.
	 */
	protected void initComponents() {
		if (partInstance.isFull()) {
			/* create a box for textual information */
			Box textBox = Box.createVerticalBox();
			textBox.add(new JLabel("You have selected a part that has " + "already "));
			textBox
					.add(new JLabel("the maximum number of " + partInstance.getComposedKind()
							+ "s."));
			textBox.add(new JLabel("Thus you cannot add a new " + partInstance.getComposedKind()
					+ " to this part."));
			textBox.add(new JLabel("Please select another part."));
			super.addComponent(textBox);

			JPanel infoPanel = new JPanel(new GridLayout(3, 3));
			infoPanel.add(new JLabel("Part"));
			infoPanel.add(Margins.createHorizontalMarginS());
			infoPanel.add(new JLabel(partInstance.getName()));
			infoPanel.add(new JLabel("Parent " + partInstance.getComposedKind()));
			infoPanel.add(Margins.createHorizontalMarginS());
			infoPanel.add(new JLabel(partInstance.getParent().toString()));
			infoPanel.add(new JLabel("Instantiated " + partInstance.getComposedKind() + "s"));
			infoPanel.add(Margins.createHorizontalMarginS());
			infoPanel.add(new JLabel("" + partInstance.getNumberOfInstances()));
			super.addComponent(infoPanel);
		} else {
			/* create a box for textual information */
			Box textBox = Box.createVerticalBox();
			textBox.add(new JLabel("This adds a new " + partInstance.getComposedKind()
					+ " to the configuration."));
			textBox.add(new JLabel("If applicable, please select the type " + "of the "
					+ partInstance.getComposedKind() + ","));
			textBox.add(new JLabel("interfaces to be included, and initial "
					+ "values to attributes."));
			super.addComponent(textBox);

			/* create an area for name and type info */
			super.addComponent(createNameTypeArea());

			if (hasInterfaces) {
				/* create an area for setting interfaces on / off */
				super.addComponent(createInterfaceArea());
			}

			if (hasAttributes) {
				/* create an area for attribute values */
				super.addComponent(createAttributeArea());
			}
		}

		ok = new JButton("Add");
		ok.addActionListener(this);
		super.addButton(ok, true);
		cancel = new JButton("Cancel");
		cancel.addActionListener(this);
		super.addButton(cancel, false);

		if (partInstance.isFull()) {
			ok.setEnabled(false);
		}

		/* initialize all components */
		super.initComponents();
	}

	/**
	 * Creates and returns an area that contains information about name and type of the instance to
	 * be added. Includes a combobox that can be used to select the type of the component.
	 * 
	 * @return JComponent an area for properties
	 */
	private JComponent createNameTypeArea() {
		/* create a combobox for potential types */
		typeCombo = new JComboBox();
		for (Iterator it = possibleTypes.iterator(); it.hasNext();) {
			typeCombo.addItem(it.next());
		}
		typeCombo.addActionListener(this);

		/* create a panel for name and type information */
		JPanel panel = new JPanel(new GridLayout(4, 2));
		panel.add(new JLabel("Name "));
		panel.add(new JLabel(partInstance.getDisplayName()));
		panel.add(new JLabel("Type "));
		panel.add(typeCombo);
		panel.add(new JLabel("Parent name "));
		panel.add(new JLabel(partInstance.getParent().getDisplayName()));
		panel.add(new JLabel("Parent type "));
		panel.add(new JLabel(partInstance.getParent().getType().getDisplayName()));

		int areaWidth = MIN_AREA_WIDTH;
		int areaHeight = 0;
		if (panel.getPreferredSize().width > areaWidth) {
			areaWidth = panel.getPreferredSize().width;
		}
		if (panel.getPreferredSize().height > areaHeight) {
			areaHeight = panel.getPreferredSize().height;
		}
		panel.setPreferredSize(new Dimension(areaWidth, areaHeight));
		panel.setMaximumSize(new Dimension(areaWidth, areaHeight));
		panel.setMinimumSize(new Dimension(areaWidth, areaHeight));
		panel.setBorder(Margins.createEmptyBorderS());

		JScrollPane scroll = new JScrollPane(panel);

		/* create a box for label "properties" */
		Box labelArea = Box.createHorizontalBox();
		labelArea.add(new JLabel("Properties"));
		labelArea.add(Box.createHorizontalGlue());

		/* put it all together in a box */
		Box box = Box.createVerticalBox();
		box.add(labelArea);
		box.add(Margins.createVerticalMarginS());
		box.add(scroll);
		return box;
	}

	/**
	 * Creates and returns an area that contains information about interfaces to be added. Includes
	 * checkboxes for each interface.
	 * 
	 * @return JComponent an area for interfaces
	 */
	private JComponent createInterfaceArea() {
		interfacePanel = new JPanel(new CardLayout());
		interfacePanel.setBorder(Margins.createEmptyBorderS());
		for (Iterator it = possibleTypes.iterator(); it.hasNext();) {
			PossibleTypeItem item = (PossibleTypeItem) it.next();
			JComponent area = createInterfaceAreaForType(item);
			interfacePanel.add(area, item.toString());
		}

		Box labelArea = Box.createHorizontalBox();
		labelArea.add(new JLabel("Interfaces"));
		labelArea.add(Box.createHorizontalGlue());

		Box box = Box.createVerticalBox();
		box.add(labelArea);
		box.add(Margins.createVerticalMarginS());
		JScrollPane scroll = new JScrollPane(interfacePanel);
		box.add(scroll);
		box.add(Box.createVerticalGlue());

		box.setMinimumSize(new Dimension(MIN_AREA_WIDTH, MIN_AREA_HEIGHT));
		return box;
	}

	/**
	 * Creates and returns an area that contains information about attributes values of the
	 * instance. Includes combobox for each attribute.
	 * 
	 * @return JComponent an area for attributes
	 */
	private JComponent createAttributeArea() {
		attributePanel = new JPanel(new CardLayout());
		attributePanel.setBorder(Margins.createEmptyBorderS());
		for (Iterator it = possibleTypes.iterator(); it.hasNext();) {
			PossibleTypeItem item = (PossibleTypeItem) it.next();
			JComponent area = createAttributeAreaForType(item);
			attributePanel.add(area, item.toString());
		}

		Box labelArea = Box.createHorizontalBox();
		labelArea.add(new JLabel("Attributes"));
		labelArea.add(Box.createHorizontalGlue());

		Box box = Box.createVerticalBox();
		box.add(labelArea);
		box.add(Margins.createVerticalMarginS());
		JScrollPane scroll = new JScrollPane(attributePanel);
		box.add(Box.createVerticalGlue());
		box.add(scroll);

		box.setMinimumSize(new Dimension(MIN_AREA_WIDTH, MIN_AREA_HEIGHT));
		return box;
	}

	/**
	 * Creates and returns and area for one particular type that contains checkboxes for interfaces.
	 * 
	 * @param item item that corresponds to a certain composable type
	 * @return JComponent an area for interfaces for a certain type
	 */
	private JComponent createInterfaceAreaForType(PossibleTypeItem item) {
		Box box = Box.createVerticalBox();
		Iterator it = item.getInterfaceSelections();
		JPanel grid = new JPanel(new GridLayout(item.getNumberOfInterfaces(), 2));
		while (it.hasNext()) {
			InterfaceSelectionItem isi = (InterfaceSelectionItem) it.next();
			grid.add(isi.getNameComponent());
			grid.add(isi.getValueComponent());
		}
		grid.setMaximumSize(grid.getPreferredSize());
		box.add(grid);
		box.add(Box.createVerticalGlue());
		return box;
	}

	/**
	 * Creates and returns and area for one particular type that contains comboboxes for attributes.
	 * 
	 * @param item item that corresponds to a certain composable type
	 * @return JComponent an area for attributes for a certain type
	 */
	private JComponent createAttributeAreaForType(PossibleTypeItem item) {
		Box box = Box.createVerticalBox();
		Iterator it = item.getAttributeSelections();
		JPanel grid = new JPanel(new GridLayout(item.getNumberOfAttributes(), 2));
		while (it.hasNext()) {
			AttributeSelectionItem asi = (AttributeSelectionItem) it.next();
			grid.add(asi.getNameComponent());
			grid.add(asi.getValueComponent());
		}
		grid.setMaximumSize(grid.getPreferredSize());
		box.add(grid);
		box.add(Box.createVerticalGlue());
		return box;
	}

	// -------------- METHOD TO PARSE TASK FOR READY DIALOG ---------------------

	/**
	 * Creates a task that matches what the user has entered to the dialog. Checks that the dialog
	 * has been successfully dismissed.
	 * 
	 * @return AddComposableTask a task to create a new composable instance, or null if dialog was
	 *         not successfully finished
	 */
	public AddComposableTask getTask(TaskHandler taskHandler) {
		AddComposableTask task = null;

		/* check status */
		if (getStatus() != STATUS_OK) {
			return task;
		}

		/* fetch type and total instance to be created */
		PossibleTypeItem item = (PossibleTypeItem) typeCombo.getSelectedItem();
		ComposableType type = item.getType();
		task = taskHandler.createAddComposableTask(partInstance, type);

		if (type instanceof ComponentType) {
			Iterator it = item.getInterfaceSelections();
			while (it.hasNext()) {
				InterfaceSelectionItem isi = (InterfaceSelectionItem) it.next();
				if (isi.isSelected()) {
					taskHandler.addInterfaceToAddComposableTask(task, isi.getInterface(), isi
							.getInterfaceType());
				}
			}
		}

		/* create subtasks for setting attribute values */
		Iterator it = item.getAttributeSelections();
		while (it.hasNext()) {
			AttributeSelectionItem asi = (AttributeSelectionItem) it.next();
			AttributeDefinition attr = asi.getAttribute();
			
			// TODO The interfaces could be done like this as well, see above. -toasikai 25.7.2006
			task.setAttributeValue(attr.getName(), asi.getSelectedValue());
		}

		/* return task */
		return task;
	}

	// -------------- METHODS THAT REACT TO EVENTS IN THE DIALOG -----------------

	/**
	 * Listening actions from buttons (ok and cancel) and from type combobox. If type combobox has
	 * been changed, we must update the dialog to show corresponding interfaces and attributes
	 * 
	 * @param e action event
	 */
	public void actionPerformed(ActionEvent e) {
		Object o = e.getSource();
		if (o == ok) {
			setStatus(STATUS_OK);
			super.dispose();
		} else if (o == cancel) {
			super.dispose();
		} else if (o == typeCombo) {
			updateToType();
			repaint();
		}
	}

	/**
	 * Updates the dialog to reflect changing type. Shows interfaces and attributes for the
	 * corresponing type
	 */
	private void updateToType() {
		PossibleTypeItem item = (PossibleTypeItem) typeCombo.getSelectedItem();
		if (item == null) {
			return;
		}
		if (interfacePanel != null) {
			CardLayout cl = (CardLayout) interfacePanel.getLayout();
			cl.show(interfacePanel, item.toString());
		}
		if (attributePanel != null) {
			CardLayout cl = (CardLayout) attributePanel.getLayout();
			cl.show(attributePanel, item.toString());
		}
	}

	// ----------- CLASS FOR ITEMS THAT REPRESENT TYPE INFO -------------------------

	class PossibleTypeItem {
		private ComposableType type;
		private List interfaceSelections;
		private List attributeSelections;

		public PossibleTypeItem(ComposableType type) {
			this.type = type;
			this.interfaceSelections = new LinkedList();
			this.attributeSelections = new LinkedList();
			initInterfaceSelections();
			initAttributeSelections();
		}

		public String toString() {
			return type.getDisplayName();
		}

		public ComposableType getType() {
			return type;
		}

		public Iterator getInterfaceSelections() {
			return interfaceSelections.iterator();
		}

		public Iterator getAttributeSelections() {
			return attributeSelections.iterator();
		}

		public int getNumberOfAttributes() {
			return attributeSelections.size();
		}

		public int getNumberOfInterfaces() {
			return interfaceSelections.size();
		}

		private void initInterfaceSelections() {
			if (type instanceof ComponentType) {
				Iterator it = ((ComponentType) type).getInterfaces();
				while (it.hasNext()) {
					InterfaceDefinition intDef = (InterfaceDefinition) it.next();
					interfaceSelections.add(new InterfaceSelectionItem(intDef));
				}
			}
		}

		private void initAttributeSelections() {
			Iterator it = type.getDirectAttributes();
			while (it.hasNext()) {
				AttributeDefinition attrDef = (AttributeDefinition) it.next();
				attributeSelections.add(new AttributeSelectionItem(attrDef));
			}
		}
	}

	// -------------- ITEMS FOR SELECTING INTERFACES ---------------------------

	class InterfaceSelectionItem {

		private JCheckBox check;

		private JComboBox combo;

		/** interface definition that can be selected with this checkbox */
		private InterfaceDefinition interfaceDef;

		/**
		 * Creates a new InterfaceCheckBox based on the interface definition
		 * 
		 * @param interfaceDef interface definition
		 */
		public InterfaceSelectionItem(InterfaceDefinition interfaceDef) {
			this.interfaceDef = interfaceDef;
			this.check = new JCheckBox(interfaceDef.getDisplayName() + " ("
					+ (interfaceDef.isProvides() ? "provided" : "required") + ")");
			this.combo = new JComboBox();
			Iterator it = interfaceDef.getPossibleTypes();
			while (it.hasNext()) {
				this.combo.addItem(it.next());
			}
			this.check.setSelected(true);

			this.combo.setPreferredSize(new Dimension(MIN_AREA_WIDTH / 2, this.combo
					.getMinimumSize().height));
			this.combo.setMaximumSize(new Dimension(MIN_AREA_WIDTH / 2,
					this.combo.getMinimumSize().height));
			this.combo.setMinimumSize(new Dimension(MIN_AREA_WIDTH / 2,
					this.combo.getMinimumSize().height));
			this.check.setPreferredSize(new Dimension(MIN_AREA_WIDTH / 2, this.check
					.getMinimumSize().height));
			this.check.setMaximumSize(new Dimension(MIN_AREA_WIDTH / 2,
					this.check.getMinimumSize().height));
			this.check.setMinimumSize(new Dimension(MIN_AREA_WIDTH / 2,
					this.check.getMinimumSize().height));
		}

		/**
		 * Returns the interface definition
		 * 
		 * @return InterfaceDefinition interface definition
		 */
		public InterfaceDefinition getInterface() {
			return this.interfaceDef;
		}

		/**
		 * Returns the interface definition
		 * 
		 * @return InterfaceDefinition interface definition
		 */
		public InterfaceType getInterfaceType() {
			return (InterfaceType) this.combo.getSelectedItem();
		}

		/**
		 * Returns a component that represents the name of this item
		 * 
		 * @return JComponent name component
		 */
		public JComponent getNameComponent() {
			return this.check;
		}

		/**
		 * Returns a component that represents the value of this item
		 * 
		 * @return JComponent value component
		 */
		public JComponent getValueComponent() {
			return this.combo;
		}

		/**
		 * Tells whether this item has been selected (whether its checkbox is currently selected)
		 * 
		 * @return boolean
		 */
		public boolean isSelected() {
			return this.check.isSelected();
		}
	}

	// -------------- ITEMS FOR SETTING ATTRIBUTE VALUES -----------------------

	class AttributeSelectionItem {

		/** a combo box for selecting attribute value */
		private JComboBox combo;

		/** a label for showing attribute name */
		private JLabel label;

		/** attribute definition */
		private AttributeDefinition attributeDef;

		/** a map for mapping display values to actual values */
		private HashMap valueMap;

		/**
		 * Creates a new AttributeComboBox based on the given attribute definition
		 * 
		 * @param attributeDef attribute definition
		 */
		public AttributeSelectionItem(AttributeDefinition attributeDef) {
			this.attributeDef = attributeDef;
			this.combo = new JComboBox();
			this.label = new JLabel(attributeDef.getDisplayName());
			this.valueMap = new HashMap();
			Iterator it = attributeDef.getType().getValues();
			while (it.hasNext()) {
				Name value = (Name) it.next();
				valueMap.put(value.getDisplayName(), value.getName());
				combo.addItem(value.getDisplayName());
			}
			this.combo.setPreferredSize(new Dimension(MIN_AREA_WIDTH / 2, this.combo
					.getMinimumSize().height));
			this.combo.setMaximumSize(new Dimension(MIN_AREA_WIDTH / 2,
					this.combo.getMinimumSize().height));
			this.combo.setMinimumSize(new Dimension(MIN_AREA_WIDTH / 2,
					this.combo.getMinimumSize().height));
			this.label.setPreferredSize(new Dimension(MIN_AREA_WIDTH / 2, this.label
					.getMinimumSize().height));
			this.label.setMaximumSize(new Dimension(MIN_AREA_WIDTH / 2,
					this.label.getMinimumSize().height));
			this.label.setMinimumSize(new Dimension(MIN_AREA_WIDTH / 2,
					this.label.getMinimumSize().height));
		}

		/**
		 * Returns the attribute definition
		 * 
		 * @param AttributeDefinition
		 */
		public AttributeDefinition getAttribute() {
			return this.attributeDef;
		}

		/**
		 * Returns a component that represents the name of this item
		 * 
		 * @return JComponent name component
		 */
		public JComponent getNameComponent() {
			return this.label;
		}

		/**
		 * Returns a component that represents the value of this item
		 * 
		 * @return JComponent value component
		 */
		public JComponent getValueComponent() {
			return this.combo;
		}

		/**
		 * Returns a proper attribute value instead of a display value.
		 */
		public String getSelectedValue() {
			String displayName = (String) combo.getSelectedItem();
			return (String) valueMap.get(displayName);
		}
	}

}
