/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * EditAttributesDialog.java
 *
 * Created on 10. toukokuuta 2004, 11:22
 */

package kumbang.configurator.client.gui.dialog;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.SwingConstants;

import kumbang.configurator.client.core.TaskHandler;
import kumbang.configurator.client.gui.core.MainGUI;
import kumbang.core.configuration.instance.AttributeInstance;
import kumbang.core.configuration.instance.ComposableInstance;
import kumbang.core.configuration.instance.AttributedInstance;
import kumbang.core.configuration.task.EditAttributesTask;
import kumbang.core.model.type.Name;
import kumbang.util.gui.Icons;
import kumbang.util.gui.Margins;

/**
 * 
 * @author vmyllarn
 */
public class EditAttributesDialog extends TemplateDialog implements ActionListener {

	private AttributedInstance instance;

//	private ConsequenceHandler cnsqHandler;

	private List selectionItems;

	private JButton ok;
	private JButton cancel;

	public static final int MIN_AREA_WIDTH = 300;
	public static final int MIN_AREA_HEIGHT = 150;

	/** Creates a new instance of EditAttributesDialog */
	public EditAttributesDialog(MainGUI gui, AttributedInstance instance) {
		super(gui, "Edit attributes for " + instance);
		this.instance = instance;
//		this.cnsqHandler = gui.getConsequenceHandler();
		this.selectionItems = new LinkedList();
		// AttributeComboBoxRenderer renderer = new AttributeComboBoxRenderer();
		Iterator it = instance.getAttributes();
		while (it.hasNext()) {
			AttributeInstance attr = (AttributeInstance) it.next();
			AttributeSelectionItem item = new AttributeSelectionItem(attr);
			// item.setRenderer(renderer);
			selectionItems.add(item);
		}
		initComponents();
	}

	protected void initComponents() {
		/* create a box for textual information */
		Box textBox = Box.createVerticalBox();
		textBox
				.add(new JLabel("Edit attributes of " + instance.getAsTypedInstance().getInstanceKind() + " " + instance));
		textBox.add(new JLabel("by selecting values below."));
		super.addComponent(textBox);

		super.addComponent(getInfoComponent(instance.getAsTypedInstance()));

		if (instance.hasAttributes()) {
			super.addComponent(createAttributeArea());
		}

		ok = new JButton("OK");
		ok.addActionListener(this);
		super.addButton(ok, true);
		cancel = new JButton("Cancel");
		cancel.addActionListener(this);
		super.addButton(cancel, false);

		super.initComponents();
	}

	/**
	 * Creates and returns an area that contains information about attributes to be added. Includes
	 * checkboxes for each attribute.
	 * 
	 * @return JComponent an area for attributes
	 */
	private JComponent createAttributeArea() {
		Box box = Box.createVerticalBox();
		Iterator it = selectionItems.iterator();
		JPanel grid = new JPanel(new GridLayout(selectionItems.size(), 2));
		while (it.hasNext()) {
			AttributeSelectionItem asi = (AttributeSelectionItem) it.next();
			grid.add(asi.getNameComponent());
			grid.add(asi.getValueComponent());
		}
		grid.setMaximumSize(grid.getPreferredSize());

		Box labelArea = Box.createHorizontalBox();
		labelArea.add(new JLabel("Attributes"));
		labelArea.add(Box.createHorizontalGlue());

		JScrollPane scroll = new JScrollPane(grid);
		box.add(labelArea);
		box.add(Margins.createVerticalMarginS());
		box.add(scroll);
		box.add(Box.createVerticalGlue());

		box.setMinimumSize(new Dimension(MIN_AREA_WIDTH, MIN_AREA_HEIGHT));
		return box;
	}

	// private JComponent createAttributeArea() {
	// Box box = Box.createVerticalBox();
	// Iterator it = comboBoxes.iterator();
	// JPanel grid = new JPanel(new GridLayout(comboBoxes.size(), 2));
	// while (it.hasNext()) {
	// JComboBox combo = (JComboBox)it.next();
	// combo.setPreferredSize(new Dimension(ATTRIBUTE_WIDTH/2, combo.getMinimumSize().height));
	// combo.setMaximumSize(new Dimension(ATTRIBUTE_WIDTH/2, combo.getMinimumSize().height));
	// JLabel name = new JLabel(combo.toString());
	// name.setPreferredSize(new Dimension(ATTRIBUTE_WIDTH/2, name.getMinimumSize().height));
	// name.setMaximumSize(new Dimension(ATTRIBUTE_WIDTH/2, name.getMinimumSize().height));
	// grid.add(name);
	// grid.add(combo);
	// }
	// grid.setMaximumSize(grid.getPreferredSize());
	// box.add(grid);
	// box.add(Box.createVerticalGlue());
	//        
	// box.setPreferredSize(new Dimension(ATTRIBUTE_WIDTH,ATTRIBUTE_HEIGHT));
	// box.setMaximumSize(new Dimension(ATTRIBUTE_WIDTH,ATTRIBUTE_HEIGHT));
	// box.setMinimumSize(new Dimension(ATTRIBUTE_WIDTH,ATTRIBUTE_HEIGHT));
	// return box;
	// }

	public void actionPerformed(ActionEvent e) {
		Object o = e.getSource();
		if (o == ok) {
			setStatus(STATUS_OK);
			super.dispose();
		} else if (o == cancel) {
			super.dispose();
		}
	}

	public EditAttributesTask getTask(TaskHandler taskHandler) {
		EditAttributesTask task = null;
		/* check status */
		if (getStatus() != STATUS_OK) {
			return task;
		}
		task = new EditAttributesTask(this.instance, true);
		Iterator it = selectionItems.iterator();
		while (it.hasNext()) {
			AttributeSelectionItem asi = (AttributeSelectionItem) it.next();
			AttributeInstance attribute = asi.getAttribute();
			Name value = asi.getSelectedValue();
			if (!value.equals(attribute.getValue())) {
				task.setAttribute(attribute.getName(), value.getName());
			}
		}
		return task;
	}

	// -------------- ITEMS FOR SETTING ATTRIBUTE VALUES -----------------------

	class AttributeSelectionItem {

		/** a combo box for selecting attribute value */
		private JComboBox combo;

		/** a label for showing attribute name */
		private JLabel label;

		/** attribute instance */
		private AttributeInstance attribute;

		/** a map for mapping display values to actual values */
		// private HashMap valueMap;
		/**
		 * Creates a new AttributeComboBox based on the given attribute definition
		 * 
		 * @param attributeDef attribute definition
		 */
		public AttributeSelectionItem(AttributeInstance attribute) {
			this.attribute = attribute;
			this.combo = new JComboBox();
			this.label = new JLabel(attribute.getDisplayName());
			// this.valueMap = new HashMap();
			Iterator it = attribute.getPossibleValues();
			while (it.hasNext()) {
				Name value = (Name) it.next();
				AttributeComboItem item = new AttributeComboItem(value, false/*cnsqHandler
						.isExcludedAttributeValue(instance.getID(), attribute.getName(), value)*/);
				this.combo.addItem(item);
				if (attribute.isCurrentValue(value)) {
					this.combo.setSelectedItem(item);
				}
			}
			this.combo.setRenderer(new AttributeComboBoxRenderer());
			this.combo.setPreferredSize(new Dimension(MIN_AREA_WIDTH / 2, this.combo
					.getMinimumSize().height));
			this.combo.setMaximumSize(new Dimension(MIN_AREA_WIDTH / 2,
					this.combo.getMinimumSize().height));
			this.combo.setMinimumSize(new Dimension(MIN_AREA_WIDTH / 2,
					this.combo.getMinimumSize().height));
			this.label.setPreferredSize(new Dimension(MIN_AREA_WIDTH / 2, this.label
					.getMinimumSize().height));
			this.label.setMaximumSize(new Dimension(MIN_AREA_WIDTH / 2,
					this.label.getMinimumSize().height));
			this.label.setMinimumSize(new Dimension(MIN_AREA_WIDTH / 2,
					this.label.getMinimumSize().height));
		}

		/**
		 * Returns the attribute definition
		 * 
		 * @param AttributeDefinition
		 */
		public AttributeInstance getAttribute() {
			return this.attribute;
		}

		/**
		 * Returns a component that represents the name of this item
		 * 
		 * @return JComponent name component
		 */
		public JComponent getNameComponent() {
			return this.label;
		}

		/**
		 * Returns a component that represents the value of this item
		 * 
		 * @return JComponent value component
		 */
		public JComponent getValueComponent() {
			return this.combo;
		}

		/**
		 * Returns a proper attribute value instead of a display value.
		 */
		public Name getSelectedValue() {
			return ((AttributeComboItem) combo.getSelectedItem()).getValue();
		}
	}

	// -------------- COMBOITEMS FOR SETTING ATTRIBUTE VALUES -----------------------

	class AttributeComboItem {
		private Name value;
		private boolean excluded;

		public AttributeComboItem(Name value, boolean excluded) {
			this.value = value;
			this.excluded = excluded;
		}

		public Name getValue() {
			return value;
		}

		public boolean isExcluded() {
			return excluded;
		}

		public String toString() {
			return value.getDisplayName();
		}
	}

	class AttributeComboBoxRenderer extends JLabel implements ListCellRenderer {
		public AttributeComboBoxRenderer() {
			setOpaque(true);
			setHorizontalTextPosition(SwingConstants.LEADING);
			// setHorizontalAlignment(CENTER);
			// setVerticalAlignment(CENTER);
		}

		public Component getListCellRendererComponent(JList list, Object value, int index,
				boolean isSelected, boolean cellHasFocus) {
			if (isSelected) {
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			} else {
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			}
			if (value instanceof AttributeComboItem) {
				AttributeComboItem item = (AttributeComboItem) value;
				setText(item.toString());
				if (item.isExcluded()) {
					setIcon(Icons.STATUS_INVALID);
					setForeground(java.awt.Color.RED);
				} else {
					setIcon(null);
				}
			} else {
				setText("");
			}

			return this;
		}
	}

}
