/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * EditInterfacesDialog.java
 *
 * Created on 29. elokuuta 2004, 21:49
 */

package kumbang.configurator.client.gui.dialog;

import kumbang.configurator.client.core.TaskHandler;


import kumbang.configurator.client.gui.core.MainGUI;

import kumbang.util.gui.Margins;

import kumbang.core.configuration.instance.ComponentInstance;
import kumbang.core.configuration.instance.InterfaceInstance;
import kumbang.core.configuration.task.EditInterfacesTask;

import kumbang.core.model.type.InterfaceDefinition;
import kumbang.core.model.type.InterfaceType;

import java.awt.Dimension;
import java.awt.GridLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 *
 * @author  vmyllarn
 */
public class EditInterfacesDialog extends TemplateDialog implements ActionListener {
    
    private ComponentInstance parent;

    private List selectionItems;
    
    private JButton ok;
    private JButton cancel;
    
    public static final int MIN_AREA_WIDTH = 300;
    public static final int MIN_AREA_HEIGHT = 150;
    
    /** Creates a new instance of EditInterfacesDialog */
    public EditInterfacesDialog(MainGUI mainGUI, ComponentInstance parent) {
        super(mainGUI, "Edit interfaces for "+parent.toString());
        this.parent = parent;
        this.selectionItems = new LinkedList();
        Iterator it = parent.getComponentType().getInterfaces();
        while (it.hasNext()) {
            InterfaceDefinition intDef = (InterfaceDefinition) it.next();
            selectionItems.add(new InterfaceSelectionItem(intDef));
        }
        initComponents();
    }
    
    protected void initComponents() {
        /* create a box for textual information */
        Box textBox = Box.createVerticalBox();
        textBox.add(new JLabel("Edit interfaces of " + parent.getInstanceKind()
                               + " " + parent));
        textBox.add(new JLabel("by checking interfaces on or off below."));
        super.addComponent(textBox);

        super.addComponent(getInfoComponent(parent));
        
        if (parent.getComponentType().hasInterfaces()) {
            super.addComponent(createInterfaceArea());
        }

        ok = new JButton("OK");
        cancel = new JButton("Cancel");
        ok.addActionListener(this);
        cancel.addActionListener(this);
        super.addButton(ok, true);
        super.addButton(cancel, false);
        
        super.initComponents();
    }
    
    /**
     * Creates and returns an area that contains information about
     * interfaces to be added. Includes checkboxes for each interface.
     * @return JComponent an area for interfaces
     */
    private JComponent createInterfaceArea() {
        Box box = Box.createVerticalBox();
        Iterator it = selectionItems.iterator();
        JPanel grid 
                = new JPanel(new GridLayout(selectionItems.size(), 2));
        while (it.hasNext()) {
            InterfaceSelectionItem isi = (InterfaceSelectionItem) it.next();
            grid.add(isi.getNameComponent());
            grid.add(isi.getValueComponent());
        }
        grid.setMaximumSize(grid.getPreferredSize());
        
        Box labelArea = Box.createHorizontalBox();
        labelArea.add(new JLabel("Interfaces"));
        labelArea.add(Box.createHorizontalGlue());
        
        JScrollPane scroll = new JScrollPane(grid);
        box.add(labelArea);
        box.add(Margins.createVerticalMarginS());
        box.add(scroll);
        box.add(Box.createVerticalGlue());
        
        box.setMinimumSize(new Dimension(MIN_AREA_WIDTH, MIN_AREA_HEIGHT));
        return box;
    }
    
//    private JComponent createInterfaceArea() {
//        Box box = Box.createVerticalBox();
//        Iterator it = checkBoxes.iterator();
//        while (it.hasNext()) {
//            JCheckBox checkBox = (JCheckBox)it.next();
//            box.add(checkBox);
//        }
//        box.add(Box.createVerticalGlue());       
//        box.setPreferredSize(new Dimension(INTERFACE_WIDTH, INTERFACE_HEIGHT));
//        box.setMaximumSize(new Dimension(INTERFACE_WIDTH, INTERFACE_HEIGHT));
//        box.setMinimumSize(new Dimension(INTERFACE_WIDTH, INTERFACE_HEIGHT));
//        return box;
//    }
    
    
    public void actionPerformed(ActionEvent e) {
        Object o = e.getSource();
        if (o == ok) {
            setStatus(STATUS_OK);
            super.dispose();
        }
        else if (o == cancel) {
            super.dispose();
        }
    }
    
    public EditInterfacesTask getTask(TaskHandler taskHandler) {
    	EditInterfacesTask task = null;
        /* check status */
        if (getStatus() != STATUS_OK) {
            return task;
        }
        task = new EditInterfacesTask(this.parent, true);
        Iterator it = selectionItems.iterator();
        while (it.hasNext()) {
            InterfaceSelectionItem isi = (InterfaceSelectionItem) it.next();
            InterfaceDefinition def = isi.getInterfaceDefinition();
            InterfaceType type = isi.getInterfaceType();
            InterfaceInstance inst = isi.getInterfaceInstance();
            if (isi.isSelected() && inst == null) {
            	task.addInterface(def, type);
            } else if (!isi.isSelected() && inst != null) {
                task.removeInterface(inst);
            }
        }
        
        return task;
    }
    
    
    
    //-------------- ITEMS FOR SELECTING INTERFACES ---------------------------

    class InterfaceSelectionItem {
        
        private JCheckBox check;
        
        private JComboBox combo;
        
        /** interface definition that can be selected with this checkbox */
        private InterfaceDefinition interfaceDef;
        
//        /** parent component instance */
//        private ComponentInstance parent;
        
        /** a possibly exising interface instance for this definition */
        private InterfaceInstance instance;
        
        /** 
         * Creates a new InterfaceCheckBox based on the interface definition
         * @param interfaceDef interface definition
         */
        public InterfaceSelectionItem(InterfaceDefinition interfaceDef) {
            this.interfaceDef = interfaceDef;
            this.check = new JCheckBox(interfaceDef.getDisplayName() + " (" 
                    + (interfaceDef.isProvides() ? "provided" : "required")
                    + ")");
            this.combo = new JComboBox();
            Iterator it = interfaceDef.getPossibleTypes();
            while (it.hasNext()) {
                this.combo.addItem(it.next());
            }
            this.instance = parent.getInterface(interfaceDef);
            if (instance != null) {
                combo.setSelectedItem(instance.getType());
                this.check.setSelected(true);
            } else {
                this.check.setSelected(false);
            }
            this.combo.setPreferredSize(new Dimension(MIN_AREA_WIDTH / 2, 
                    this.combo.getMinimumSize().height));
            this.combo.setMaximumSize(new Dimension(MIN_AREA_WIDTH / 2, 
                    this.combo.getMinimumSize().height));
            this.combo.setMinimumSize(new Dimension(MIN_AREA_WIDTH / 2,
                    this.combo.getMinimumSize().height));
            this.check.setPreferredSize(new Dimension(MIN_AREA_WIDTH / 2, 
                    this.check.getMinimumSize().height));
            this.check.setMaximumSize(new Dimension(MIN_AREA_WIDTH / 2, 
                    this.check.getMinimumSize().height));
            this.check.setMinimumSize(new Dimension(MIN_AREA_WIDTH / 2,
                    this.check.getMinimumSize().height));
        }
        
        /**
         * Returns the interface definition
         * @return InterfaceDefinition interface definition
         */
        public InterfaceDefinition getInterfaceDefinition() {
            return this.interfaceDef;
        }
        
        /**
         * Returns the interface definition
         * @return InterfaceDefinition interface definition
         */
        public InterfaceType getInterfaceType() {
            return (InterfaceType) this.combo.getSelectedItem();
        }   
        
        /**
         * Returns the possibly exising interface instance for this
         * definition, or null if there does not exist such interface.
         * @return InterfaceInstance
         */
        public InterfaceInstance getInterfaceInstance() {
            return this.instance;
        }
        
        /**
         * Returns a component that represents the name of this item
         * @return JComponent name component
         */
        public JComponent getNameComponent() {
            return this.check;
        }
        
        /**
         * Returns a component that represents the value of this item
         * @return JComponent value component
         */
        public JComponent getValueComponent() {
            return this.combo;
        }
        
        /**
         * Tells whether this item has been selected (whether
         * its checkbox is currently selected)
         * @return boolean
         */
        public boolean isSelected() {
            return this.check.isSelected();
        }
    }
}
