/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * ExportModelDialog.java
 *
 * Created on 1. kes�kuuta 2004, 15:51
 */

package kumbang.configurator.client.gui.dialog;

import kumbang.configurator.client.core.ClientProperties;

import kumbang.configurator.client.gui.core.MainGUI;

import kumbang.util.gui.Margins;
import kumbang.util.gui.KumbangFileChooser;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;

import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

/**
 *
 * @author  vmyllarn
 */
public class ExportConfigurationDialog extends TemplateDialog implements ActionListener {
    
    public static final int TEXT_FIELD_SIZE = 30;
    
    private MainGUI mainGUI;
    
    private File file;
    private File dir;
    private String name;
    private int format;
    
    private JTextField dirField;
    private JTextField nameField;
    
    private JRadioButton formatKumbang;
    private JRadioButton formatXML;
    
    private JButton ok;
    private JButton cancel;
    private JButton changeDir;
    
    /** Creates a new instance of ExportModelDialog */
    public ExportConfigurationDialog(MainGUI mainGUI) {
        super(mainGUI, "Export configuration description");
        this.mainGUI = mainGUI;
        initComponents();
    }
    
    protected void initComponents() {
        /* create a box for textual information */
        Box textBox = Box.createVerticalBox();
        textBox.add(new JLabel("This exports a description of the current configuration"));
        textBox.add(new JLabel("to a file in your filesystem."));
        super.addComponent(textBox);
        
        GridBagLayout grid = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        JPanel filePanel = new JPanel(grid);
        
        c.gridx = 0;
        c.gridy = 0;
        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.insets = new Insets(0, 0, 0, Margins.M);
        JLabel temp = new JLabel("File name");
        filePanel.add(temp, c);
        
        c.gridx++;
        c.insets = new Insets(0, 0, 0, 0);
        nameField = new JTextField(TEXT_FIELD_SIZE);
        nameField.setText(mainGUI.getConfigurationName());
        nameField.addKeyListener(this);
        filePanel.add(nameField, c);
        
        c.gridy++;
        temp = new JLabel("Give file name without extension.");
        filePanel.add(temp, c);
        
        c.gridy++;
        temp = new JLabel("Avoid names with special characters.");
        filePanel.add(temp, c);
        
        c.gridy++;
        c.insets = new Insets(0, 0, Margins.L, 0);
        temp = new JLabel("Prefer letters (A-Z, a-z) and numbers.");
        filePanel.add(temp, c);
        
        c.gridx = 0;
        c.gridy++;
        c.insets = new Insets(0, 0, 0, Margins.M);
        temp = new JLabel("Directory");
        filePanel.add(temp, c);
        
        c.gridx++;
        c.insets = new Insets(0, 0, 0, 0);
        dirField = new JTextField(TEXT_FIELD_SIZE);
        dirField.setText(new File(".").getAbsolutePath());
        dirField.setEditable(false);
        filePanel.add(dirField, c);
        
        c.gridy++;
        temp = new JLabel("Give directory where output file is saved.");
        filePanel.add(temp, c);
        
        c.gridy++;
        c.anchor = GridBagConstraints.FIRST_LINE_END;
        changeDir = new JButton("Browse...");
        changeDir.addActionListener(this);
        filePanel.add(changeDir, c);
        
        super.addComponent(filePanel);
        
        /* radio buttons */
        
        JPanel radioPanel = new JPanel(grid);
        
        c.gridx = 0;
        c.gridy = 0;
        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.insets = new Insets(0, 0, 0, Margins.M);
        temp = new JLabel("Output format");
        radioPanel.add(temp, c);
        
        c.gridx++;
        c.insets = new Insets(0, 0, 0, 0);
        formatKumbang = new JRadioButton("Kumbang (extension \""
                                +ClientProperties.EXTENSION_EXPORT+"\")");
        formatKumbang.setSelected(true);
        radioPanel.add(formatKumbang, c);
        
        c.gridy++;
        formatXML = new JRadioButton("XML (extension \""+
                                ClientProperties.EXTENSION_EXPORT_XML+"\")");
        radioPanel.add(formatXML, c);
        
        ButtonGroup group = new ButtonGroup();
        group.add(formatKumbang);
        group.add(formatXML);
        
        super.addComponent(radioPanel);
        
        /* accept buttons */
        
        ok = new JButton("Export");
        cancel = new JButton("Cancel");
        ok.addActionListener(this);
        cancel.addActionListener(this);
        super.addButton(ok, true);
        super.addButton(cancel, false);
        
        super.initComponents();
    }
    
    public void actionPerformed(ActionEvent e) {
        Object o = e.getSource();
        if (o == ok) {
            if (nameField.getText() == null) {
                JOptionPane.showMessageDialog(this, "You must enter a name for the configuration.",
                 "Error when exporting configuration", JOptionPane.ERROR_MESSAGE);
                return;
            }
            name = nameField.getText();
            if (dirField.getText() == null) {
                JOptionPane.showMessageDialog(this, "You must specify a directory.",
                 "Error when exporting configuration", JOptionPane.ERROR_MESSAGE);
                return;
            }
            dir = new File(dirField.getText());
            if (formatKumbang.isSelected()) {
                format = ClientProperties.EXPORT_FORMAT_TEXT;
            }
            else {
                format = ClientProperties.EXPORT_FORMAT_XML;
            }
            file = new File(dir, name+getExtension());
            if (file.exists()) {
                int retValue = JOptionPane.showConfirmDialog(this, 
                    "A file with name "+file.getName()+" already exists.\n"+
                    "Click yes to proceed and overwrite the file.\n"+
                    "Click no to give another name for the file.", 
                    "Export configuration", JOptionPane.YES_NO_OPTION);
                if (retValue == JOptionPane.NO_OPTION) {
                    return;
                }
            }
            setStatus(STATUS_OK);
            super.dispose();
        }
        else if (o == cancel) {
            super.dispose();
        }
        else if (o == changeDir) {
            dir = setExportDir();
            if (dir != null) {
                dirField.setText(dir.toString());
                dirField.repaint();
            }
        }
    }
    
    public int getFormat() {
        return format;
    }
    
    public File getExportFile() {
        return file;
    }
    
    private String getExtension() {
        if (format == ClientProperties.EXPORT_FORMAT_TEXT) {
            return ClientProperties.EXTENSION_EXPORT;
        }
        else if (format == ClientProperties.EXPORT_FORMAT_XML) {
            return ClientProperties.EXTENSION_EXPORT_XML;
        }
        else {
            return "";
        }
    }
    
    private File setExportDir() {
        KumbangFileChooser fileChooser = mainGUI.getFileChooser();
        if (!fileChooser.showChooseDirectoryDialog(this)) {
            return null;
        }
        return fileChooser.getSelectedFile();
    }
}
