/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * ImportModelDialog.java
 *
 * Created on 23. huhtikuuta 2004, 14:03
 */

package kumbang.configurator.client.gui.dialog;

import kumbang.configurator.client.gui.core.MainGUI;

import kumbang.util.gui.Margins;
import kumbang.util.gui.KumbangFileChooser;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 *
 * @author  vmyllarn
 */
public class ImportModelDialog extends TemplateDialog implements ActionListener {
    
    public static final int TEXT_FIELD_SIZE = 30;
    
    private MainGUI mainGUI;
    
    private File file;
    private String name;
    
    private JTextField fileField;
    private JTextField nameField;
    
    private JButton changeFile;
    private JButton ok;
    private JButton cancel;
    
    /** Creates a new instance of ImportModelDialog */
    public ImportModelDialog(MainGUI mainGUI) {
        super(mainGUI, "Import configuration model to repository");
        this.mainGUI = mainGUI;
        initComponents();
    }
    
    protected void initComponents() {
        /* create a box for textual information */
        Box textBox = Box.createVerticalBox();
        textBox.add(new JLabel("This imports a configuration model to the repository."));
        textBox.add(new JLabel("The model is imported from your local filesystem."));
        textBox.add(new JLabel("The model should be a textual Kumbang file."));
        super.addComponent(textBox);
        
        GridBagLayout grid = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        JPanel filePanel = new JPanel(grid);
        
        c.gridx = 0;
        c.gridy = 0;
        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.insets = new Insets(0, 0, 0, Margins.M);
        JLabel temp = new JLabel("Model file");
        filePanel.add(temp, c);
        
        c.gridx++;
        c.insets = new Insets(0, 0, 0, 0);
        fileField = new JTextField(TEXT_FIELD_SIZE);
        fileField.addKeyListener(this);
        filePanel.add(fileField, c);
        
        c.gridy++;
        temp = new JLabel("File should contain a valid Kumbang model.");
        filePanel.add(temp, c);
        
        c.gridy++;
        c.anchor = GridBagConstraints.FIRST_LINE_END;
        c.insets = new Insets(0, 0, Margins.L, 0);
        changeFile = new JButton("Browse...");
        changeFile.addActionListener(this);
        changeFile.addKeyListener(this);
        filePanel.add(changeFile, c);
        
        c.gridx = 0;
        c.gridy++;
        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.insets = new Insets(0, 0, 0, Margins.M);
        temp = new JLabel("Model name");
        filePanel.add(temp, c);
        
        c.gridx++;
        c.insets = new Insets(0, 0, 0, 0);
        nameField = new JTextField(TEXT_FIELD_SIZE);
        nameField.addKeyListener(this);
        filePanel.add(nameField, c);
        
        c.gridy++;
        temp = new JLabel("Avoid names with special characters.");
        filePanel.add(temp, c);
        
        c.gridy++;
        temp = new JLabel("Try to use letters (A-Z, a-z) and numbers.");
        filePanel.add(temp, c);
        
        super.addComponent(filePanel);
        
        
        ok = new JButton("Import");
        cancel = new JButton("Cancel");
        ok.addActionListener(this);
        cancel.addActionListener(this);
        super.addButton(ok, true);
        super.addButton(cancel, false);
        
        super.initComponents();
    }
    
    public File getModelFile() {
        return file;
    }
    
    public String getName() {
        return name;
    }
        
    public void actionPerformed(ActionEvent e) {
        Object o = e.getSource();
        if (o == ok) {
            String fileText = fileField.getText();
            name = nameField.getText();
            if (fileText != null && fileText.length() > 0) {
                file = new File(fileText);
            }
            if (file == null) {
                JOptionPane.showMessageDialog(this, "You must specify a model file.",
                 "Error when importing model", JOptionPane.ERROR_MESSAGE);
                return;
            }
            if (name == null || name.length() == 0) {
                JOptionPane.showMessageDialog(this, "You must enter a name for the model.",
                 "Error when importing model", JOptionPane.ERROR_MESSAGE);
                return;
            }
            setStatus(STATUS_OK);
            super.dispose();
        }
        else if (o == cancel) {
            super.dispose();
        }
        else if (o == changeFile) {
            file = setModelFile();
            if (file != null) {
                fileField.setText(file.toString());
                fileField.repaint();
                // if there is no model name, extract name of the file
                if (nameField.getText() == null 
                        || nameField.getText().length() == 0) {
                    String[] split = file.getName().split("\\.");
                    nameField.setText(split[0]);
                }
            }
        }
    }
    
    private File setModelFile() {
        KumbangFileChooser fileChooser = mainGUI.getFileChooser();
        if (!fileChooser.showChooseModelFileDialog(this)) {
            return null;
        }
        return fileChooser.getSelectedFile();
    }
}
