/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * OpenModelDialog.java
 *
 * Created on 26. huhtikuuta 2004, 14:45
 */

package kumbang.configurator.client.gui.dialog;

import kumbang.configurator.client.gui.core.MainGUI;

import kumbang.util.gui.Margins;

import java.awt.Component;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Calendar;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

/**
 *
 * @author  vmyllarn
 */
public class OpenModelDialog extends TemplateDialog implements ActionListener, ListSelectionListener {
    
    private String[] modelNames;
    
    private String modelName;
    private String configurationName;
    
    private JList nameList;
    
    private JTextField nameField;
    
    private JButton ok;
    private JButton cancel;
    
    /** Creates a new instance of OpenModelDialog */
    public OpenModelDialog(MainGUI mainGUI, String[] modelNames) {
        super(mainGUI, "Open configuration model");
        this.modelNames = modelNames;
        initComponents();
    }
    
    protected void initComponents() {
        /* create a box for textual information */
        Box textBox = Box.createVerticalBox();
        textBox.add(new JLabel("This opens a configuration model from the repository"));
        textBox.add(new JLabel("and initialises a new configuration."));
        textBox.add(new JLabel("Please select a model from the list."));
        super.addComponent(textBox);
        
        /* create a box for model list */
        JLabel listLabel = new JLabel("Available models");
        listLabel.setMaximumSize(listLabel.getPreferredSize());
        listLabel.setAlignmentY(Component.TOP_ALIGNMENT);
        nameList = new JList(modelNames);
        nameList.addKeyListener(this);
        nameList.addListSelectionListener(this);
        JScrollPane scroll = new JScrollPane(nameList);
        scroll.setAlignmentY(Component.TOP_ALIGNMENT);
        nameList.setPreferredSize(Margins.LIST_SIZE);
        nameList.setMaximumSize(nameList.getPreferredSize());
        Box listBox = Box.createHorizontalBox();
        listBox.add(listLabel);
        listBox.add(Margins.createHorizontalMarginS());
        listBox.add(scroll);
        listBox.add(Box.createHorizontalGlue());
        super.addComponent(listBox);
        
        /* add textual information */
        //super.addComponent(new JLabel("Please give name for the new configuration."));
        
        /* add text field for configuration name */
        Box nameBox = Box.createHorizontalBox();
        nameBox.add(new JLabel("Configuration name"));
        nameBox.add(Margins.createHorizontalMarginS());
        nameField = new JTextField(20);
        nameField.addKeyListener(this);
        nameBox.add(nameField);
        nameBox.add(Box.createHorizontalGlue());
        super.addComponent(nameBox);
        
        ok = new JButton("Open");
        cancel = new JButton("Cancel");
        ok.addActionListener(this);
        cancel.addActionListener(this);
        super.addButton(ok, true);
        super.addButton(cancel, false);
        
        super.initComponents();
        
        if (modelNames.length > 0) {
            nameList.setSelectedIndex(0);
            nameList.requestFocusInWindow();
        } else {
            cancel.requestFocusInWindow();
        }
    }
    
    public void actionPerformed(ActionEvent e) {
        Object o = e.getSource();
        if (o == ok) {
            modelName = (String)nameList.getSelectedValue();
            if (modelName == null || modelName.length() == 0) {
                JOptionPane.showMessageDialog(this, "You must choose a model from the list.",
                 "Error when opening model", JOptionPane.ERROR_MESSAGE);
                return;
            }
            configurationName = nameField.getText();
            if (configurationName == null || configurationName.length() == 0) {
                JOptionPane.showMessageDialog(this, "You must give a name for the configuration.",
                 "Error when opening model", JOptionPane.ERROR_MESSAGE);
                return;
            }
            setStatus(STATUS_OK);
            super.dispose();
        }
        else if (o == cancel) {
            super.dispose();
        }
    }
    
    public void valueChanged(ListSelectionEvent e) {
        String str = (String)nameList.getSelectedValue();
        if (str == null || str.length() == 0) {
            return;
        }
        Calendar cal = Calendar.getInstance();
        str += "_"+cal.get(Calendar.YEAR)+"-"+(cal.get(Calendar.MONTH)+1)+"-"+cal.get(Calendar.DATE);
        nameField.setText(str);
    }
    
    public String getModelName() {
        return modelName;
    }
    
    public String getConfigurationName() {
        return configurationName;
    }
    
    
}
