/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * SaveConfigurationDialog.java
 *
 * Created on 2. kes�kuuta 2004, 16:46
 */

package kumbang.configurator.client.gui.dialog;

import kumbang.configurator.client.gui.core.MainGUI;

import kumbang.util.gui.Margins;
import kumbang.util.gui.KumbangFileChooser;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 *
 * @author  vmyllarn
 */
public class SaveConfigurationDialog extends TemplateDialog implements ActionListener {
    
    public static final int TEXT_FIELD_SIZE = 30;
    
    private MainGUI mainGUI;
    
    private File file;
    
    private JTextField dirField;
    private JTextField nameField;
    
    private JButton ok;
    private JButton cancel;
    private JButton changeDir;
    
    /** Creates a new instance of SaveConfigurationDialog */
    public SaveConfigurationDialog(MainGUI mainGUI) {
        super(mainGUI, "Save configuration as...");
        this.mainGUI = mainGUI;
        initComponents();
    }
    
    
    protected void initComponents() {
        JLabel temp = new JLabel("Export configuration to a file");
        super.addComponent(temp);
        
        GridBagLayout grid = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        JPanel filePanel = new JPanel(grid);
        
        c.gridx = 0;
        c.gridy = 0;
        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.insets = new Insets(0, 0, 0, Margins.M);
        temp = new JLabel("File name");
        filePanel.add(temp, c);
        
        c.gridx++;
        c.insets = new Insets(0, 0, 0, 0);
        nameField = new JTextField(TEXT_FIELD_SIZE);
        nameField.setText(mainGUI.getConfigurationName());
        nameField.addKeyListener(this);
        filePanel.add(nameField, c);
        
        c.gridy++;
        temp = new JLabel("Give file name without extension.");
        filePanel.add(temp, c);
        
        c.gridy++;
        temp = new JLabel("Avoid names with special characters.");
        filePanel.add(temp, c);
        
        c.gridy++;
        c.insets = new Insets(0, 0, Margins.L, 0);
        temp = new JLabel("Prefer letters (A-Z, a-z) and numbers.");
        filePanel.add(temp, c);
        
        c.gridx = 0;
        c.gridy++;
        c.insets = new Insets(0, 0, 0, Margins.M);
        temp = new JLabel("Directory");
        filePanel.add(temp, c);
        
        c.gridx++;
        c.insets = new Insets(0, 0, 0, 0);
        dirField = new JTextField(TEXT_FIELD_SIZE);
        dirField.setText(new File(".").getAbsolutePath());
        dirField.setEditable(false);
        filePanel.add(dirField, c);
        
        c.gridy++;
        temp = new JLabel("Give directory where configuration is saved.");
        filePanel.add(temp, c);
        
        c.gridy++;
        c.anchor = GridBagConstraints.FIRST_LINE_END;
        changeDir = new JButton("Browse...");
        changeDir.addActionListener(this);
        changeDir.addKeyListener(this);
        filePanel.add(changeDir, c);
        
        super.addComponent(filePanel);
        
        /* accept buttons */
        
        ok = new JButton("Export");
        cancel = new JButton("Cancel");
        ok.addActionListener(this);
        cancel.addActionListener(this);
        super.addButton(ok, true);
        super.addButton(cancel, false);
        
        super.initComponents();
    }
    
    public void actionPerformed(ActionEvent e) {
        Object o = e.getSource();
        if (o == ok) {
            if (nameField.getText() == null) {
                JOptionPane.showMessageDialog(this, "You must enter a name for the configuration.",
                 "Error when exporting configuration", JOptionPane.ERROR_MESSAGE);
                return;
            }
            if (dirField.getText() == null) {
                JOptionPane.showMessageDialog(this, "You must specify a directory.",
                 "Error when exporting configuration", JOptionPane.ERROR_MESSAGE);
                return;
            }
            File dir = new File(dirField.getText());
            file = new File(dir, nameField.getText()+".kbc");
            if (file.exists()) {
                int retValue = JOptionPane.showConfirmDialog(this, 
                    "A file with name "+file.getName()+" already exists.\n"+
                    "Click yes to proceed and overwrite the file.\n"+
                    "Click no to give another name for the file.", 
                    "Save configuration", JOptionPane.YES_NO_OPTION);
                if (retValue == JOptionPane.NO_OPTION) {
                    return;
                }
            }
            setStatus(STATUS_OK);
            super.dispose();
        }
        else if (o == cancel) {
            super.dispose();
        }
        else if (o == changeDir) {
            File dir = setSaveDir();
            if (file != null) {
                dirField.setText(dir.toString());
                dirField.repaint();
            }
        }
    }
    
    public File getSaveFile() {
        return file;
    }
    
    private File setSaveDir() {
        KumbangFileChooser fileChooser = mainGUI.getFileChooser();
        if (!fileChooser.showChooseDirectoryDialog(this)) {
            return null;
        }
        return fileChooser.getSelectedFile();
    }
}
