/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * SelectPartDialog.java
 *
 * Created on 27. toukokuuta 2004, 14:35
 */

package kumbang.configurator.client.gui.dialog;

import kumbang.configurator.client.gui.core.MainGUI;

import kumbang.util.gui.Margins;

import kumbang.core.configuration.instance.ComposableInstance;
import kumbang.core.configuration.instance.PartInstance;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Iterator;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

/**
 *
 * @author  vmyllarn
 */
public class SelectPartDialog extends TemplateDialog implements ActionListener {
    
    private ComposableInstance instance;
    
    private JComboBox parts;
    
    private JButton ok;
    private JButton cancel;
    
    /** Creates a new instance of SelectPartDialog */
    public SelectPartDialog(MainGUI mainGUI, ComposableInstance instance) {
        super(mainGUI, "Select part");
        this.instance = instance;
        
        initComponents();
    }
    
    protected void initComponents() {
        Box textBox = Box.createVerticalBox();
        textBox.add(new JLabel("Before you can add a new " + instance.getInstanceKind() + " under " + instance + ","));
        textBox.add(new JLabel("you must select the part under which the " + instance.getInstanceKind() + " can be added."));
        textBox.add(new JLabel("Please select a part from the following list."));
        super.addComponent(textBox);
        
        super.addComponent(getInfoComponent(instance));
        
        Box partBox = Box.createHorizontalBox();
        parts = new JComboBox();
        Iterator iter = instance.getParts();
        while (iter.hasNext()) {
            parts.addItem(iter.next());
        }
        partBox.add(new JLabel("Select a part:"));
        partBox.add(Margins.createHorizontalMarginM());
        partBox.add(parts);
        partBox.add(Box.createHorizontalGlue());
        super.addComponent(partBox);
        
        ok = new JButton("OK");
        cancel = new JButton("Cancel");
        ok.addActionListener(this);
        cancel.addActionListener(this);
        super.addButton(ok, true);
        super.addButton(cancel, false);
        
        super.initComponents();
    }
    
    public void actionPerformed(ActionEvent e) {
        Object o = e.getSource();
        if (o == ok) {
            if (getPart() == null) {
                JOptionPane.showMessageDialog(this, "You must select a part first!",
                 "Select part", JOptionPane.ERROR_MESSAGE);
                return;
            }
            if (isSelectedPartFull()) {
                JOptionPane.showMessageDialog(this,
                     "You selected a part that has already the maximum number of "
                     + instance.getInstanceKind() + "s.\n"
                     + "Thus you cannot add a new " + instance.getInstanceKind() + " to this part.\n"
                     + "Please select another part.", "Select part", 
                     JOptionPane.ERROR_MESSAGE);
                 return;
            }
            setStatus(STATUS_OK);
            super.dispose();
        }
        else if (o == cancel) {
            super.dispose();
        }
    }
    
    public PartInstance getPart() {
        return (PartInstance)parts.getSelectedItem();
    }
    
    private boolean isSelectedPartFull() {
        PartInstance part = getPart();
        if (part != null) {
            return part.isFull();
        }
        else {
            return false;
        }
    }
}
