/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * TemplateDialog.java
 *
 * Created on 11. maaliskuuta 2004, 12:59
 */

package kumbang.configurator.client.gui.dialog;

import kumbang.configurator.client.gui.core.MainGUI;

import kumbang.util.gui.Margins;

import kumbang.core.configuration.instance.TypedInstance;
import kumbang.core.configuration.instance.InterfaceInstance;


import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Insets;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;



/**
 *
 * @author  vmyllarn
 */
public abstract class TemplateDialog extends JDialog implements KeyListener {

    public static final boolean STATUS_OK = true;
    public static final boolean STATUS_CANCEL = false;
    
    public static final int MINIMUM_WIDTH = 200;
    public static final int MINIMUM_HEIGHT = 300;
    
    private ImageIcon logoIcon = new ImageIcon("pics/logo.gif");
    
    private List components;
    
    private JButton okButton;
    
    private List buttons;
    
    private String title;
    
    private boolean status;
    
    /** Creates a new instance of TemplateDialog */
    public TemplateDialog(MainGUI gui, String title) {
        super(gui, title, true);
        this.title = title;
        this.status = STATUS_CANCEL;
        components = new LinkedList();
        buttons = new LinkedList();
        setLocationRelativeTo(gui);
    }
    
    public boolean getStatus() {
        return this.status;
    }
    
    public void keyTyped(KeyEvent e) {}

    public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() != KeyEvent.VK_ENTER) {
            return;
        }
        if (e.getComponent() instanceof JButton) {
            ((JButton) e.getComponent()).doClick();
        } else {
            this.okButton.doClick();
        }
    }

    public void keyReleased(KeyEvent e) {}
    
    protected void setStatus(boolean status) {
        this.status = status;
    }
        
    protected void addButton(JButton button, boolean ok) {
        buttons.add(button);
        button.addKeyListener(this);
        if (ok) {
            this.okButton = button;
        }
    }
    
    protected void addComponent(JComponent component) {
        if (component == null) {
            return;
        }
        component.setAlignmentX(Component.LEFT_ALIGNMENT);
        component.setMaximumSize(component.getPreferredSize());
        components.add(component);
    }
    
    protected void initComponents() {
        Box innerBox = Box.createVerticalBox();
        Iterator it = components.iterator();
        while (it.hasNext()) {
            innerBox.add((JComponent)it.next());
            if (it.hasNext()) {
                innerBox.add(Margins.createVerticalMarginL());
            }
        }
        Dimension prefDim = new Dimension(innerBox.getPreferredSize());
        if (prefDim.width < MINIMUM_WIDTH) {
            prefDim.width = MINIMUM_WIDTH;
        }
        if (prefDim.height < MINIMUM_HEIGHT) {
            prefDim.height = MINIMUM_HEIGHT;
        }
        innerBox.setPreferredSize(prefDim);
        
        JPanel titledPanel = new JPanel();
        //titledPanel.setLayout(new BoxLayout(titledPanel, BoxLayout.LINE_AXIS));
        titledPanel.setLayout(new BorderLayout());
        titledPanel.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createTitledBorder(title), 
            Margins.createEmptyBorderL()
        ));
        titledPanel.add(innerBox);
        //titledPanel.add(Box.createHorizontalGlue());
        
        
        Box buttonBox = Box.createHorizontalBox();
        buttonBox.add(Box.createHorizontalGlue());
        it = buttons.iterator();
        while (it.hasNext()) {
            buttonBox.add((JButton)it.next());
            if (it.hasNext()) {
                buttonBox.add(Margins.createHorizontalMarginS());
            }
        }
        
        
        Box componentBox = Box.createVerticalBox();
        titledPanel.setAlignmentX(Component.RIGHT_ALIGNMENT);
        componentBox.add(titledPanel);
        componentBox.add(Margins.createVerticalMarginL());
        buttonBox.setAlignmentX(Component.RIGHT_ALIGNMENT);
        componentBox.add(buttonBox);
        componentBox.add(Box.createVerticalGlue());
        componentBox.setAlignmentY(Component.TOP_ALIGNMENT);
        
        
        JLabel icon = new JLabel(logoIcon);
        icon.setAlignmentY(Component.TOP_ALIGNMENT);

        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.LINE_AXIS));
        mainPanel.setBorder(Margins.createEmptyBorderL());
        mainPanel.add(icon);
        mainPanel.add(Margins.createHorizontalMarginL());
        mainPanel.add(componentBox);
         
        getContentPane().add(mainPanel);
        pack();
        
        int locationX = getLocation().x-getWidth()/2;
        int locationY = getLocation().y - getHeight()/2;
        if (locationX < 0) {
            locationX = 0;
        }
        if (locationY < 0) {
            locationY = 0;
        }
        setLocation(locationX, locationY);
    }
    
    public JComponent getInfoComponent(TypedInstance instance) {
        if (instance == null) {
            return getInfoComponent("", "", "", "");
        } else if (instance.getParent() != null) {
            return getInfoComponent(instance.getDisplayName(), 
                    instance.getType().getDisplayName(), 
                    instance.getParent().getDisplayName(),
                    instance.getParent().getType().getDisplayName());
        } else {
            return getInfoComponent(instance.getDisplayName(), 
                                    instance.getType().getDisplayName(), 
                                    "", "");
        }
    }
    
    public JComponent getInfoComponent(InterfaceInstance instance) {
        if (instance == null) {
            return getInfoComponent("", "", "", "");
        } else if (instance.getParent() != null) {
            return getInfoComponent(instance.getDisplayName(), 
                    instance.getType().getDisplayName(), 
                    instance.getParent().getDisplayName(),
                    instance.getParent().getType().getDisplayName());
        } else {
            return getInfoComponent(instance.getDisplayName(), 
                                    instance.getType().getDisplayName(), 
                                    "", "");
        }
    }
    
    public JComponent getInfoComponent(String name, String type, 
                                       String parentName, String parentType) {
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.insets = new Insets(0, 0, Margins.S, Margins.S);
        c.anchor = GridBagConstraints.FIRST_LINE_START;
        c.gridx = 0;
        c.gridy = 0;
        
        // create a panel for name and type information
        JPanel infoPanel = new JPanel(gridbag);
        
        // name
        JLabel infoLabel = new JLabel("Name");
        gridbag.setConstraints(infoLabel, c);
        infoPanel.add(infoLabel);
        c.gridy++;
        
        // type
        infoLabel = new JLabel("Type");
        gridbag.setConstraints(infoLabel, c);
        infoPanel.add(infoLabel);
        c.gridy++;
        
        // parent name
        infoLabel = new JLabel("Parent name");
        gridbag.setConstraints(infoLabel, c);
        infoPanel.add(infoLabel);
        c.gridy++;
        
        // parent type
        infoLabel = new JLabel("Parent type");
        gridbag.setConstraints(infoLabel, c);
        infoPanel.add(infoLabel);
            
        c.gridx++;
        c.gridy = 0;
        
        // name
        infoLabel = new JLabel(name);
        gridbag.setConstraints(infoLabel, c);
        infoPanel.add(infoLabel);
        c.gridy++;
        
        // type
        infoLabel = new JLabel(type);
        gridbag.setConstraints(infoLabel, c);
        infoPanel.add(infoLabel);
        c.gridy++;
        
        // parent name
        infoLabel = new JLabel(parentName);
        gridbag.setConstraints(infoLabel, c);
        infoPanel.add(infoLabel);
        c.gridy++;
        
        // parent type
        infoLabel = new JLabel(parentType);
        gridbag.setConstraints(infoLabel, c);
        infoPanel.add(infoLabel);
        
        return infoPanel;
    }
}
