/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * ConfigurationNode.java
 *
 * Created on 9. syyskuuta 2004, 12:48
 */

package kumbang.configurator.client.gui.tree;

import kumbang.util.gui.Icons;

import javax.swing.ImageIcon;
import javax.swing.tree.DefaultMutableTreeNode;

/**
 *
 * @author  vmyllarn
 */
public class ConfigurationNode extends DefaultMutableTreeNode {
    
    public static final int COMPONENT_ROOT   = 0;
    public static final int FEATURE_ROOT     = 1;
    public static final int PART_HOLDER      = 2;
    public static final int ATTRIBUTE_HOLDER = 3;
    public static final int INTERFACE_HOLDER = 4;
    
    /** smaller icon for this node */
    private ImageIcon iconSmall;
    
    /** larger icon for this node */
    private ImageIcon iconLarge;
    
    /** Creates a new instance of ConfigurationNode */
    public ConfigurationNode(Object value, ImageIcon iconSmall, ImageIcon iconLarge) {
        super(value);
        this.iconSmall = iconSmall;
        this.iconLarge = iconLarge;
    }
    
    public ConfigurationNode(int type) {
        switch (type) {
            case COMPONENT_ROOT:
                super.setUserObject("Component configuration");
                this.iconSmall = Icons.STRUCTURE;
                this.iconLarge = Icons.STRUCTURE_L;
                break;
            case FEATURE_ROOT:
                super.setUserObject("Feature configuration");
                this.iconSmall = Icons.STRUCTURE;
                this.iconLarge = Icons.STRUCTURE_L;
                break;
            case PART_HOLDER:
                super.setUserObject("Parts");
                this.iconSmall = Icons.DOWN;
                this.iconLarge = Icons.DOWN_L;
                break;
            case ATTRIBUTE_HOLDER:
                super.setUserObject("Attributes");
                this.iconSmall = Icons.DOWN;
                this.iconLarge = Icons.DOWN_L;
                break;
            case INTERFACE_HOLDER:
                super.setUserObject("Interfaces");
                this.iconSmall = Icons.DOWN;
                this.iconLarge = Icons.DOWN_L;
        }
    }
    
    public ImageIcon getSmallIcon() {
        return this.iconSmall;
    }
    
    public ImageIcon getLargeIcon() {
        return this.iconLarge;
    }
    
    protected void setSmallIcon(ImageIcon icon) {
        this.iconSmall = icon;
    }
    
    protected void setLargeIcon(ImageIcon icon) {
        this.iconLarge = icon;
    }
    
    public String toString() {
        return getUserObject().toString();
    }
}
