/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.actions;

import kumbang.configurator.Activator;
import kumbang.configurator.client.core.ClientServices;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.event.ConfigurationModificationListener;
import kumbang.configurator.client.plugin.event.ConfigurationModifiedEvent;
import kumbang.configurator.client.plugin.event.ConfigurationOpenedClosedEvent;
import kumbang.configurator.client.plugin.event.ConfigurationOpenedClosedListener;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;


/**
 * Our sample action implements workbench action delegate.
 * The action proxy will be created by the workbench and
 * shown in the UI. When the user tries to use the action,
 * this delegate will be created and execution will be 
 * delegated to it.
 * @see IWorkbenchWindowActionDelegate
 */
public class CloseConfiguration implements IWorkbenchWindowActionDelegate, ConfigurationOpenedClosedListener, ConfigurationModificationListener {
	private IWorkbenchWindow window;
	private ClientServices client;
	private IAction action;
	/**
	 * The constructor.
	 */
	public CloseConfiguration() {
	}

	/**
	 * The action has been activated. The argument of the
	 * method represents the 'real' action sitting
	 * in the workbench UI.
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {
		this.action = action;
		
		client = Activator.getDefault().getHandler().getClientServices();
		
		if (client == null) {
			Activator.getDefault().getHandler().fireOpenCloseEvent(ConfigurationOpenedClosedEvent.EVENT_CLOSE);
			return;
		}
			
		
		if (client.isConfigurationModified()) {
			if (MessageDialog.openConfirm(window.getShell(), "Confirm Closing", "Are you sure you want to close? Unsaved changes will be lost")) {
				client.closeConfiguration();
			} else {
				return;
			}
		}
		else {
			System.out.println("Config was not modified");
			client.closeConfiguration();
		}
		
		Activator.getDefault().getHandler().fireOpenCloseEvent(ConfigurationOpenedClosedEvent.EVENT_CLOSE);
	}

	/**
	 * Selection in the workbench has been changed. We 
	 * can change the state of the 'real' action here
	 * if we want, but this can only happen after 
	 * the delegate has been created.
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
        if (this.action == null) {
            action.setEnabled(false);
            this.action = action;
        }
        // This is done to make the action disabled on startup when there's 
		// no model open
//		if (this.action == null) {
//			if (KumbangConfiguratorPlugin.getDefault().isServerAvailable()) {
//				if (!KumbangConfiguratorPlugin.getDefault().getClientServices().isModelInitialized()) {
//					action.setEnabled(false);
//				}
//			} else {
//				action.setEnabled(false);
//			}
//		}
//			this.action = action;
//}
    }


	/**
	 * We can use this method to dispose of any system
	 * resources we previously allocated.
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
		Activator.getDefault().getHandler().removeConfigurationModificationListener(this);
		Activator.getDefault().getHandler().removeOpenCloseListener(this);
	}

	/**
	 * We will cache window object in order to
	 * be able to provide parent shell for the message dialog.
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
		Activator.getDefault().getHandler().addConfigurationModificationListener(this);
		Activator.getDefault().getHandler().addOpenCloseListener(this);
		
	}

	public void handleOpenCloseEvent(ConfigurationOpenedClosedEvent e) {
		if (e.getEventType().equals(ConfigurationOpenedClosedEvent.EVENT_INITIALIZE) ||
				e.getEventType().equals(ConfigurationOpenedClosedEvent.EVENT_SAVEOPENED)) {
			if (this.action != null) {
				this.action.setEnabled(true);
			}
		} else if (e.getEventType().equals(ConfigurationOpenedClosedEvent.EVENT_CLOSE)) {
			if (this.action != null) {
				this.action.setEnabled(false);
			}
		}
	} 

	public void configurationModified(ConfigurationModifiedEvent e) {
		// TODO Auto-generated method stub
		
	}
}
