/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.actions;

import kumbang.configurator.Activator;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.dialogs.ExportConfigurationDialog;
import kumbang.configurator.client.plugin.event.ConfigurationModificationListener;
import kumbang.configurator.client.plugin.event.ConfigurationModifiedEvent;
import kumbang.configurator.client.plugin.event.ConfigurationOpenedClosedEvent;
import kumbang.configurator.client.plugin.event.ConfigurationOpenedClosedListener;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

public class ExportConfiguration 			
implements IWorkbenchWindowActionDelegate, 
		ConfigurationModificationListener,
		ConfigurationOpenedClosedListener {

	private Shell shell; 
	private IAction action;
	
	public void dispose() {
		Activator.getDefault().getHandler().removeConfigurationModificationListener(this);
		Activator.getDefault().getHandler().removeOpenCloseListener(this);
	}

	public void init(IWorkbenchWindow window) {
		shell = Activator.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell();
		Activator.getDefault().getHandler().addConfigurationModificationListener(this);
		Activator.getDefault().getHandler().addOpenCloseListener(this);
	}

	public void run(IAction action) {
		this.action = action;
		ExportConfigurationDialog dlg = new ExportConfigurationDialog(shell);
		dlg.open();
	}

	public void selectionChanged(IAction action, ISelection selection) {
	    if (this.action == null) {
	        this.action = action;
	        action.setEnabled(false);
        }
        // This is done to make the action disabled on startup when there's 
		// no model open
//		if (this.action == null) {
//			if (KumbangConfiguratorPlugin.getDefault().isServerAvailable()) {
//				if (!KumbangConfiguratorPlugin.getDefault().getClientServices().isModelInitialized()) {
//					action.setEnabled(false);
//				}
//			} else {
//				action.setEnabled(false);
//			}
//		}
//			this.action = action;
	}

	public void configurationModified(ConfigurationModifiedEvent e) {
		if (Activator.getDefault().getHandler().getClientServices().isConfigurationStateComplete()) {
			if (this.action != null)
				this.action.setEnabled(true);
		} else {
			if (this.action != null)
				this.action.setEnabled(false);
		}
	}

	public void handleOpenCloseEvent(ConfigurationOpenedClosedEvent e) {
		if (e.getEventType().equals(ConfigurationOpenedClosedEvent.EVENT_INITIALIZE) ||
				e.getEventType().equals(ConfigurationOpenedClosedEvent.EVENT_SAVEOPENED)) {
			if (Activator.getDefault().getHandler().getClientServices().isConfigurationStateComplete()) {
				if (this.action != null) {
					this.action.setEnabled(true);
				}
			}
		} else if (e.getEventType().equals(ConfigurationOpenedClosedEvent.EVENT_CLOSE)) {
			if (this.action != null) {
				this.action.setEnabled(false);
			}
		}
		
	}

}
