/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.actions;

import java.io.File;

import kumbang.configurator.Activator;
import kumbang.configurator.client.core.exception.ClientException;
import kumbang.configurator.client.core.exception.OpenException;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.PluginHandler;
import kumbang.configurator.client.plugin.event.ConfigurationOpenedClosedEvent;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

public class OpenConfiguration implements IWorkbenchWindowActionDelegate {

	private Shell shell;
	
	public void dispose() {
		// TODO Auto-generated method stub
	}

	public void init(IWorkbenchWindow window) {
		shell = Activator.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell();

	}

	public void run(IAction action) {
		
        if (!Activator.getDefault().getHandler().isServerAvailable()) {
            MessageDialog.openError(shell, "Server not available", "You cannot open saved model, if server is not available.");
        }
        
		if (Activator.getDefault().getHandler().getClientServices().isModelInitialized()) {
			if(!MessageDialog.openQuestion(shell, "Are you sure?", "You have an existing configuration open, unsaved changes will be lost if you open another one."))
				return;
		}
		
		String path = PluginHandler.getActivator()
		.getPluginDir("kumbang.configurator.client.plugin")
		+ "build/testmodels/";

		FileDialog dlg = new FileDialog(shell, SWT.OPEN);
		dlg.setFilterPath(path);
		dlg.setFilterExtensions(new String[] { "*" + SaveConfiguration.SAVE_EXTENSION, "*.*" });
		dlg.setText("Select a saved configuration to open");
		
		// Get the file
		String filename = dlg.open();
		if (filename != null) {
			File loadFile = new File(filename);
			try {
				Activator.getDefault().getHandler().getClientServices().closeConfiguration();
				Activator.getDefault().getHandler().fireOpenCloseEvent(ConfigurationOpenedClosedEvent.EVENT_CLOSE);
				Activator.getDefault().getHandler().getClientServices().openConfiguration(loadFile);
				Activator.getDefault().getHandler().fireOpenCloseEvent(ConfigurationOpenedClosedEvent.EVENT_SAVEOPENED);
			} catch (OpenException e) {
				MessageDialog.openError(shell, "Error opening a file", "Error opening the file " + filename);
				Activator.getDefault().getHandler().getClientServices().closeConfiguration();
				Activator.getDefault().getHandler().fireOpenCloseEvent(ConfigurationOpenedClosedEvent.EVENT_CLOSE);
				//e.printStackTrace();
			} catch (ClientException e) {
				// Grab the model name from the exception message
				String message = e.getMessage();
				String modelName = "";
				String dialogMessage ="";
				if (e.getMessage().lastIndexOf("named") != -1) {
					modelName = message.substring(message.lastIndexOf("named") + 5);
					dialogMessage = "The model used in the saved configuration is probably not on" + 
					" the server, please try importing it first: " + modelName;
				} else {
					dialogMessage = e.getMessage();
				}
				MessageDialog.openError(shell, "Error opening a saved configuration", 
						dialogMessage);
				Activator.getDefault().getHandler().getClientServices().closeConfiguration();
				Activator.getDefault().getHandler().fireOpenCloseEvent(ConfigurationOpenedClosedEvent.EVENT_CLOSE);
			}
		} else {
			return; // user canceled?
		}
	}

	public void selectionChanged(IAction action, ISelection selection) {
		// TODO Auto-generated method stub
	}
}
