/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.actions;

import java.io.File;

import kumbang.configurator.Activator;
import kumbang.configurator.client.core.exception.SaveException;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.event.ConfigurationOpenedClosedEvent;
import kumbang.configurator.client.plugin.event.ConfigurationOpenedClosedListener;
import kumbang.configurator.client.plugin.preferences.PreferenceConstants;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

public class SaveConfiguration extends Action implements 
IWorkbenchWindowActionDelegate, ConfigurationOpenedClosedListener

{
	public static final String SAVE_EXTENSION = ".sav";
	private Shell shell;
	private IAction action;
	
	public void dispose() {
		Activator.getDefault().getHandler().removeOpenCloseListener(this);

	}

	public void init(IWorkbenchWindow window) {
		shell = Activator.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell();
		Activator.getDefault().getHandler().addOpenCloseListener(this);
		if (this.action != null)
			action.setEnabled(false);
//		this.action = getAction("kumbang.configurator.client.plugin.actions.SaveConfiguration");
//		service.unregisterAction(this.action);

	}

	public void run(IAction action) {
		this.action = action;
		
		if (!Activator.getDefault().getHandler().getClientServices().isModelInitialized()) {
			MessageDialog.openError(shell, "Cannot save", "No configuration opened");
			action.setEnabled(false);
			return;
		}
		
		 
		String path = "";
		FileDialog dlg = new FileDialog(shell, SWT.SAVE);
		
		/* If there's an existing save of this configuration, suggest its name */
		String initialSaveName = "";
		if (Activator.getDefault().getHandler().getClientServices().getSaveFile() != null) {
			path = Activator.getDefault().getHandler().getClientServices().getSaveFile().getAbsolutePath();
			initialSaveName = Activator.getDefault().getHandler().getClientServices().getSaveFile().getName(); 
		} else {
			initialSaveName = Activator.getDefault().getHandler().getClientServices().getConfigurationName() + SaveConfiguration.SAVE_EXTENSION;
			path = Activator.getDefault().getPreferenceStore().getString(PreferenceConstants.P_SAVEDIRECTORY);
		}

		dlg.setFileName(initialSaveName);
		dlg.setFilterPath(path);
		dlg.setFilterExtensions(new String[] { "*" + SaveConfiguration.SAVE_EXTENSION, "*.*" });
		dlg.setText("Where do you want to save the configuration");

//		Get the file
		String filename = "";
		boolean done = false;
		File saveFile = null;
		
		while (!done) {
			filename = dlg.open();
			if (filename == null) {
				// User canceled
				done = true;
			} else {
				// Check if the selected file exists
				saveFile = new File(filename);
				if (saveFile.exists()) {
					// Need confirmation
					done = MessageDialog.openConfirm(shell, "File already exists", "The file " + filename + " already " +
					"exists. Are you sure you want to overwrite it?");
				} else {
					// File doesn't exist
					done = true;
				}
			}
		}

		
		if (saveFile != null) {
			//File saveFile = new File(filename);
			try {
				System.out.println("Saving to " + filename);
				Activator.getDefault().getHandler().getClientServices().saveConfiguration(saveFile);
				Activator.getDefault().getHandler().fireSaveEvent(saveFile);
			} catch (SaveException e) {
				MessageDialog.openError(shell, "Error when saving", "Something went wrong when saving: " + e.getMessage());
				e.printStackTrace();
			}
		}
//		} else {
//			return; // user canceled?
//		}
	}

	public void selectionChanged(IAction action, ISelection selection) {
	    if (this.action == null) {
	        this.action = action;
	        action.setEnabled(false);
        }
        
        // This is done to make the action disabled on startup when there's 
		// no model open
//		if (this.action == null) {
//		    if (KumbangConfiguratorPlugin.getDefault().isServerAvailable()) {
//		        if (!KumbangConfiguratorPlugin.getDefault().getClientServices().isModelInitialized()) {
//		            action.setEnabled(false);
//		        }
//		    } else {
//		    	action.setEnabled(false);
//		    }
//		}
//		this.action = action;
	}

	public void handleOpenCloseEvent(ConfigurationOpenedClosedEvent e) {
		//System.out.println("SaveConfiguration caught an event");
		if (e.getEventType() == ConfigurationOpenedClosedEvent.EVENT_CLOSE) {
			this.action.setEnabled(false);
		} else {
			this.action.setEnabled(true);
		}
	}
}
