/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.diagrams;

import java.util.Iterator;
import java.util.ListIterator;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;

import kumbang.configurator.Activator;
import kumbang.configurator.client.plugin.Colors;
import kumbang.configurator.client.plugin.IKumbangPlugin;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.diagrams.ComponentDiagramLevel;
import kumbang.configurator.client.plugin.diagrams.ComponentElement;
import kumbang.configurator.client.plugin.diagrams.DiagramElement;
import kumbang.configurator.client.plugin.diagrams.DiagramFilter;
import kumbang.configurator.client.plugin.diagrams.FilteredElement;


public class ComponentDiagram extends Figure {

	private ComponentDiagramLevel level;

	private DiagramFilter filter;

	/**
	 * Creates a new instance of ComponentDiagram
	 */
	public ComponentDiagram() {
		this.filter = new DiagramFilter();
		setOpaque(true);
		setBackgroundColor(getPlugin()
				.getColorRegistry().get(Colors.COMPONENT));
	}

	public void setDiagramLevel(ComponentDiagramLevel level) {
		this.level = level;
		updateSize();
	}

	public void showNames(boolean show) {
		filter.showNames(show);
	}

	public void showTypes(boolean show) {
		filter.showTypes(show);
	}

	public void showConnections(boolean show) {
		filter.showConnections(show);
	}

	public void showPotential(boolean show) {
		filter.showPotential(show);
	}

	public void paint(Graphics g) {
		g.setBackgroundColor(getPlugin()
				.getColorRegistry().get(Colors.COMPONENT));
		g.fillRectangle(0, 0, this.bounds.width, this.bounds.width);
		if (level == null) {
			return;
		}
		ComponentElement root = level.getRootComponentElement();
		if (root != null) {
			root.paint(g, filter);
		}
		Iterator it = level.getChildElements();
		while (it.hasNext()) {
			DiagramElement e = (DiagramElement) it.next();
			if (e instanceof FilteredElement) {
				((FilteredElement) e).paint(g, filter);
			} else {
				e.paint(g);
			}
		}
	}

	public DiagramElement getElementAt(int x, int y) {
		if (level == null) {
			return null;
		}
		DiagramElement root = level.getRootElement();
		if (root == null || !root.isInside(x, y)) {
			return null;
		}
		ListIterator it = level.getChildElementsFromEnd();
		while (it.hasPrevious()) {
			DiagramElement e = (DiagramElement) it.previous();
			if (e.isInside(x, y)) {
				return e;
			}
		}
		return root;
	}

	public Dimension getPreferredScrollableViewportSize() {
		return getPreferredSize();
	}

	public int getScrollableUnitIncrement(Rectangle visibleRect,
			int orientation, int direction) {
		return 2;
	}

	public int getScrollableBlockIncrement(Rectangle visibleRect,
			int orientation, int direction) {
		return 10;
	}

	public boolean getScrollableTracksViewportWidth() {
		return false;
	}

	public boolean getScrollableTracksViewportHeight() {
		return false;
	}

	public void updateSize() {
		if (level != null) {
			Dimension size = new Dimension(level.getPreferredWidth(), level
					.getPreferredHeight());
			setPreferredSize(size);
			setBounds(new Rectangle(0, 0, level.getPreferredWidth(), level
					.getPreferredHeight()));
		}
	}
	
	protected IKumbangPlugin getPlugin() {
		return Activator.getDefault();
	}
	
}
