/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.diagrams;

//import java.awt.Color;
//import java.awt.Graphics;
import org.eclipse.draw2d.Graphics;
import org.eclipse.swt.graphics.Color;

import kumbang.configurator.client.plugin.Icons;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.diagrams.BoxedElement;
import kumbang.configurator.client.plugin.diagrams.DiagramFilter;
import kumbang.configurator.client.plugin.diagrams.FilteredElement;
import kumbang.configurator.client.plugin.diagrams.InstantiatedElement;
import kumbang.core.configuration.instance.ComposableInstance;
import kumbang.core.configuration.instance.Instance;

public abstract class ComposableElement extends BoxedElement implements InstantiatedElement, FilteredElement {
    
    private ComposableInstance instance;
    
    private boolean isRoot;
    
    private boolean isNavigable;
    
    public static final int DEFAULT_WIDTH = 80;
    public static final int DEFAULT_HEIGHT = 50;
    
    public static final int MARGIN_ICONS = 3;
    

    
    /** Creates a new instance of ComposableElement */
    public ComposableElement(ComposableInstance instance, Color background) {
        this(instance, background, false);
    }
    
    /** Creates a new instance of ComposableElement */
    public ComposableElement(ComposableInstance instance, Color background, 
                             boolean isRoot) {
    	super(instance.getName(), DEFAULT_WIDTH, DEFAULT_HEIGHT, background);
        this.instance = instance;
        this.isRoot = isRoot;
        this.isNavigable = instance.hasParts() && !isRoot;
    }
    
    /**
     * Returns the elementary instance this element represents
     * @return ElementaryInstance 
     */
    public Instance getInstance() {
        return this.instance;
    }
    
    public ComposableInstance getComposableInstance() {
        return this.instance;
    }
    
    public boolean isRootElement() {
        return isRoot;
    }
    
    public boolean isNavigable() {
        return isNavigable;
    }
    
    public boolean areAttributesEditable() {
        return instance.hasAttributes();
    }
    
    public void paint(Graphics g, DiagramFilter filter) {
        boolean showsName = ((filter != null) ? filter.showsNames() : true);
        boolean showsType = ((filter != null) ? filter.showsTypes() : true);
        super.paint(g);
        
        g.drawString(getDescription(showsName, showsType), 
                getLocationX() - super.getWidth() / 2 + 2, 
                getLocationY() - super.getHeight() / 2 - 14);
        int iconX = getLocationX() + super.getWidth() / 2 - MARGIN_ICONS
                - Icons.ICON_SIZE;
        int iconY = getLocationY() + super.getHeight() / 2 - MARGIN_ICONS
                - Icons.ICON_SIZE;
        if (isNavigable && !isRoot) {
        	g.drawImage(getPlugin().getImageRegistry().get(Icons.ZOOM), iconX, iconY);
            iconX -= (MARGIN_ICONS + Icons.ICON_SIZE);
        }
        if (instance.hasAttributes()) {
        	g.drawImage(getPlugin().getImageRegistry().get(Icons.ATTRIBUTE), iconX, iconY);
        }
    }
    
    public void paint(Graphics g) {
        this.paint(g, null);
    }
    
    public boolean equals(Object o) {
        return o instanceof ComposableElement
            && ((ComposableElement) o).getComposableInstance().equals(instance);
    }
    
    public String toString() {
        return instance.getDisplayName() + ":" 
                + instance.getType().getDisplayName();
    }
    
    private String getDescription(boolean name, boolean type) {
        String ret = "";
        if (!name && !type) {
            return ret;
        }
        if (name) {
            ret += instance.getDisplayNameWithIndex();
        }
        ret += ":";
        if (type) {
            ret += instance.getType().getDisplayName();
        }
        return ret;
    }
}
