/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.dialogs;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import kumbang.configurator.Activator;
import kumbang.configurator.client.core.TaskHandler;
import kumbang.configurator.client.core.exception.ClientException;
import kumbang.configurator.client.plugin.IKumbangPlugin;
import kumbang.configurator.client.plugin.Icons;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.tree.TreeNode;
import kumbang.core.configuration.core.ConfigurationException;
import kumbang.core.configuration.instance.AttributeInstance;
import kumbang.core.configuration.instance.ComposableInstance;
import kumbang.core.configuration.instance.AttributedInstance;
import kumbang.core.configuration.task.EditAttributesTask;
import kumbang.core.model.type.Name;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

public class EditAttributesDialog extends TitleAreaDialog {

	private TreeNode selectedNode;

	private AttributedInstance instance;

	private List selectionItems;

	/**
	 * The Constructor.
	 * 
	 * @param parentShell parent shell
	 * @param node selected tree-node
	 */
	public EditAttributesDialog(Shell parentShell, TreeNode node) {
		super(parentShell);
		this.selectedNode = node;
		this.instance = (AttributedInstance) selectedNode.getInstance();
		this.selectionItems = new LinkedList();
	}

	/*
	 * TODO This constructor looks too much like the one above. -toasikai 31.7.2006
	 */
	public EditAttributesDialog(Shell parentShell, AttributedInstance instance) {
		super(parentShell);
		this.selectedNode = null;
		this.instance = instance;
		this.selectionItems = new LinkedList();
	}

	/**
	 * Closes the dialog
	 */
	public boolean close() {
		return super.close();
	}

	/**
	 * Creates the dialog's contents
	 * 
	 * @param parent the parent composite
	 * @return Control
	 */
	protected Control createContents(Composite parent) {
		Control contents = super.createContents(parent);

		if (selectedNode != null) {
			setTitle("Edit attributes");
			setMessage("Edit attributes of " + selectedNode.getInstance().getInstanceKind() + " "
					+ selectedNode.getInstance() + " by selecting values below.",
					IMessageProvider.INFORMATION);
		} else {
			setTitle(instance.toString());
			setMessage("Edit attributes of " + instance.getAsTypedInstance().getInstanceKind() + " " + instance
					+ " by selecting values below.", IMessageProvider.INFORMATION);
		}

		return contents;
	}

	/**
	 * Creates the gray area
	 * 
	 * @param parent the parent composite
	 * @return Control composite
	 */
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite) super.createDialogArea(parent);

		Composite innerComposite = new Composite(composite, SWT.NONE);
		innerComposite.setLayout(DialogLayout.innerCompositeLayout());
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		innerComposite.setLayoutData(gridData);

		// Shows feature information
		getInfoComposite((ComposableInstance) instance, innerComposite);

		// Shows the logo
		Label imageLabel = new Label(innerComposite, SWT.BORDER);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.verticalSpan = 2;
		gridData.verticalAlignment = SWT.TOP;
		gridData.horizontalAlignment = SWT.RIGHT;
		imageLabel.setLayoutData(gridData);
		imageLabel.setImage(getPlugin().getImageRegistry().get(Icons.LOGO));

		// Shows attribute-comboboxes of the selected feature
		Group group = new Group(innerComposite, SWT.NONE);
		group.setText("Attributes");
		GridLayout glayout = new GridLayout(2, false);
		glayout.marginWidth = 10;
		group.setLayout(glayout);
		Iterator it = ((ComposableInstance) instance).getAttributes();
		while (it.hasNext()) {
			AttributeInstance attr = (AttributeInstance) it.next();
			AttributeSelectionItem item = new AttributeSelectionItem(attr, group);
			selectionItems.add(item);
		}

		return composite;
	}

	/**
	 * Shows the information of current instance (feature of component)
	 * 
	 * @param instance composable instance
	 * @param parent parent composite
	 */
	public void getInfoComposite(ComposableInstance instance, Composite parent) {

		Group group = new Group(parent, SWT.SHADOW_ETCHED_IN);
		group.setText("Info");
		GridLayout gl = new GridLayout(2, false);
		gl.marginWidth = DialogLayout.MARGIN_INNER;
		group.setLayout(gl);

		if (instance == null) {
			return;
		} else if (instance.getParent() != null) {
			new Label(group, SWT.NONE).setText("Name:");
			new Label(group, SWT.NONE).setText(instance.getDisplayName());

			new Label(group, SWT.NONE).setText("Type:");
			new Label(group, SWT.NONE).setText(instance.getType().getDisplayName());

			new Label(group, SWT.NONE).setText("Parent name:");
			new Label(group, SWT.NONE).setText(instance.getParent().getDisplayName());

			new Label(group, SWT.NONE).setText("Parent type:");
			new Label(group, SWT.NONE).setText(instance.getParent().getType().getDisplayName());

			return;

		} else {
			new Label(group, SWT.NONE).setText("Name:");
			new Label(group, SWT.NONE).setText(instance.getDisplayName());

			new Label(group, SWT.NONE).setText("Type:");
			new Label(group, SWT.NONE).setText(instance.getType().getDisplayName());

			new Label(group, SWT.NONE).setText("Parent name:");
			new Label(group, SWT.NONE).setText("");

			new Label(group, SWT.NONE).setText("Parent type:");
			new Label(group, SWT.NONE).setText("");

			return;
		}
	}

	/**
	 * Creates the buttons.
	 * 
	 * @param parent the parent composite
	 */
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
		createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
	}

	/**
	 * Handles a button press.
	 * 
	 * @param buttonId the ID of the pressed button
	 */
	protected void buttonPressed(int buttonId) {

		if (buttonId == IDialogConstants.OK_ID) {

			EditAttributesTask task = getTask(getPlugin().getHandler().getClientServices().getTaskHandler());
			if (task == null) {
				return;
			}
			
			try {
				getPlugin().getHandler().fireConfigurationModifiedEvent(getPlugin().getHandler().getClientServices().handleTask(task));
				close();
			} catch (ClientException e) {
				MessageBox mbox1 = new MessageBox(this.getShell(), SWT.ICON_WARNING);
				mbox1.setText("Error when editing attributes");
				mbox1.setMessage(e.getMessage());
				mbox1.open();
			} catch (ConfigurationException e) {
				MessageBox mbox1 = new MessageBox(this.getShell(), SWT.ICON_WARNING);
				mbox1.setText("Error when editing attributes");
				mbox1.setMessage("Setting these values would make configuration inconsistent.\n"
						+ "No changes has been made.");
				mbox1.open();
			}

		} else {
			// Cancel-button has been pressed
			close();
		}
	}

	/**
	 * Sets the title of Edit Attributes -window
	 * 
	 */
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("Edit Attributes");
	}

	/**
	 * Sets a certain TreeNode as selected
	 * 
	 * @param node selected node from the tree
	 */
	public void setSelectedTreeNode(TreeNode node) {
		this.selectedNode = node;
	}

	/**
	 * Creates and returns the task
	 * 
	 * @param taskHandler
	 * @return task
	 */
	public EditAttributesTask getTask(TaskHandler taskHandler) {
		EditAttributesTask task = null;

		task = new EditAttributesTask(this.instance, true);
		Iterator it = selectionItems.iterator();
		while (it.hasNext()) {
			AttributeSelectionItem asi = (AttributeSelectionItem) it.next();
			AttributeInstance attribute = asi.getAttribute();

			/*
			 * The index of the selected item is the best method. Using the string appearing
			 * in the combo box as such does not work, as it is the display value. A mapping
			 * between the display value and the actual value does not work should to values have
			 * the same display value. -toasikai 13.9.2006
			 * 
			 * TODO The index should be used for other combo boxes as well.
			 */
			String value = attribute.getType().getValue(asi.combo.getSelectionIndex());
			if (!value.equals(attribute.getValue())) {
				task.setAttribute(attribute.getName(), value);
			}
		}
		return task;
	}

	protected IKumbangPlugin getPlugin() {
		return Activator.getDefault();
	}

	/**
	 * This class represents the comboboxes of different attributes.
	 * 
	 */
	class AttributeSelectionItem {

		// private boolean excluded;

		// combo box for selecting attribute value
		private Combo combo;

		// label for showing attribute name
		private Label label;

		// attribute instance
		private AttributeInstance attribute;

		/**
		 * Creates comboboxes based on the given attribute definition
		 * 
		 * @param attribute attribute instance
		 * @param composite parent composite
		 */
		public AttributeSelectionItem(AttributeInstance attribute, Composite composite) {
			this.attribute = attribute;
			this.label = new Label(composite, SWT.NONE);
			// this.valueMap = new HashMap<String, String>();

			label.setText(attribute.getName());

			this.combo = new Combo(composite, SWT.READ_ONLY);

			Iterator it = attribute.getPossibleValues();
			int i = 0;
			while (it.hasNext()) {
				Name value = (Name) it.next();

				this.combo.add(value.getDisplayName());
				if (attribute.isCurrentValue(value)) {
					this.combo.select(i);
				}

				i++;
			}

			if (this.combo.getText().equals(""))
				this.combo.select(0);
		}

		/**
		 * Returns the attribute definition
		 * 
		 * @param AttributeDefinition
		 */
		public AttributeInstance getAttribute() {
			return this.attribute;
		}
	}
}
