/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.dialogs;

import java.io.File;

import kumbang.configurator.client.core.ClientProperties;
import kumbang.configurator.client.core.exception.ExportException;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.PluginHandler;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class ExportConfigurationDialog extends TitleAreaDialog {

	private Text filename;
	private Text directory;
	private Button btnXML;
	private Button btnKBE;
	
	public ExportConfigurationDialog(Shell parentShell) {
		super(parentShell);
		// TODO Auto-generated constructor stub
	}

	/**
	 * Closes the dialog
	 */
	public boolean close() {
		return super.close();
	}
	
	// Create the title area
	protected Control createContents(Composite parent) {
		Control contents = super.createContents(parent);
		setTitle("Export Configuration Description");
		setMessage("This exports a description of the current " +
				"configuration to a file in your filesystem");
		return contents;
	}
	
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite) super.createDialogArea(parent);
		
		Composite innerComposite = new Composite(composite, SWT.NONE);
		innerComposite.setLayout(DialogLayout.exportCompositeLayout());
		
		
		
		Label lbl = new Label(innerComposite, SWT.NONE);
		lbl.setText("File name, without extension:");
		
		this.filename = new Text(innerComposite, SWT.SINGLE | SWT.BORDER);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		this.filename.setLayoutData(gd);
		
		
		this.filename.setText(PluginHandler.getInstance().getClientServices().getConfigurationName());
		lbl = new Label(innerComposite, SWT.NONE);
		lbl.setText("Directory:");
		
		this.directory = new Text(innerComposite, SWT.SINGLE | SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		this.directory.setLayoutData(gd);
		
		Button btn = new Button(innerComposite, SWT.PUSH);
		btn.setText("Browse..");
		btn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				String path;
				if (directory.getText().equals(""))
					path = PluginHandler.getActivator().getPluginDir(PluginHandler.getActivator().getPluginID()); 
				else
					path = directory.getText();
				
				DirectoryDialog dlg = new DirectoryDialog(ExportConfigurationDialog.this.getShell());
				dlg.setFilterPath(path);
				dlg.setText("Export");
				dlg.setMessage("Select a directory");
				String dir = dlg.open();
				if (dir != null) {
					ExportConfigurationDialog.this.directory.setText(dir);
				}
			}
		});
		
		

		
		
		
		
		gd = new GridData();
		gd.horizontalSpan = 2;
		
		Group exportModeGroup = new Group(innerComposite, SWT.SHADOW_ETCHED_IN);
		exportModeGroup.setLayoutData(gd);
		exportModeGroup.setText("Export format");
		exportModeGroup.setLayout(new FillLayout());
		btnXML = new Button(exportModeGroup, SWT.RADIO);
		btnXML.setText("XML");
		btnXML.setSelection(true);
		btnKBE = new Button(exportModeGroup, SWT.RADIO);
		btnKBE.setText(".kbe");
		return composite;
	}
	
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, "Export",
				true);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}
	
	protected void buttonPressed(int buttonId) {
		
		if (buttonId == IDialogConstants.OK_ID) {

			if (filename.getText().equals("")) {
				MessageDialog.openError(this.getShell(), "Empty filename", "Please specify a file name");
				return; 
			}
			
			File dir = new File(directory.getText());
			if (!dir.isDirectory()) {
				MessageDialog.openError(this.getShell(), "Invalid directory", "The specified directory\n '" + directory.getText() + "'\nis invalid or doesn't exists. Please enter a proper existing directory.");
				return;
			}
			
			String completeFileName = directory.getText() + File.separator + filename.getText() + getFileExtension();
			
			File exportFile = new File(completeFileName);
			
			if (exportFile.exists()) {
				// User clicks No -> just return to the dialog
				if(!MessageDialog.openConfirm(this.getShell(), "File alreadyexists", "There's already a file " + completeFileName + ". Do you want to overwrite it?")) {
					return;
				}
			}
			try {
				//System.out.println(completeFileName);
				PluginHandler.getInstance().getClientServices().exportConfiguration(new File(completeFileName), getExportMode());
				MessageDialog.openInformation(this.getShell(), "Export succesful", "Exported configuration to " + completeFileName);
			} catch (ExportException e) {
				MessageDialog.openError(this.getShell(), "Export error", "Error when trying to export to " + completeFileName);
				e.printStackTrace();
			}
			finally {
				close();
			}
			
		} else {
			// Cancel-button has been pressed
			close();
		}
	}

	
	private int getExportMode() {
		if (btnXML.getSelection())
			return ClientProperties.EXPORT_FORMAT_XML;
		else
			return ClientProperties.EXPORT_FORMAT_TEXT;
	}
	
	private String getFileExtension() {
		if (getExportMode() == ClientProperties.EXPORT_FORMAT_XML)
			return ".xml";
		else
			return ".kbe";
	}
	
}
