/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.dialogs;

import java.util.Iterator;

import kumbang.configurator.client.plugin.tree.TreeNode;
import kumbang.core.configuration.instance.ComposableInstance;
import kumbang.core.configuration.instance.PartInstance;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

public class SelectPartDialog extends TitleAreaDialog {

	private Shell parent;
	private TreeNode selectedNode;
	private ComposableInstance instance;
	private PartSelectionItem selectionItem;
	
	private PartInstance selectedPart;

	public SelectPartDialog(Shell parentShell, TreeNode node) {
		super(parentShell);
		this.parent = parentShell;
		this.selectedNode = node;
		this.instance = (ComposableInstance)node.getInstance();
	}
	
	
	/**
	 * Closes the dialog
	 */
	public boolean close() {
		return super.close();
	}
	
	protected Control createContents(Composite parent) {
		Control contents = super.createContents(parent);

		setTitle("Select part");
		setMessage("Select the part under which the feature can be added.", IMessageProvider.INFORMATION);

		return contents;
	}
	
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite) super.createDialogArea(parent);
		// InnerComposite must be used for the layouts
		Composite innerComposite = new Composite(composite, SWT.NONE);
		innerComposite.setLayout(DialogLayout.innerCompositeLayout());
		GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.FILL;
		innerComposite.setLayoutData(gridData);
		// Position this at the top
		Label lbl = new Label(innerComposite, SWT.NONE);
		lbl.setText("Before you can add a new " + instance.getInstanceKind() +
				" under " + instance + ",\n" + " you must select the part under" +
				" which the " + instance.getInstanceKind() + " can be added.\n" +
				"Please select a part from the following list.");
				
		GridData gd = new GridData();
		gd.horizontalSpan = 2;
		lbl.setLayoutData(gd);
		// Next stuff will go in two columns
		
		// Create the info group with the helper method..
		Group grp = DialogHelper.getInfoComponent(innerComposite, this.instance);
		gd = new GridData();
		gd.horizontalSpan = 2;
		grp.setLayoutData(gd);
		
		
		selectionItem = new PartSelectionItem(this.instance, innerComposite);
		
		return composite;
	}


	protected void createButtonsForButtonBar(Composite parent) {
		
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				true);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}
	
	/**
	 * Handles a button press.
	 * 
	 * @param buttonId
	 *            the ID of the pressed button
	 */
	protected void buttonPressed(int buttonId) {
		
		if (buttonId == IDialogConstants.OK_ID) {
//			close();
			this.selectedPart = getPart();
			
		} else {
			// Cancel-button has been pressed
			this.selectedPart = null;
		}
		
		if (this.selectedPart != null && this.selectedPart.isFull()) {
			MessageBox msgbox = new MessageBox(this.parent, SWT.ICON_ERROR | SWT.OK);
			msgbox.setText("Error");
			msgbox.setMessage("You have selected a part that has already\n" +
					" the maximum number of " + this.selectedPart.getComposedKind() + "s.\n" +
					"Thus you cannot add a new " + this.selectedPart.getComposedKind() + " to this part.\n" +
			"Please select another part.");
			msgbox.open();
			return;
		}
		super.buttonPressed(buttonId);
	}
	
	
	/**
	 * Sets the title of Edit Attributes -window
	 * 
	 */
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("Select part");
	}
	
	/**
	 * Sets a certain TreeNode as selected
	 * 
	 * @param node
	 *            selected node from the tree
	 */
	public void setSelectedTreeNode(TreeNode node) {
		this.selectedNode = node;
	}


	class PartSelectionItem {
		// combo box for selecting part
		private Combo combo;

		// label for showing part name
		private Label label;

		// part instance
		private ComposableInstance instance;
		
		/**
		 * Creates comboboxes based on the given attribute definition
		 * 
		 * @param attribute
		 *            attribute instance
		 * @param composite
		 *            parent composite
		 */
		public PartSelectionItem(ComposableInstance instance,
				Composite composite) {
						
			this.instance = instance;
					
			this.label = new Label(composite, SWT.NONE);
			label.setText("Select a part");
			GridData gd = new GridData();
			label.setLayoutData(gd);
			this.combo = new Combo(composite, SWT.READ_ONLY);
			gd = new GridData();
			gd.grabExcessHorizontalSpace = true;
			gd.horizontalAlignment = SWT.BEGINNING;
			this.combo.setLayoutData(gd);
			Iterator it = instance.getParts();
//			int i = 0;
			while (it.hasNext()) {
				//Name value = (Name) it.next();
				PartInstance p = (PartInstance)it.next();
				// AttributeComboItem item = new AttributeComboItem(value,
				// cnsqHandler.isExcludedAttributeValue(((ComposableInstance)
				// instance).getID(),
				// attribute.getName(), value));
				
				
				this.combo.add(p.getName());
//				if (attribute.isCurrentValue(value)) {
//				this.combo.select(i);
//				}
//				i++;
			}
			
			if (this.combo.getText().equals("")) {
				this.combo.select(0);
			}
		}
	}
	
	/**
	 * Fetches the selected part
	 * @return
	 */
	private PartInstance getPart() {
        int selection = this.selectionItem.combo.getSelectionIndex();
        Iterator it = instance.getParts();
        while (it.hasNext()) {
			PartInstance part = (PartInstance) it.next();
			if (part.getName().equals(selectionItem.combo.getItems()[selection])) 
				return part;
		}
        return null;
		
    }
	
	public PartInstance getSelectedPart() {
		return selectedPart;
	}
	
}



