/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.perspective;

import kumbang.configurator.Constants;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;

import org.eclipse.ui.IFolderLayout;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IPerspectiveFactory;

/**
 * Defines the Kumbang Perspective
 * @author jlemmett
 *
 */
public class KumbangPerspective implements IPerspectiveFactory {

	public void createInitialLayout(IPageLayout layout) {
		defineActions(layout);
		defineLayout(layout);
	}

    private void defineActions(IPageLayout layout) {
        // Add "new wizards"
        layout.addNewWizardShortcut("KumbangTools.InitializeWizard");

        // Add "show views"
        layout.addShowViewShortcut(Constants.FEATUREVIEW_ID);  
        layout.addShowViewShortcut(Constants.COMPONENTVIEW_ID);
        layout.addShowViewShortcut(Constants.STATUSVIEW_ID);  
        layout.addShowViewShortcut(Constants.DIAGRAMVIEW_ID); 
        layout.addShowViewShortcut(IPageLayout.ID_PROP_SHEET);          
        layout.addActionSet("kumbang.configurator.client.plugin.actionSet");
        
        
    }
    
	private void defineLayout(IPageLayout layout) {
		// Editor is already in place
		String editorArea = layout.getEditorArea();
		layout.setEditorAreaVisible(false);
		// Place Component and Feature views to left of the editor area
		// The float specifies that the folder will occupy 26% of the original space
		// of the editor area
		
		IFolderLayout topLeft;
		IFolderLayout bottomLeft;
		IFolderLayout right;
		IFolderLayout bottom;
		
		topLeft = layout.createFolder("topLeft", IPageLayout.LEFT, 0.34f, editorArea);
		topLeft.addView(Constants.FEATUREVIEW_ID);
		topLeft.addView(Constants.COMPONENTVIEW_ID);
		
		bottomLeft = layout.createFolder("bottomLeft", IPageLayout.BOTTOM, 0.75f, "topLeft");
		bottomLeft.addView(Constants.STATUSVIEW_ID); // Status view
		
		right = layout.createFolder("right", IPageLayout.RIGHT, 0.75f, editorArea);
		right.addView(Constants.DIAGRAMVIEW_ID);
		
		bottom = layout.createFolder("bottom", IPageLayout.BOTTOM, 0.75f, "right");
		bottom.addView(IPageLayout.ID_PROP_SHEET);
	}



}


