/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.preferences;

import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.PluginHandler;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.DirectoryFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.RadioGroupFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * This class represents a preference page that
 * is contributed to the Preferences dialog. By 
 * subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows
 * us to create a page that is small and knows how to 
 * save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They
 * are stored in the preference store that belongs to
 * the main plug-in class. That way, preferences can
 * be accessed directly via the preference store.
 */

public class PreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {
    
    private RadioGroupFieldEditor comMethod;
    private StringFieldEditor serverAddress;
    private StringFieldEditor serverPort;
    private DirectoryFieldEditor saveDirectory;
    private BooleanFieldEditor umlInterfaces;
    private BooleanFieldEditor completenessPopups;


    
	public PreferencePage() {
		super(GRID);
		setPreferenceStore(PluginHandler.getActivator().getPreferenceStore());
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {

        
        comMethod = new RadioGroupFieldEditor(
                PreferenceConstants.P_COMMUNICATIONMETHOD,
                "Select the client-server communication method (changing this will kill an existing connection):",
                1, new String[][] { { "Local", "local" }, { "RMI", "rmi" } },
                getFieldEditorParent());
        addField(comMethod);
        
        serverAddress = new StringFieldEditor(
                PreferenceConstants.P_SERVERADDRESS, "Server adress", 20,
                getFieldEditorParent());
        serverAddress.setEmptyStringAllowed(false);
        addField(serverAddress);
        
        
        serverPort = new StringFieldEditor(
                PreferenceConstants.P_SERVERPORT, "Port", 5,
                getFieldEditorParent());
        serverPort.setEmptyStringAllowed(false);
        serverPort.setTextLimit(5);
        addField(serverPort);
        
        saveDirectory = new DirectoryFieldEditor(PreferenceConstants.P_SAVEDIRECTORY, 
        		"Default save directory", getFieldEditorParent());
        addField(saveDirectory);
        
        umlInterfaces = new BooleanFieldEditor(PreferenceConstants.P_UML_INTERFACES,
        		"Use UML 2.0 symbols for interfaces in the tree", getFieldEditorParent());
        addField(umlInterfaces);
        
        completenessPopups = new BooleanFieldEditor(PreferenceConstants.P_COMPLETENESS_POPUPS,
                "Show pop-up windows when configuration state becomes complete", getFieldEditorParent());
        addField(completenessPopups);
        
        Control[] temp = comMethod.getRadioBoxControl(getFieldEditorParent()).getChildren();
        Button btn;
        for (int i = 0; i < temp.length; i++) {
            btn = (Button) temp[i];
            btn.addSelectionListener(new SelectionListener() {
                public void widgetSelected(SelectionEvent e) {
                    if (((Button)e.getSource()).getText().equals("Local")) {
                        serverAddress.setEnabled(false, getFieldEditorParent());
                        serverPort.setEnabled(false, getFieldEditorParent());
                    }
                    if (((Button)e.getSource()).getText().equals("RMI")) {
                        serverAddress.setEnabled(true, getFieldEditorParent());
                        serverPort.setEnabled(true, getFieldEditorParent());
                    }
                }
                public void widgetDefaultSelected(SelectionEvent e) {}
            });
        }
        
        
        if (PluginHandler.getActivator().getPreferenceStore().getString(PreferenceConstants.P_COMMUNICATIONMETHOD).equals("local")) {
            serverAddress.setEnabled(false, getFieldEditorParent());
            serverPort.setEnabled(false, getFieldEditorParent());
        } else {
            serverAddress.setEnabled(true, getFieldEditorParent());
            serverPort.setEnabled(true, getFieldEditorParent());
        }
        
        
		this.adjustGridLayout();
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

	
}
