/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.properties;


import java.util.Iterator;

import kumbang.core.configuration.instance.ComponentInstance;
import kumbang.core.configuration.instance.PartInstance;
import kumbang.core.model.type.AttributeDefinition;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;

public class ComponentPropertySource implements IPropertySource {

	private static final String NAME_ID = "component.name";
	private static final String TYPE_ID = "component.type";
	private static final String SUBCOMPONENTS_ID = "component.subs";
	private static final String INTERFACES_ID = "component.ifaces";
	private static final String ATTRIBUTES_ID = "component.attrs";
	
	private IPropertyDescriptor[] propertyDescriptors; 

	private ComponentInstance cInstance;
	
	public ComponentPropertySource(ComponentInstance instance) {
		super();
		this.cInstance = instance;
		
		// TODO Auto-generated constructor stub
	}

	
	public Object getEditableValue() {
		// TODO Auto-generated method stub
		return null;
	}

	public IPropertyDescriptor[] getPropertyDescriptors() {
		if (propertyDescriptors == null) {
			PropertyDescriptor nameDescriptor = new PropertyDescriptor(NAME_ID, "1. Component name");
			PropertyDescriptor typeDescriptor = new PropertyDescriptor(TYPE_ID, "2. Component type");
			PropertyDescriptor subDescriptor = new PropertyDescriptor(SUBCOMPONENTS_ID, "3. Subcomponents");
			PropertyDescriptor ifaceDescriptor = new PropertyDescriptor(INTERFACES_ID, "4. Interfaces");
			PropertyDescriptor attrDescriptor = new PropertyDescriptor(ATTRIBUTES_ID, "5. Attributes");
	
			propertyDescriptors = new IPropertyDescriptor[] {
					nameDescriptor,
					typeDescriptor,
					subDescriptor,
					ifaceDescriptor,
					attrDescriptor};
			
		}
		return propertyDescriptors;
	}

	public Object getPropertyValue(Object id) {
		if (NAME_ID.equals(id)) {
			return cInstance.getName();
		}
		
		if (TYPE_ID.equals(id)) {
			return cInstance.getType().getDisplayName();
		}
		
		if (SUBCOMPONENTS_ID.equals(id)) {
			String str = "";
			Iterator it = cInstance.getParts();
			while (it.hasNext()) {
				for (Iterator pit = ((PartInstance)it.next()).getInstances(); pit.hasNext(); ) {
					str += ((ComponentInstance) pit.next()).getDisplayName() + ", ";
				}
			}
			if (str.length() > 1)
				return str.substring(0, str.length()-2);
			else
				return str;
		}
		
		if (INTERFACES_ID.equals(id)) {
			String str = "";
			Iterator it = cInstance.getInterfaces();
			while (it.hasNext()) {
				//System.out.println("Starting gathering interface");
				Object element = (Object)it.next();
				//str += cInstance.getInterfaceOfDefinition(element).toString() + ", ";
				//cInstance.getComponentType().getSourceInterfaces()
				//str += cInstance.getComponentType().getInterface(element.getName()) + ", ";
				str += element + ", ";
				//str+= "Interfeis";
				//System.out.println("End gathering interface");
			}
			if (str.length() > 1)
				return str.substring(0, str.length() - 2);
			else
				return str;
		}
		
		if (ATTRIBUTES_ID.equals(id)) {
			String str = "";
			Iterator it = cInstance.getComponentType().getAttributes();
			while (it.hasNext()) {
				AttributeDefinition element = (AttributeDefinition) it.next();
				str += cInstance.getAttribute(element.toString()) + ", ";
			}
			if (str.length() > 1)
				return str.substring(0, str.length() - 2);
			else
				return str;
		}
		

		
		
		return null;
	}

	public boolean isPropertySet(Object id) {
		// TODO Auto-generated method stub
		return false;
	}

	public void resetPropertyValue(Object id) {
		// TODO Auto-generated method stub

	}

	public void setPropertyValue(Object id, Object value) {
		// TODO Auto-generated method stub

	}
//	
//	public Object getPropertyValue(Object id) {
//		
//
//	}
//
//	

}
