/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.properties;

import java.util.Iterator;

import kumbang.core.configuration.instance.AttributeInstance;
import kumbang.core.configuration.instance.FeatureInstance;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;

public class FeaturePropertySource implements IPropertySource {

	private static final String NAME_ID = "feature.name";
	private static final String TYPE_ID = "feature.type";
	private static final String SUBFEATURES_ID = "feature.subs";
	private static final String ATTRIBUTES_ID = "feature.attrs";
	
	private IPropertyDescriptor[] propertyDescriptors; 
	
	private FeatureInstance fInstance;
	
	public FeaturePropertySource(FeatureInstance instance) {
		super();
		this.fInstance = instance;
	}

	public Object getEditableValue() {
		// TODO Auto-generated method stub
		return null;
	}

	public IPropertyDescriptor[] getPropertyDescriptors() {
		if (propertyDescriptors == null) {
			PropertyDescriptor nameDescriptor = new PropertyDescriptor(NAME_ID, "1. Feature name");
			PropertyDescriptor typeDescriptor = new PropertyDescriptor(TYPE_ID, "2. Feature type");
			PropertyDescriptor subDescriptor = new PropertyDescriptor(SUBFEATURES_ID, "3. Subfeatures");
			PropertyDescriptor attrDescriptor = new PropertyDescriptor(ATTRIBUTES_ID, "4. Attributes");
		
			propertyDescriptors = new IPropertyDescriptor[] {
					nameDescriptor,
					typeDescriptor,
					subDescriptor,
					attrDescriptor};
		}
		return propertyDescriptors;
	}

	public Object getPropertyValue(Object id) {
		if (NAME_ID.equals(id)) {
			return fInstance.getName();
		}
		
		if (TYPE_ID.equals(id)) {
			return fInstance.getType().getName();
		}
		
		if (SUBFEATURES_ID.equals(id)) {
			String str = "";
			Iterator it = fInstance.getPartInstances();
			while (it.hasNext()) {
				Object element = it.next();
				if (element instanceof FeatureInstance) {
					str += element + ", ";
				}
			}
			if (str.length() > 1)
				return str.substring(0, str.length()-2);
			else
				return str;
		}
		
		if (ATTRIBUTES_ID.equals(id)) {
			String str = "";
			Iterator it = fInstance.getAttributes();
			
			while (it.hasNext()) {
				AttributeInstance element = (AttributeInstance) it.next();
				
				str += element + ", ";
			}
			if (str.length() > 1)
				return str.substring(0, str.length() - 2);
			else
				return str;
		}
		return null;
	}

	public boolean isPropertySet(Object id) {
		// TODO Auto-generated method stub
		return false;
	}

	public void resetPropertyValue(Object id) {
		// TODO Auto-generated method stub

	}

	public void setPropertyValue(Object id, Object value) {
		// TODO Auto-generated method stub

	}

}
