/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.properties;

import java.util.Iterator;

import kumbang.core.configuration.instance.BindingInstance;
import kumbang.core.configuration.instance.InterfaceInstance;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;

public class InterfacePropertySource implements IPropertySource {
	
	private static final String NAME_ID = "interface.name";
	private static final String TYPE_ID = "interface.type";
	private static final String BINDINGS_ID = "interface.bindings";
	private static final String FUNCTIONS_ID = "interface.funcs";
	private static final String DIRECTION_ID = "interface.dir"; 
	private static final String OPTIONAL_ID = "interface.opt";
	private static final String GROUNDED_ID = "interface.grnd";
	
	private IPropertyDescriptor[] propertyDescriptors;
	
	private InterfaceInstance iInstance;
	
	public InterfacePropertySource(InterfaceInstance instance) {
		super();
		this.iInstance = instance;
	}

	public Object getEditableValue() {
		// TODO Auto-generated method stub
		return null;
	}

	public IPropertyDescriptor[] getPropertyDescriptors() {
		if (propertyDescriptors == null) {
			PropertyDescriptor nameDescriptor = new PropertyDescriptor(NAME_ID, "1. Interface name");
			PropertyDescriptor typeDescriptor = new PropertyDescriptor(TYPE_ID, "2. Interface type");
			PropertyDescriptor bindDescriptor = new PropertyDescriptor(BINDINGS_ID, "3. Bindings");
			PropertyDescriptor funcDescriptor = new PropertyDescriptor(FUNCTIONS_ID, "4. Functions");
			PropertyDescriptor dirDescriptor = new PropertyDescriptor(DIRECTION_ID, "5. Direction");
			PropertyDescriptor optionalDescriptor = new PropertyDescriptor(OPTIONAL_ID, "6. Optional");
			PropertyDescriptor groundDescriptor = new PropertyDescriptor(GROUNDED_ID, "7. Grounded");

			propertyDescriptors = new IPropertyDescriptor[] {
					nameDescriptor,
					typeDescriptor,
					bindDescriptor,
					funcDescriptor,
					dirDescriptor,
					optionalDescriptor,
					groundDescriptor};
			
		}
		return propertyDescriptors;

	}

	public Object getPropertyValue(Object id) {
		if (NAME_ID.equals(id)) {
			return iInstance.getName();
		}
		
		if (TYPE_ID.equals(id)) {
			return iInstance.getType().getDisplayName();
		}
		
		if (BINDINGS_ID.equals(id)) {
			String str = "";
			Iterator it = iInstance.getBindings();
			while (it.hasNext()) {
				BindingInstance element = (BindingInstance) it.next();
				str += element.toString() + ", ";
			}
			if (str.length() > 1)
				return str.substring(0, str.length()-2);
			else
				return str;
		}
		
		if (FUNCTIONS_ID.equals(id)) {
			String str = "";
			Iterator it = iInstance.getFunctions();
			while (it.hasNext()) {
				String element = (String) it.next();
				str += element + ", ";
			}
			if (str.length() > 1)
				return str.substring(0, str.length()-2);
			else
				return str;
		}

		if (DIRECTION_ID.equals(id)) {
			return iInstance.isProvided() ? "provided" : "required";
		}
		
		if (OPTIONAL_ID.equals(id)) {
			return iInstance.isOptional() ? "true" : "false";
		}
		
		
		if (GROUNDED_ID.equals(id)) {
			return iInstance.isGrounded() ? "true" : "false";
		}
		
		return null;
	}

	public boolean isPropertySet(Object id) {
		// TODO Auto-generated method stub
		return false;
	}

	public void resetPropertyValue(Object id) {
		// TODO Auto-generated method stub

	}

	public void setPropertyValue(Object id, Object value) {
		// TODO Auto-generated method stub

	}

}
