/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.tree;

import java.util.List;

import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.views.properties.IPropertySource;

import kumbang.configurator.client.plugin.Icons;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.PluginHandler;
import kumbang.configurator.client.plugin.properties.BindingPropertySource;
import kumbang.core.configuration.instance.BindingInstance;
import kumbang.core.configuration.instance.Instance;
import kumbang.core.configuration.instance.InterfaceInstance;

public class BindingNode extends TreeNode {

	private BindingInstance bInstance;
	private InterfaceInstance iInstance;
	private String text;
	private BindingPropertySource pSource;
	
	public BindingNode(BindingInstance binding, InterfaceInstance interf) {
		this(null, binding, interf);
	}
	
	public BindingNode(ITreeNode parent, BindingInstance binding, InterfaceInstance interf) {
		super(parent);
		bInstance = binding;
		iInstance = interf;
		if (binding.getTargetInterface().equals(interf)) {
			this.text  
			= binding.getSourceInterface().getParent().toString() 
			+ " . " + binding.getSourceInterface().toString();
		} else {
			this.text 
			= binding.getTargetInterface().getParent().toString() 
			+ " . " + binding.getTargetInterface().toString();
		}
	}
	
	// BindingInstances don't have children
	protected void createChildren(List children) {
	}
	
	// BindingInstances don't have children
	public boolean hasChildren() {
		return false; 
	}

	public Instance getInstance() {
        return this.bInstance;
    }
    
    public BindingInstance getBindingInstance() {
        return this.bInstance;
    }
    
    public InterfaceInstance getInterfaceInstance() {
        return this.iInstance;
    }

	public String toString() {
		return text;
	}
	
	public String getName() {
		return toString();
	}
	
	
	public Image getImage() {
		
		if (bInstance.getSourceInterface().equals(((InterfaceNode)this.getParent()).getInterfaceInstance()))
			return PluginHandler.getActivator().getImageRegistry().get(Icons.BINDING_SOURCE);
		else
			return PluginHandler.getActivator().getImageRegistry().get(Icons.BINDING_TARGET);
		
	}

	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (pSource == null) {
				// XXX: potential problem source !!
				this.pSource = new BindingPropertySource(bInstance);
			}
			return pSource;
		}
		return null;
	}
	

	public boolean equals(Object obj) {
		if (obj == null) return false;
		
		if (obj instanceof BindingNode) {
			return ((BindingNode)obj).getInstance().equals(this.bInstance) && 
			((BindingNode)obj).getInterfaceInstance().equals(this.iInstance);
		} else
			return false;
	}

	
}
