/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.tree;

import java.util.Iterator;
import java.util.List;

import kumbang.configurator.client.plugin.Icons;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.PluginHandler;
import kumbang.configurator.client.plugin.properties.ComponentPropertySource;
import kumbang.core.configuration.instance.AttributeInstance;
import kumbang.core.configuration.instance.ComponentInstance;
import kumbang.core.configuration.instance.Instance;
import kumbang.core.configuration.instance.InterfaceInstance;
import kumbang.core.configuration.instance.PartInstance;

import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.views.properties.IPropertySource;

public class ComponentNode extends TreeNode {
	
	private ComponentInstance cInstance;
	private Object pSource;

	public ComponentNode(ComponentInstance component) {
		this(null, component);
	}
	
	public ComponentNode(ITreeNode parent, ComponentInstance component) {
		super(parent);
		cInstance = component;
	}
	

	protected void createChildren(List children) {
		if (cInstance.hasAttributes()) {
			Iterator it2 = cInstance.getAttributes();
			while (it2.hasNext()) {
				children.add(new AttributeNode(this, (AttributeInstance)it2.next()));
			}
		}
		
		if (cInstance.hasInterfaces()) {
			Iterator it4 = cInstance.getInterfaces();
			while (it4.hasNext())
				children.add(new InterfaceNode(this, (InterfaceInstance)it4.next()));
		}
		
		if (cInstance.hasParts()) {
			Iterator it3 = cInstance.getParts();
			while (it3.hasNext()) {
				children.add(new PartNode(this, (PartInstance)it3.next()));
			}
		}
	}
	
	public boolean equals(Object obj) {
		if (obj instanceof ComponentNode)
			return getName().equals(((ComponentNode)obj).getName());
		else
			return false;
	}

	public String getName() {
		return cInstance.getType().getDisplayName();
	}
	
	public Image getImage() {
		return PluginHandler.getActivator().getImageRegistry().get(Icons.COMPONENT); 
	}
	
	public Instance getInstance() {
		return this.cInstance;
	}

	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (pSource == null) {
				// XXX: potential problem source !!
				this.pSource = new ComponentPropertySource(cInstance);
			}
			return pSource;
		}
		return null;
	}
	
}




