/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.tree;

import java.util.Iterator;
import java.util.List;

import kumbang.configurator.client.plugin.Icons;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.PluginHandler;
import kumbang.configurator.client.plugin.properties.FeaturePropertySource;
import kumbang.core.configuration.instance.AttributeInstance;
import kumbang.core.configuration.instance.FeatureInstance;
import kumbang.core.configuration.instance.Instance;
import kumbang.core.configuration.instance.PartInstance;

import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.views.properties.IPropertySource;

public class FeatureNode extends TreeNode {

	private FeatureInstance fInstance;
	private FeaturePropertySource pSource;
	
	public FeatureNode(FeatureInstance feature) {
		this(null, feature);

	}

	public FeatureNode(ITreeNode parent, FeatureInstance feature) {
		super(parent);
		fInstance = feature;
	}
	
	public boolean hasChildren() {
		if (fInstance.hasInstancesAsPart() || fInstance.hasAttributes()) {
			return true;
		}
		else {
			return false;
		}
	}
	
	protected void createChildren(List children) {
		Iterator it = fInstance.getPartInstances();
        
		if (fInstance.hasAttributes()) {
			Iterator it2 = fInstance.getAttributes();
			while (it2.hasNext()) {
				children.add(new AttributeNode(this, (AttributeInstance)it2.next()));
			}
		}
		
		if (fInstance.hasParts()) {
			it = fInstance.getParts();
			while (it.hasNext()) {
				PartInstance p = (PartInstance)it.next();
				children.add(new PartNode(this, p));
			}
		}
	}

	public String getName() {
		return fInstance.getType().getDisplayName();
	}
	
	public boolean equals(Object obj) {
		if (obj instanceof FeatureNode)
			return getName().equals(((FeatureNode)obj).getName());
		else
			return false;
	}
	
	public Image getImage() {
		return PluginHandler.getActivator().getImageRegistry().get(Icons.FEATURE); 
	}
	
	public Instance getInstance() {
		return this.fInstance;
	}

	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (pSource == null) {
				this.pSource = new FeaturePropertySource(fInstance);
			}
			return pSource;
		}
		return null;
	}
	
}
