/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.tree;

import java.util.Iterator;
import java.util.List;

import kumbang.configurator.client.plugin.Icons;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.PluginHandler;
import kumbang.configurator.client.plugin.properties.InterfacePropertySource;
import kumbang.core.configuration.instance.BindingInstance;
import kumbang.core.configuration.instance.Instance;
import kumbang.core.configuration.instance.InterfaceInstance;

import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.views.properties.IPropertySource;

public class InterfaceNode extends TreeNode {
	
	private InterfaceInstance iInstance;
	private InterfacePropertySource pSource;
	
	public InterfaceNode(InterfaceInstance iface) {
		this(null, iface);
	}
	
	public InterfaceNode(ITreeNode parent, InterfaceInstance iface) {
		super(parent);
		iInstance = iface;
	}
	
	protected void createChildren(List children) {
		Iterator it = iInstance.getBindings();
		Object obj;
		while (it.hasNext()) {
			obj = it.next();
			
			if (obj instanceof BindingInstance)
				children.add(new BindingNode(this, (BindingInstance)obj, iInstance));			
		}
	}

	public boolean hasChildren() {
		return iInstance.hasBindings();
	}
	
	public String getName() {
		return iInstance.toString();
	}
	
	public Image getImage() {
		Image provided;
		Image required;
		if (PluginHandler.getActivator().isUMLPreferenceSet()) {
			provided = PluginHandler.getActivator().getImageRegistry().get(Icons.PROVIDED_INTERFACE_UML);
			required = PluginHandler.getActivator().getImageRegistry().get(Icons.REQUIRED_INTERFACE_UML);
		} else {
			provided = PluginHandler.getActivator().getImageRegistry().get(Icons.PROVIDED_INTERFACE);
			required = PluginHandler.getActivator().getImageRegistry().get(Icons.REQUIRED_INTERFACE);
		}
		
		if (this.iInstance.isProvided())
			return provided;
		else if (this.iInstance.isRequired()) {
			return required;
		} else
			return null;
	}
	
	public InterfaceInstance getInterfaceInstance() {
		return iInstance;
	}
	
	public Instance getInstance() {
		return this.iInstance;
	}

	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (pSource == null) {
				// XXX: potential problem source !!
				this.pSource = new InterfacePropertySource(iInstance);
			}
			return pSource;
		}
		return null;
	
	}

	public boolean equals(Object obj) {
		if (obj == null) return false;
		
		if (obj instanceof InterfaceNode) {
			return ((InterfaceNode)obj).getInstance().equals(this.iInstance);
		} else
			return false;
	}

	
}
