/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.tree;

import java.util.Iterator;
import java.util.List;

import kumbang.configurator.client.plugin.Icons;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.PluginHandler;
import kumbang.configurator.client.plugin.properties.PartPropertySource;
import kumbang.core.configuration.instance.ComponentInstance;
import kumbang.core.configuration.instance.FeatureInstance;
import kumbang.core.configuration.instance.Instance;
import kumbang.core.configuration.instance.PartInstance;

import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.views.properties.IPropertySource;

public class PartNode extends TreeNode {
	
	private PartInstance pInstance;
	private PartPropertySource pSource;

	public PartNode(PartInstance part) {
		this(null, part);
	}
	
	public PartNode(ITreeNode parent, PartInstance part) {
		super(parent);
		pInstance = part;
	}

	protected void createChildren(List children) {
		Iterator it = pInstance.getInstances();
		Object obj;
		while (it.hasNext()) {
			obj = it.next();
			
			if (obj instanceof PartInstance)
				children.add(new PartNode(this, (PartInstance)obj));
			
			else if (obj instanceof ComponentInstance)
				children.add(new ComponentNode(this, (ComponentInstance)obj));
			
			else if (obj instanceof FeatureInstance) {
				children.add(new FeatureNode(this, (FeatureInstance)obj));
			}
		}
	}
	
	public boolean hasChildren() {
		
		return pInstance.hasInstances();
	}
	
	public boolean equals(Object obj) {
		if (obj instanceof PartNode)
			return getName().equals(((PartNode)obj).getName());
		else
			return false;
	}
	
	public String getName() {
		return pInstance.toString();
	}
	
	public Image getImage() {
		return PluginHandler.getActivator().getImageRegistry().get(Icons.PART); 
	}
	
	public Instance getInstance() {
		return this.pInstance;
	}

	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySource.class) {
			if (pSource == null) {
				// XXX: potential problem source !!
				this.pSource = new PartPropertySource(pInstance);
			}
			return pSource;
		}
		return null;
	}
	

    
}
