/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.views;

import kumbang.configurator.Activator;
import kumbang.configurator.client.core.ClientServices;
import kumbang.configurator.client.core.TaskHandler;
import kumbang.configurator.client.core.exception.ClientException;
import kumbang.configurator.client.plugin.Colors;
import kumbang.configurator.client.plugin.IKumbangPlugin;
import kumbang.configurator.client.plugin.Icons;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.diagrams.BindingElement;
import kumbang.configurator.client.plugin.diagrams.BoxedElement;
import kumbang.configurator.client.plugin.diagrams.ComponentDiagram;
import kumbang.configurator.client.plugin.diagrams.ComponentDiagramLevel;
import kumbang.configurator.client.plugin.diagrams.ComponentElement;
import kumbang.configurator.client.plugin.diagrams.DiagramElement;
import kumbang.configurator.client.plugin.diagrams.DiagramHandler;
import kumbang.configurator.client.plugin.diagrams.DiagramLevel;
import kumbang.configurator.client.plugin.diagrams.InstantiatedElement;
import kumbang.configurator.client.plugin.diagrams.InterfaceElement;
import kumbang.configurator.client.plugin.diagrams.PartElement;
import kumbang.configurator.client.plugin.dialogs.AddComposableDialog;
import kumbang.configurator.client.plugin.dialogs.EditAttributesDialog;
import kumbang.configurator.client.plugin.dialogs.EditInterfacesDialog;
import kumbang.configurator.client.plugin.event.ConfigurationModificationListener;
import kumbang.configurator.client.plugin.event.ConfigurationModifiedEvent;
import kumbang.configurator.client.plugin.event.ConfigurationOpenedClosedEvent;
import kumbang.configurator.client.plugin.event.ConfigurationOpenedClosedListener;
import kumbang.configurator.client.plugin.tree.BindingNode;
import kumbang.configurator.client.plugin.tree.ComponentNode;
import kumbang.configurator.client.plugin.tree.InterfaceNode;
import kumbang.configurator.client.plugin.tree.TreeNode;
import kumbang.core.configuration.core.ConfigurationException;
import kumbang.core.configuration.instance.BindingInstance;
import kumbang.core.configuration.instance.ComponentInstance;
import kumbang.core.configuration.instance.ComposableInstance;
import kumbang.core.configuration.instance.Instance;
import kumbang.core.configuration.instance.InterfaceInstance;
import kumbang.core.configuration.instance.PartInstance;
import kumbang.core.configuration.task.AddBindingTask;
import kumbang.core.configuration.task.RemoveInstanceTask;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.MouseEvent;
import org.eclipse.draw2d.MouseListener;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.resource.ColorRegistry;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.CoolBar;
import org.eclipse.swt.widgets.CoolItem;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.PageBook;
import org.eclipse.ui.part.ViewPart;

public class ComponentDiagramView extends ViewPart implements		
		ConfigurationModificationListener, ConfigurationOpenedClosedListener, MouseListener {

	public final static String COMPONENTVIEW_ID = "KumbangTools.view2";
	
	private FigureCanvas canvas;

	private Figure parentFigure;

	private ComponentDiagram diagram;

	private ClientServices client;

	private DiagramHandler diagramHandler;

	private ComponentElement rootElement;

	private Composite compositeDiagram;

	private Composite compositeNoDiagram;

	private Label infoLabel;

	private PageBook pagebook;

	private DiagramElement selected;

	private BoxedElement inAction;

	private Action actionEditAttributes;

	private Action actionEditInterfaces;

	private Action actionRemoveComponents;

	private Action actionShowInTree;
	
	private Action actionZoomIn;
	
	private Action actionZoomOut;

	private ToolItem toolItemArrow;
	
	private ToolItem toolItemZoom;

	private ToolItem toolItemZoomOut;

	private ToolItem toolItemConnect;

	private ToolItem toolItemDisconnect;

	private int mouseX;

	private int mouseY;

	protected static final int PAGE_EMPTY = 0;

	protected static final int PAGE_MODEL = 1;

	private static ColorRegistry cr = getPlugin()
			.getColorRegistry();

	private static ImageRegistry ir = getPlugin()
			.getImageRegistry();

	private ISelectionListener pageSelectionListener;

	
	
	/**
	 * Constructor
	 */
	public ComponentDiagramView() {
		super();
		selected = null;
		inAction = null;
	}

	/**
	 * createPartControl is needed when extending ViewPart
	 */
	public void createPartControl(Composite parent) {
		
		pagebook = new PageBook(parent, SWT.NONE);

		// Create the normal view
		createEnabledView(pagebook);

		// Create the disabled view
		createDisabledView(pagebook);

		// Show the empty page initially
		showPage(PAGE_EMPTY);

		// check if a model is already open and react by showing the tree page
		resetView();

		// Start listening for configuration init/close events and conf mod
		// events
		getPlugin().getHandler().addOpenCloseListener(this);

        
		// Listen for ESC
        canvas.addKeyListener(new KeyListener() {
            public void keyPressed(KeyEvent e) {
                if (e.character == SWT.ESC) {
                    ComponentDiagramView.this.toolItemArrow.setSelection(true);
                    ComponentDiagramView.this.toolItemConnect.setSelection(false);
                    ComponentDiagramView.this.toolItemDisconnect.setSelection(false);
                    ComponentDiagramView.this.toolItemZoom.setSelection(false);
                    ComponentDiagramView.this.canvas.setCursor(new Cursor(
                            canvas.getDisplay(), SWT.CURSOR_ARROW));
                }
            }

            public void keyReleased(KeyEvent e) {}
        });
        

	}

	/**
	 * Shows either the empty page or the diagram page in the view
	 * 
	 * @param page
	 */
	protected void showPage(int page) {
		if (page == PAGE_EMPTY) {
			pagebook.showPage(compositeNoDiagram);
		} else if (page == PAGE_MODEL) {
			pagebook.showPage(compositeDiagram);
		}
	}

	/**
	 * Resets the diagram-view completely
	 */
	protected void resetView() {
		if (getPlugin().getHandler().isServerAvailable()) {
			this.client = getPlugin().getHandler()
					.getClientServices();
			if (client.getComponentRoot() != null) {
				initAndShowModel();
			} else {
				showPage(PAGE_EMPTY);
			}

		} else {
			// TODO: Add a no server available page or something
			showPage(PAGE_EMPTY);
		}
	}

	/**
	 * Updates the diagram-view
	 * 
	 */
	protected void updateView() {
		parentFigure.removeAll();
		diagram.repaint();
		diagram.updateSize();
		parentFigure.add(diagram);
		parentFigure.repaint();
		parentFigure.setPreferredSize(diagram.getPreferredSize());
		parentFigure.getUpdateManager().performUpdate();
	}

	/**
	 * Public interface for updating the view
	 */
	public void refresh() {
		if (getPlugin().getHandler().isServerAvailable())
			updateView();
	}

	/**
	 * Initializes the model and shows the diagram
	 * 
	 */
	protected void initAndShowModel() {

		this.parentFigure.removeAll();
		showPage(PAGE_MODEL);
		getPlugin().getHandler()
				.addConfigurationModificationListener(this);

		diagramHandler = new DiagramHandler();
		diagramHandler.init(client.getComponentRoot());
		diagram = new ComponentDiagram();

		diagram.setDiagramLevel(diagramHandler.getCurrentComponentLevel());
		if (diagramHandler.getCurrentComponentLevel() != null) {
			rootElement = diagramHandler.getCurrentComponentLevel()
					.getRootComponentElement();
		}

		diagram.updateSize();
		diagram.addMouseListener(this);

		parentFigure.add(diagram);
		parentFigure.setPreferredSize(diagram.getPreferredSize());

		// Add menumanager
		makeActions();
		MenuManager menuManager = new MenuManager();
		menuManager.setRemoveAllWhenShown(true);
		
		menuManager.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				// get selection from the diagram and only add what's required
				Object obj = selected;
				ComponentInstance c = (ComponentInstance) rootElement.getInstance();
				if (obj instanceof ComponentElement) {
					ComponentElement elem = (ComponentElement) obj;
					// Zoom in
					if (elem.isNavigable()) 
						manager.add(actionZoomIn);
				}
					// Zoom out
				if (!c.equals(client.getComponentRoot()))
					manager.add(actionZoomOut);
				if (obj instanceof ComponentElement) {
					ComponentElement elem = (ComponentElement) obj;
					// if selected component has attributes
					if (elem.areAttributesEditable()) {
						manager.add(actionEditAttributes);
						actionEditAttributes.setEnabled(true);
					} else {
						manager.add(actionEditAttributes);
						actionEditAttributes.setEnabled(false);
					}

					// if selected component has interfaces
					if (elem.areInterfacesEditable()) {
						manager.add(actionEditInterfaces);
						actionEditInterfaces.setEnabled(true);
					} else {
						manager.add(actionEditInterfaces);
						actionEditInterfaces.setEnabled(false);
					}
					manager.add(actionShowInTree);
					// remove component
					if (!elem.isRootElement()) {
						manager.add(actionRemoveComponents);
						actionRemoveComponents.setText("Remove this component");
					}
				}
								
				if (obj instanceof InterfaceElement) {
					manager.add(actionEditInterfaces);
					actionEditInterfaces.setEnabled(true);
					manager.add(actionShowInTree);
				}
				if (obj instanceof BindingElement) {
					manager.add(actionRemoveComponents);
					actionRemoveComponents.setText("Remove this binding");
					manager.add(actionShowInTree);
				}

			}
		});
		canvas.setMenu(menuManager.createContextMenu(canvas));

	}

	/**
	 * Creates the 'enabled' view when server is available.
	 * 
	 * @param parent
	 */
	private void createEnabledView(Composite parent) {
		compositeDiagram = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.horizontalSpacing = 0;
		layout.verticalSpacing = 0;
		layout.marginWidth = 0;
		layout.marginHeight = 0;
		layout.marginTop = 0;
		layout.marginBottom = 0;
		compositeDiagram.setLayout(layout);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		compositeDiagram.setLayoutData(gd);
		
		
		

		// Here comes the toolbar
		Composite coolBarComposite = new Composite(compositeDiagram, SWT.BORDER
				| SWT.SHADOW_NONE);
		layout = new GridLayout();
		layout.marginWidth = 1;
		layout.marginHeight = 1;
		coolBarComposite.setLayout(layout);
		coolBarComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		CoolBar coolbar = new CoolBar(coolBarComposite, SWT.NONE);

		// Create toolbar coolitem
		final CoolItem item = new CoolItem(coolbar, SWT.DROP_DOWN);
		item.setControl(createToolBar(coolbar));
		calcSize(item);

		// Setting of the composites
		Composite innerDiagram = new Composite(compositeDiagram, SWT.NULL);
		innerDiagram.setLayout(new FillLayout());
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		innerDiagram.setLayoutData(gd);

		canvas = new FigureCanvas(innerDiagram);
		canvas.setBackground(cr.get(Colors.COMPONENT));
		canvas.setScrollBarVisibility(FigureCanvas.AUTOMATIC);

		parentFigure = new Figure();
		parentFigure.setBackgroundColor(cr.get(Colors.COMPONENT));
		parentFigure.setLayoutManager(new XYLayout());
		canvas.setContents(parentFigure);
		
		// Start listening for selection events
		hookPageSelection();

	}

	/**
	 * Creates a toolbar inside the coolbar.
	 * 
	 * @param coolbar
	 * @return toolbar
	 */
	private Control createToolBar(CoolBar coolbar) {

		ToolBar toolBar = new ToolBar(coolbar, SWT.NONE);

		this.toolItemArrow = new ToolItem(toolBar, SWT.CHECK);
		toolItemArrow.setImage(ir.get(Icons.SELECT));
		toolItemArrow.setSelection(true);
		toolItemArrow.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (toolItemArrow.getSelection()) {
					toolItemZoom.setSelection(false);
					toolItemConnect.setSelection(false);
					toolItemDisconnect.setSelection(false);
					canvas.setCursor(new Cursor(canvas.getDisplay(),
							SWT.CURSOR_ARROW));
				}
                setArrowAsDefaultToolItem();
			}
		});
		
		
		this.toolItemZoom = new ToolItem(toolBar, SWT.CHECK);
		toolItemZoom.setImage(ir.get(Icons.ZOOM));
		toolItemZoom.setToolTipText("Zoom in");
		toolItemZoom.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (toolItemZoom.getSelection()) {
					toolItemArrow.setSelection(false);
					toolItemConnect.setSelection(false);
					toolItemDisconnect.setSelection(false);
				}
                setArrowAsDefaultToolItem();
                
				unSelectAll();

				if (toolItemZoom.getSelection())
					canvas.setCursor(new Cursor(canvas.getDisplay(), ir.get(
							Icons.ZOOM).getImageData(), 5, 5));
				else
					canvas.setCursor(new Cursor(canvas.getDisplay(),
							SWT.CURSOR_ARROW));
			}
		});

		this.toolItemZoomOut = new ToolItem(toolBar, SWT.PUSH);
		toolItemZoomOut.setImage(ir.get(Icons.ZOOM_OUT));
		toolItemZoomOut.setToolTipText("Zoom out");
		toolItemZoomOut.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				navigateLevelUp();
			}
		});

		new ToolItem(toolBar, SWT.SEPARATOR);

		this.toolItemConnect = new ToolItem(toolBar, SWT.CHECK);
		toolItemConnect.setImage(ir.get(Icons.CONNECT));
		toolItemConnect.setToolTipText("Bind components");
		toolItemConnect.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (toolItemConnect.getSelection()) {
					toolItemArrow.setSelection(false);
					toolItemDisconnect.setSelection(false);
					toolItemZoom.setSelection(false);
				}
                setArrowAsDefaultToolItem();
				unSelectAll();

				if (toolItemConnect.getSelection())
					canvas.setCursor(new Cursor(canvas.getDisplay(), ir.get(
							Icons.CURSOR_CONNECT_1).getImageData(), 0, 0));
				else
					canvas.setCursor(new Cursor(canvas.getDisplay(),
							SWT.CURSOR_ARROW));

			}
		});

		this.toolItemDisconnect = new ToolItem(toolBar, SWT.CHECK);
		toolItemDisconnect.setImage(ir.get(Icons.DISCONNECT));
		toolItemDisconnect.setToolTipText("Disconnect components");
		toolItemDisconnect.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (toolItemDisconnect.getSelection()) {
					toolItemArrow.setSelection(false);
					toolItemConnect.setSelection(false);
					toolItemZoom.setSelection(false);
				}
                setArrowAsDefaultToolItem();
				unSelectAll();

				if (toolItemDisconnect.getSelection())
					canvas.setCursor(new Cursor(canvas.getDisplay(), ir.get(
							Icons.CURSOR_DISCONNECT).getImageData(), 0, 0));
				else
					canvas.setCursor(new Cursor(canvas.getDisplay(),
							SWT.CURSOR_ARROW));
			}
		});

		return toolBar;
	}

	/**
	 * Helper method to calculate the size of the cool item
	 * 
	 * @param item
	 *            the cool item
	 */
	private void calcSize(CoolItem item) {
		Control control = item.getControl();
		Point pt = control.computeSize(SWT.DEFAULT, SWT.DEFAULT);
		pt = item.computeSize(pt.x, pt.y);
		item.setSize(pt);
	}

	/**
	 * Setup the empty page
	 * 
	 * @param parent
	 */
	private void createDisabledView(Composite parent) {
		this.compositeNoDiagram = new Composite(parent, SWT.WRAP);
		GridLayout layout = new GridLayout();
		compositeNoDiagram.setLayout(layout);
		infoLabel = new Label(compositeNoDiagram, SWT.NULL);
		infoLabel.setText("No model initialized.");
	}

	public void setFocus() {
		// TODO Auto-generated method stub
	}
    
    private void setArrowAsDefaultToolItem() {
        if (!toolItemArrow.getSelection() && !toolItemConnect.getSelection()
                && !toolItemDisconnect.getSelection()
                && !toolItemZoom.getSelection()
                && !toolItemZoomOut.getSelection()) {
            toolItemArrow.setSelection(true);
        }
    }
    

	/**
	 * Disposes the view and removes the listeners
	 */
	public void dispose() {
		getPlugin().getHandler()
		.removeConfigurationModificationListener(this);
		getPlugin().getHandler().removeOpenCloseListener(this);
        if (diagram != null) {
            diagram.removeMouseListener(this);
        }
		
		super.dispose();

	}

	/**
	 * Implements the ConfigurationModificationListener -interface
	 */
	public void configurationModified(ConfigurationModifiedEvent e) {
		diagramHandler.configurationModified(e);
		updateView();
	}

	/**
	 * Implements the ConfigurationOpenedClosedListener-interface
	 */
	public void handleOpenCloseEvent(ConfigurationOpenedClosedEvent e) {
		String et = e.getEventType();
		if (et.equals(ConfigurationOpenedClosedEvent.EVENT_INITIALIZE) ||
				et.equals(ConfigurationOpenedClosedEvent.EVENT_SAVEOPENED)) {
			resetView();
			// Start listening for configuration modifications

		} else if (et.equals(ConfigurationOpenedClosedEvent.EVENT_CLOSE)) {
			showPage(PAGE_EMPTY);
			getPlugin().getHandler()
					.removeConfigurationModificationListener(this);
		}
	}

	/**
	 * MouseListener's method for getting mouse-clicks
	 */
	public void mousePressed(MouseEvent e) {

		if (e.getSource() != diagram) {
			return;
		}

		DiagramElement elem = diagram.getElementAt(e.x, e.y);
		mouseX = e.x;
		mouseY = e.y;

		if (e.button == 1 && !toolItemConnect.getSelection()
				&& !toolItemDisconnect.getSelection()
				&& !toolItemZoom.getSelection()) {
			selectElementAt(e.x, e.y);
			diagram.repaint();

			if (elem != rootElement
					&& (elem instanceof ComponentElement || elem instanceof PartElement)) {
				elem.setInAction(true);
				inAction = (BoxedElement) elem;
				diagram.repaint();
			}
			if (selected instanceof PartElement) {
				PartInstance part = (PartInstance) ((PartElement) selected)
						.getInstance();
				AddComposableDialog addDlg = new AddComposableDialog(canvas
						.getShell(), part);
				addDlg.open();
				unSelectAll();
			}
		}

		if (e.button == 1 && this.toolItemZoom.getSelection()) {
			selectElementAt(e.x, e.y);
			if (selected instanceof ComponentElement) {
				ComponentElement c = (ComponentElement) selected;
				if (c.isNavigable()) {
					navigateLevelDown(c);
				}
			}
			unSelectAll();
		}

		// binding components
		if (e.button == 1 && this.toolItemConnect.getSelection()) {
			if (selected != null) {
				selected.setSelected(false);
				selected = null;
			}
			if (elem instanceof InterfaceElement) {
				elem.setInAction(true);
				inAction = (BoxedElement) elem;
				diagram.repaint();
			}
			canvas.setCursor(new Cursor(canvas.getDisplay(), ir.get(
					Icons.CURSOR_CONNECT_2).getImageData(), 0, 0));
		}

		// disconnecting components
		if (e.button == 1 && toolItemDisconnect.getSelection()) {
			if (elem instanceof BindingElement) {
				BindingInstance instance = ((BindingElement) elem)
						.getBindingInstance();

				MessageBox msgbox = new MessageBox(canvas.getShell(),
						SWT.ICON_QUESTION | SWT.YES | SWT.NO);
				msgbox.setText("Removing binding");
				msgbox
						.setMessage("Are you sure you want to remove this binding?\n\n\t"
								+ instance);
				int answer = msgbox.open();

				if (answer == SWT.YES) {
					try {
						TaskHandler taskHandler = client.getTaskHandler();
						RemoveInstanceTask task = taskHandler
								.createRemoveInstanceTask(instance);
						getPlugin().getHandler()
								.fireConfigurationModifiedEvent(
										getPlugin().getHandler()
												.getClientServices()
												.handleTask(task));
					} catch (ConfigurationException ce) {
						errorMessage("Error when removing binding", ce
								.getMessage());
					} catch (ClientException cle) {
						errorMessage("Error when removing binding", cle
								.getMessage());
					}
				}
			}
		}

		// mouse has been right-clicked
		if (e.button != 1 && !toolItemConnect.getSelection()
				&& !toolItemDisconnect.getSelection()
				&& !toolItemZoom.getSelection()) {
			selectElementAt(e.x, e.y);
			diagram.repaint();

			if (elem != rootElement && elem instanceof ComponentElement) {
				elem.setInAction(true);
				inAction = (BoxedElement) elem;
				diagram.repaint();
			}
		}

	}

	/**
	 * MouseListener's method for getting mouse-releases
	 */
	public void mouseReleased(MouseEvent e) {

		// move boxed component on the diagram
		if (inAction != null && e.button == 1
				&& (e.x != mouseX || e.y != mouseY)
				&& !toolItemConnect.getSelection()
				&& !toolItemDisconnect.getSelection()
				&& !toolItemZoom.getSelection()) {
			DiagramLevel level = inAction.getLevel();

			if (inAction instanceof BoxedElement) {
				level.moveChildElement((BoxedElement) inAction, e.x, e.y);
			}
			diagram.repaint();
		}

		// create binding between two interfaces on normal mode
		if (selected instanceof InterfaceElement && e.button == 1
				&& (e.x != mouseX || e.y != mouseY)
				&& !toolItemConnect.getSelection()
				&& !toolItemDisconnect.getSelection()
				&& !toolItemZoom.getSelection()) {

			if (!(diagram.getElementAt(e.x, e.y) instanceof InterfaceElement))
				return;

			InterfaceInstance instance1 = ((InterfaceElement) selected)
					.getInterfaceInstance();
			InterfaceInstance instance2 = ((InterfaceElement) diagram
					.getElementAt(e.x, e.y)).getInterfaceInstance();
			if (instance1.equals(instance2)) {
				return;
			}
			try {
				TaskHandler taskHandler = client.getTaskHandler();
				AddBindingTask task = taskHandler.createAddBindingTask(
						instance1, instance2);

				if (task != null)
					getPlugin().getHandler()
					.fireConfigurationModifiedEvent(
							getPlugin().getHandler()
							.getClientServices().handleTask(task));
			} catch (ConfigurationException ce) {
				errorMessage("Error when adding binding", ce.getMessage());
			} catch (ClientException cle) {
				errorMessage("Error when adding binding", cle.getMessage());
			}

			unSelectAll();
			diagram.repaint();
		}

		// create connection between two interfaces when in connect-mode
		if (e.button == 1 && toolItemConnect.getSelection()) {
			DiagramElement elem = diagram.getElementAt(e.x, e.y);
			if (inAction != null && inAction instanceof InterfaceElement
					&& elem != null && elem instanceof InterfaceElement) {
				InterfaceInstance instance1 = ((InterfaceElement) inAction)
						.getInterfaceInstance();
				InterfaceInstance instance2 = ((InterfaceElement) elem)
						.getInterfaceInstance();
				if (instance1.equals(instance2)) {
                    bindingHowTo();
                    if (inAction != null) {
                        inAction.setInAction(false);
                    }
                    inAction = null;
                    updateView();
                    return;
                }
				try {
					TaskHandler taskHandler = client.getTaskHandler();
					AddBindingTask task = taskHandler.createAddBindingTask(
							instance1, instance2);
					// fireConfigurationModifiedEvent(client.handleTask(task));
					getPlugin().getHandler()
							.fireConfigurationModifiedEvent(
									getPlugin().getHandler()
											.getClientServices().handleTask(
													task));
				} catch (ConfigurationException ce) {
					errorMessage("Error when adding binding", ce.getMessage());
				} catch (ClientException cle) {
					errorMessage("Error when adding binding", cle.getMessage());
				}

			}
			diagram.repaint();
			canvas.setCursor(new Cursor(canvas.getDisplay(), ir.get(
					Icons.CURSOR_CONNECT_1).getImageData(), 0, 0));
		}

		if (inAction != null) {
			inAction.setInAction(false);
		}
		inAction = null;
		updateView();
	}

	/**
	 * MouseListener's method for getting double-clicks
	 */
	public void mouseDoubleClicked(MouseEvent e) {

		if (e.getSource() != diagram) {
			return;
		}

		if (!toolItemConnect.getSelection()
				&& !toolItemDisconnect.getSelection()) {
			selectElementAt(e.x, e.y);
			diagram.repaint();

			if (selected instanceof ComponentElement) {
				ComponentElement c = (ComponentElement) selected;
				if (c.isNavigable()) {
					navigateLevelDown(c);
				}
			}
			if (selected instanceof InterfaceElement) {
				InterfaceInstance i = (InterfaceInstance)((InterfaceElement) selected).getInstance();
				ComponentInstance c = i.getComponent();
				EditInterfacesDialog dlg = new EditInterfacesDialog(canvas
						.getShell(), c);
				dlg.open();
			}
		}
	}

	private void unSelectAll() {
		if (selected != null) {
			selected.setSelected(false);
			selected = null;
		}
		if (inAction != null) {
			inAction.setInAction(false);
			inAction = null;
		}
		diagram.repaint();
	}

	/**
	 * Selects an element from the diagram
	 * 
	 * @param x
	 *            x-coordinate
	 * @param y
	 *            y-coordinate
	 */
	private void selectElementAt(int x, int y) {
		selectElement(diagram.getElementAt(x, y), true);
	}

	private void selectInstance(Instance instance) {
		// first check that component levels are ok
		if (diagramHandler.getCurrentComponentLevel() == null) {
			return;
		}
		// check whether the instance is already selected
		if (selected instanceof InstantiatedElement
				&& ((InstantiatedElement) selected).getInstance().equals(
						instance)) {
			return; // no need to do anything
		}
		// check whether instance corresponds to root element
		if (rootElement.getComponentInstance().equals(instance)) {
			selectElement(rootElement, false);
			updateView();
			return;
		}
		// look for instance from child elements
		DiagramElement element = diagramHandler.getCurrentComponentLevel()
				.getChildElementForInstance(instance);
		selectElement(element, false);
		updateView();
	}

	private void selectElement(DiagramElement element, boolean informOthers) {
		// deselect old selection
		if (this.selected != null) {
			this.selected.setSelected(false);
			this.selected = null;
		}
		if (element != null) {
			element.setSelected(true);
			this.selected = element;
		}
		// selectElementFromTree();
	}

	private void selectElementFromTree() {
		// Try to get a reference to the view
		ComponentView componentView = (ComponentView) (PlatformUI
				.getWorkbench().getActiveWorkbenchWindow().getActivePage()
				.findView(COMPONENTVIEW_ID));


		// If the view was not open, try to open it
		if (componentView == null) {
			try {
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView(COMPONENTVIEW_ID);
				// A bit dirty hack, otherwise the component tree viewers selection provider would cause 
				// the diagram to select a wrong element
				
				selectElementFromTree();
				return;
			} catch (PartInitException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		

		// The reset selection stuff is a hack to prevent the Eclipse's selection service 
		// to reselect the old element in the diagram
		
		if (this.selected instanceof ComponentElement) {
			ComponentElement elem = (ComponentElement) this.selected;
			ComponentInstance c = (ComponentInstance) elem.getInstance();
			componentView.getSite().getPage().activate(componentView.getSite().getPart());
			componentView.resetSelection();
			componentView.selectElement(c);
			// I don't know why, but these lines are needed for component
			// elements, or the diagram won't
			// refresh as it should
			unSelectAll();
			selectElement(elem, false);
			diagram.repaint();
		}
		if (this.selected instanceof InterfaceElement) {
			InterfaceElement elem = (InterfaceElement) this.selected;
			InterfaceInstance i = (InterfaceInstance) elem.getInstance();
			componentView.getSite().getPage().activate(componentView.getSite().getPart());
			componentView.resetSelection();
			componentView.selectElement(i);
		}
		if (this.selected instanceof BindingElement) {
			BindingElement elem = (BindingElement) this.selected;
			BindingInstance b = (BindingInstance) elem.getInstance();
			componentView.getSite().getPage().activate(componentView.getSite().getPart());
			componentView.resetSelection();
			componentView.selectElement(b);
		}
	}

	/**
	 * Navigates one level down according the element given as a parameter
	 * 
	 * @param element
	 */
	private void navigateLevelDown(ComponentElement element) {
		// go down!
		ComponentDiagramLevel level = diagramHandler
				.navigateComponentLevelDown(element);
		if (level != null) {

			diagram.setDiagramLevel(diagramHandler.getCurrentComponentLevel());
			if (diagramHandler.getCurrentComponentLevel() != null) {
				rootElement = diagramHandler.getCurrentComponentLevel()
						.getRootComponentElement();
			}
			unSelectAll();
			updateView();
		}
	}

	/**
	 * Navigates one level up in the diagram hierarchy
	 */
	private void navigateLevelUp() {
		// go up!
		ComponentDiagramLevel level = diagramHandler.navigateComponentLevelUp();
		if (level != null) {

			diagram.setDiagramLevel(diagramHandler.getCurrentComponentLevel());
			if (diagramHandler.getCurrentComponentLevel() != null) {
				rootElement = diagramHandler.getCurrentComponentLevel()
						.getRootComponentElement();
			}
			unSelectAll();
			updateView();
		}
	}

	/**
	 * Makes the actions for the pop-up menus
	 */
	private void makeActions() {
		
		// Edit attributes
		actionEditAttributes = new Action() {
			public void run() {

				// Give EditAttributeDialog the proper instance
				ComponentElement elem = (ComponentElement) selected;

				EditAttributesDialog dlg = new EditAttributesDialog(canvas
						.getShell(), (ComposableInstance) elem.getInstance());
				dlg.open();

			}
		};
		actionEditAttributes.setText("Edit Attributes");
		actionEditAttributes.setToolTipText("Edit Attributes");
		actionEditAttributes.setImageDescriptor(getPlugin()
				.getImageDescriptor(Icons.ATTRIBUTE));

		// Edit interfaces
		actionEditInterfaces = new Action() {
			public void run() {

				// Give EditAttributeDialog the proper instance
				Object o = selected;
				ComponentInstance c = null;
				if (o instanceof ComponentElement) {
					ComponentElement elem = (ComponentElement) o;
					c = (ComponentInstance) elem.getInstance();
				}
				if (o instanceof InterfaceElement) {
					InterfaceElement iElem = (InterfaceElement) o;
					InterfaceInstance i = (InterfaceInstance) iElem
							.getInterfaceInstance();
					c = i.getComponent();
				}
				EditInterfacesDialog dlg = new EditInterfacesDialog(canvas
						.getShell(), c);
				dlg.open();

			}
		};
		actionEditInterfaces.setText("Edit Interfaces");
		actionEditInterfaces.setToolTipText("Edit Interfaces");
		actionEditInterfaces.setImageDescriptor(getPlugin()
				.getImageDescriptor(Icons.REQUIRED_INTERFACE));

		// Remove components and bindings
		actionRemoveComponents = new Action() {
			public void run() {
				if (selected instanceof ComponentElement) {
					ComponentElement elem = (ComponentElement) selected;

					Shell parent = canvas.getShell();
					MessageBox msgbox = new MessageBox(parent,
							SWT.ICON_QUESTION | SWT.YES | SWT.NO);
					msgbox.setText("Removing component");
					msgbox
							.setMessage("Are you sure you want to remove this component?\n\n\t"
									+ elem.getInstance());
					int answer = msgbox.open();

					if (answer == SWT.YES) {
						if (elem.getComposableInstance().getParent() == null) {
							return;
						}
						try {
							TaskHandler taskHandler = client.getTaskHandler();
							RemoveInstanceTask task = taskHandler
									.createRemoveInstanceTask(elem
											.getComposableInstance());
							getPlugin().getHandler()
									.fireConfigurationModifiedEvent(
											client.handleTask(task));
						} catch (ClientException e) {
							errorMessage("Error when removing instance", e
									.getMessage());
						} catch (ConfigurationException ce) {
							errorMessage("Error when removing instance", ce
									.getMessage());
						}
					}
				} else if (selected instanceof BindingElement) {
					BindingInstance instance = ((BindingElement) selected)
							.getBindingInstance();

					MessageBox msgbox = new MessageBox(canvas.getShell(),
							SWT.ICON_QUESTION | SWT.YES | SWT.NO);
					msgbox.setText("Removing binding");
					msgbox
							.setMessage("Are you sure you want to remove this binding?\n\n\t"
									+ instance);
					int answer = msgbox.open();

					if (answer == SWT.YES) {
						try {
							TaskHandler taskHandler = client.getTaskHandler();
							RemoveInstanceTask task = taskHandler
									.createRemoveInstanceTask(instance);
							getPlugin().getHandler()
									.fireConfigurationModifiedEvent(
											getPlugin().getHandler()
													.getClientServices()
													.handleTask(task));
						} catch (ConfigurationException ce) {
							errorMessage("Error when removing binding", ce
									.getMessage());
						} catch (ClientException cle) {
							errorMessage("Error when removing binding", cle
									.getMessage());
						}
					}
				}
			}
		};
		actionRemoveComponents.setToolTipText("Removes the instance");
		actionRemoveComponents.setImageDescriptor(getPlugin()
				.getImageDescriptor(Icons.TRASH_CAN));

		// Show element in the component tree
		actionShowInTree = new Action() {
			public void run() {
				selectElementFromTree();
			}
		};
		actionShowInTree.setText("Show this in tree");
		actionShowInTree.setToolTipText("Show this in tree");
		actionShowInTree.setImageDescriptor(getPlugin()
				.getImageDescriptor(Icons.SELECT));
		
		actionZoomIn = new Action() {
			public void run() {
				navigateLevelDown((ComponentElement)selected);
			}
		};
		actionZoomIn.setText("Zoom in");
		actionZoomIn.setToolTipText("Zoom in");
		actionZoomIn.setImageDescriptor(getPlugin()
				.getImageDescriptor(Icons.ZOOM));

		actionZoomOut = new Action() {
			public void run() {
				navigateLevelUp();
			}
		};
		actionZoomOut.setText("Zoom out");
		actionZoomOut.setToolTipText("Zoom out");
		actionZoomOut.setImageDescriptor(getPlugin()
				.getImageDescriptor(Icons.ZOOM_OUT));
	}

	/**
	 * Shows a standard error-message dialog
	 * 
	 * @param text
	 * @param message
	 */
	private void errorMessage(String text, String message) {
		MessageBox msgbox = new MessageBox(canvas.getShell(), SWT.ICON_ERROR
				| SWT.OK);
		msgbox.setText(text);
		msgbox.setMessage(message);
		msgbox.open();
	}

	private void bindingHowTo() {
		MessageBox msgbox = new MessageBox(canvas.getShell(), SWT.ICON_INFORMATION
				| SWT.OK);
		msgbox.setText("How to bind interfaces");
		msgbox.setMessage("Press mouse button on one interface and release it on another interface. " +
				"This makes a binding between the two interfaces.");
		msgbox.open();
	}
	
	public ComponentDiagram getDiagram() {
		return this.diagram;
	}

	public void showInstance(Instance instance) {
		ComponentDiagramLevel level = diagramHandler
				.navigateToComponent(instance);

		if (level != null) {
			diagram.setDiagramLevel(level);
			selectInstance(instance);
			updateView();
		}
		if (level == null) {
			navigateLevelUp();
			selectInstance(instance);
			updateView();
		}
	}

	/**
	 * Selects the instance on the diagram if it is on the currently shown
	 * level
	 */
	public void showInstanceOnCurrentLevel(Instance instance) {
		// TODO: Doesn't show the containing component except for the
		// most zoomed out level
		ComponentDiagramLevel currentLevel = diagramHandler.getCurrentComponentLevel();
		ComponentDiagramLevel instanceLevel = diagramHandler.getLevelOfComponent(instance);
		
		if (instanceLevel != null && instanceLevel.equals(currentLevel)) {
			diagram.setDiagramLevel(instanceLevel);
			selectInstance(instance);
			updateView();
		}
		
		// I guess this means the instance is the root
		if (instanceLevel == null) {
			selectInstance(instance);
		}
	}
	
	private void hookPageSelection() {
		pageSelectionListener = new ISelectionListener() {
			/* When the selection in the component tree changes, if the
			 * selected element is on the current diagram level, select it.
			 */
		    public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		        if (selection instanceof IStructuredSelection) {
		            if (((IStructuredSelection)selection).toList().size() > 0) {
		                TreeNode node = (TreeNode)((IStructuredSelection)selection).toList().get(0);
		                if (node instanceof ComponentNode ||
		                        node instanceof InterfaceNode ||
		                        node instanceof BindingNode) {
		                    showInstanceOnCurrentLevel(node.getInstance());
		                }
		            }
		        }
		    }
		};
		getSite().getPage().addPostSelectionListener(pageSelectionListener);
	}

    protected static IKumbangPlugin getPlugin() {
        return Activator.getDefault();
    }
    
}
