/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.views;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Composite;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.PluginHandler;
import kumbang.configurator.client.plugin.preferences.PreferenceConstants;
import kumbang.configurator.client.plugin.tree.ComponentNode;
import kumbang.configurator.client.plugin.tree.RootNode;


/**
 * The view with the Component tree
 * @author jlemmett
 *
 */
public class ComponentView extends KumbangTreeView {

	private IPropertyChangeListener listener;
	
	public ComponentView() {
		super();
		listener = new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				if (event.getProperty().equals(PreferenceConstants.P_UML_INTERFACES)) {
					treeViewer.refresh();
				}
			}
			
		};
		PluginHandler.getActivator().getPreferenceStore().addPropertyChangeListener(listener);
	}
	
	protected void updateView() {
		if (PluginHandler.getInstance().isServerAvailable()) {
			this.client = PluginHandler.getInstance().getClientServices();
			if (client.getComponentRoot() != null) {
				initAndShowTreeViewer();
			} else {
				if (client.isModelInitialized())
					showPage(PAGE_NOTREE);
				else
					showPage(PAGE_EMPTY);
			}
		} else {
			// TODO: Add a no server available page or something
			showPage(PAGE_EMPTY);
		}
	}
	
	protected Object initializeTree() {
		RootNode root = new RootNode();
		root.hookTreeRoot(new ComponentNode(root, client.getComponentRoot()));
		return root;
	}

	protected void createNoTreeView(Composite parent) {
		super.createNoTreeView(parent);
		fLblNoTree.setText("This configuration has no components");
	}
	
	public void dispose() {
		PluginHandler.getActivator().getPreferenceStore().removePropertyChangeListener(listener);
		super.dispose();
	}
	
}
