/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.views;

import java.io.InputStream;

import kumbang.configurator.Activator;
import kumbang.configurator.client.core.ClientServices;
import kumbang.configurator.client.plugin.Colors;
import kumbang.configurator.client.plugin.IKumbangPlugin;
import kumbang.configurator.client.plugin.Icons;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.event.ConfigurationModificationListener;
import kumbang.configurator.client.plugin.event.ConfigurationModifiedEvent;
import kumbang.configurator.client.plugin.event.ConfigurationOpenedClosedEvent;
import kumbang.configurator.client.plugin.event.ConfigurationOpenedClosedListener;
import kumbang.configurator.client.plugin.event.ConfigurationSaveListener;
import kumbang.configurator.client.plugin.event.SaveEvent;

import org.eclipse.jface.resource.ColorRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

import sun.audio.AudioPlayer;
import sun.audio.AudioStream;

/**
 * Status view of the configuration. Shows model and configuration
 * names and the completeness of the configuration.
 * @author jlemmett
 *
 */
public class StatusView extends ViewPart 
implements ConfigurationModificationListener, 
ConfigurationOpenedClosedListener,
ConfigurationSaveListener {

	// Headers
	private static final String PROPERTIES = "Properties: ";
	private static final String CONFIG_NAME = "Configuration Name: ";
	private static final String MODEL_NAME = "Model name: ";
	private static final String CONFIG_STATUS ="Configuration status: ";
	private static final String SAVE_FILE = "Save file name: ";
	
	// Constant string values
	private static final String STR_NONE = "-";

		
	// Variable string values
	private String strConfigName;
	private String strModelName;
	private String strSaveFile;
	
	private ClientServices client;
		
	// SWT Text Widgets
	private Text txtConfigName;
	private Text txtModelName;
	private Text txtSavefile;
	
	private Label imageLabel;
	
	private Color background;
	
	private boolean bComplete;
	private boolean bReseted;
	
	private Image logo;
    private boolean newComplete;
	
	public StatusView() {
		super();
	}

	public void createPartControl(Composite parent) {
		ColorRegistry cr = getPlugin().getColorRegistry();
		background = cr.get(Colors.WHITE);
		
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setBackground(background);
		
		logo = getPlugin().getImageDescriptor(Icons.STATUS_INCOMPLETE).createImage();
		
		
		GridLayout layout = new GridLayout(2,false);
		composite.setLayout(layout);
		
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		
		Label label = new Label(composite, SWT.NONE);
		label.setText(StatusView.PROPERTIES);
		label.setBackground(background);
		label.setLayoutData(gd);
		
		
		label = new Label(composite, SWT.NULL);
		label.setText(StatusView.CONFIG_NAME);
		label.setBackground(background);
		
		txtConfigName = new Text(composite, SWT.READ_ONLY | SWT.SINGLE);
		txtConfigName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		txtConfigName.setBackground(background);
		
		label = new Label(composite, SWT.NULL);
		label.setText(StatusView.MODEL_NAME);
		label.setBackground(background);
		
		txtModelName = new Text(composite, SWT.READ_ONLY | SWT.SINGLE);
		txtModelName.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		txtModelName.setBackground(background);
		
		label = new Label(composite, SWT.NONE);
		label.setText(StatusView.SAVE_FILE);
		label.setBackground(background);
		
		txtSavefile = new Text(composite, SWT.READ_ONLY | SWT.SINGLE);
		txtSavefile.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		txtSavefile.setBackground(background);
		
		label = new Label(composite, SWT.NULL);
		label.setText(StatusView.CONFIG_STATUS);
		label.setBackground(background);
		
//		txtComplete = new Text(composite, SWT.READ_ONLY);
//		txtComplete.setText(STR_NONE);
//		txtComplete.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
//		txtComplete.setBackground(background);
		imageLabel = new Label(composite, SWT.NULL);
		imageLabel.setImage(logo);
		imageLabel.setBackground(background);

		
		//updateView();
		//getPlugin().addConfigurationModificationListener(this);
		getPlugin().getHandler().addOpenCloseListener(this);
		//getPlugin().addSaveListener(this);
		updateView();
	}
	
	/**
	 * Public interface for updating the view
	 */
	public void refresh() {
		updateView();
	}
	
	/**
	 * Determines the new status and then updates the text fields 
	 *
	 */
	private void updateView() {
		if (getPlugin().getHandler().isServerAvailable()) {
			client = getPlugin().getHandler().getClientServices();
			if (client.isConfigurationStateReseted()) {
				strConfigName = STR_NONE;
				strModelName = STR_NONE;
				strSaveFile = STR_NONE;
				bComplete = false;
				bReseted = true;
			} else {
				strConfigName = client.getConfigurationName();
				strModelName = client.getModelName();
				if (client.getSaveFile() != null)
					strSaveFile = client.getSaveFile().getName();
				else
					strSaveFile = STR_NONE;
				bComplete = client.isConfigurationStateComplete();
				bReseted = false;
			}
		} else {
			strConfigName = STR_NONE;
			strModelName = STR_NONE;
			strSaveFile = STR_NONE;
			bComplete = false;
			bReseted = true;
		}
		updateStatusTexts();
	}

	/**
	 * Does the actual updating of the text fields
	 *
	 */
	private void updateStatusTexts() {
		txtConfigName.setText(strConfigName);
		txtModelName.setText(strModelName);
		txtSavefile.setText(strSaveFile);
		if (bReseted) {
			logo = getPlugin().getImageDescriptor(Icons.STATUS_INCOMPLETE).createImage();
			imageLabel.setImage(logo);
		} else {
			if (bComplete) {
				logo = getPlugin().getImageDescriptor(Icons.STATUS_COMPLETE).createImage();
				imageLabel.setImage(logo);
				imageLabel.setToolTipText("Configuration is complete");
			} else {
				logo = getPlugin().getImageDescriptor(Icons.STATUS_INCOMPLETE).createImage();
				imageLabel.setImage(logo);
				imageLabel.setToolTipText("Configuration is incomplete");
			}
		}
	}

	public void setFocus() {
	}
	
	public void dispose() {
		super.dispose();
		getPlugin().getHandler().removeConfigurationModificationListener(this);
		getPlugin().getHandler().removeOpenCloseListener(this);
		getPlugin().getHandler().removeSaveListener(this);
	}

	public void handleOpenCloseEvent(ConfigurationOpenedClosedEvent e) {
		String et = e.getEventType();
		if (et.equals(ConfigurationOpenedClosedEvent.EVENT_INITIALIZE)) {
			//Start listening for configuration modifications
			getPlugin().getHandler().addConfigurationModificationListener(this);
			getPlugin().getHandler().addSaveListener(this);
			updateView();
		} else if (et.equals(ConfigurationOpenedClosedEvent.EVENT_SAVEOPENED)) {
			getPlugin().getHandler().addConfigurationModificationListener(this);
			getPlugin().getHandler().addSaveListener(this);
//			this.strSaveFile = getPlugin().getClientServices().getSaveFile().getName();
//			updateSaveText();
			updateView();
		
		} else if (et.equals(ConfigurationOpenedClosedEvent.EVENT_CLOSE)) {
			// Configuration closed, stop listeing for configuration
			// modifications
//			this.strSaveFile = STR_NONE;
//			updateSaveText();
			updateView();
			getPlugin().getHandler().removeConfigurationModificationListener(this);
			getPlugin().getHandler().removeSaveListener(this);
		}
	}

	public void configurationModified(ConfigurationModifiedEvent e) {
		if (strSaveFile.charAt(strSaveFile.length()-1) != '*' &&
				!strSaveFile.equals(STR_NONE)) { 	// Don't add the asterisk if there's no save file
			strSaveFile += "*";
		}
        
        boolean temp = bComplete;
        
		updateSaveText();
		updateView();
        
        if (!(temp == bComplete)) {
            newComplete = true;
        }
        
        // Shows the pop-up
        if (getPlugin().showCompletenessPopups() && bComplete && newComplete) {
            MessageBox msgbox = new MessageBox(getSite().getShell(), SWT.ICON_WORKING | SWT.OK);
            msgbox.setText("Configuration is complete");
            msgbox.setMessage("Configuration has become complete.");
            
            // Plays the sound
            try {
                InputStream in = StatusView.class.getClassLoader().getResourceAsStream("sounds/complete.wav");
                AudioStream as = new AudioStream(in);
                AudioPlayer.player.start(as); 
            }
            catch (Exception exc) {
                System.out.println(exc.toString());
            }
            newComplete = false;
            msgbox.open();
        }
        
	}

	public void handleSaveEvent(SaveEvent e) {
		this.strSaveFile = e.getSaveFile().getName();
		updateSaveText();
	}

	private void updateSaveText() {
		txtSavefile.setText(strSaveFile);
	}
    
    protected IKumbangPlugin getPlugin() {
        return Activator.getDefault();
    }

}
