/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.client.plugin.wizards;

import java.io.File;

import kumbang.configurator.Activator;
import kumbang.configurator.client.core.ClientServices;
import kumbang.configurator.client.core.exception.ClientException;
import kumbang.configurator.client.plugin.IKumbangPlugin;
import kumbang.configurator.client.plugin.KumbangConfiguratorPlugin;
import kumbang.configurator.client.plugin.PluginHandler;
import kumbang.configurator.client.plugin.preferences.PreferenceConstants;
import kumbang.core.model.exception.ModelException;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import sun.swing.UIAction;

/**
 * Defines the only page of the model initialization wizard
 * 
 * @author jlemmett
 * 
 */
public class InitWizardPage extends WizardPage {
	// JFace / Eclipse
	private ISelection selection;

	// SWT Widgets
	private Text textContainer;

	private Text textFile;

	public Text textModel;

	protected List listModels;

	private Button btnImport;
	
	protected Button btnRemove;
	
	protected ClientServices client;

	private InitWizard wizard;

	/**
	 * Constructor
	 * @param wizard 
	 */
	public InitWizardPage(InitWizard wizard) {
		super("New Kumbang configuration");
		this.setTitle("New Kumbang configuration");
		this.setMessage("This opens a configuration model from the "
				+ "repository and initializes a new configuration.\n"
				+ "You can also import new models to the repository if "
				+ "necessary.", IMessageProvider.INFORMATION);
		this.wizard = wizard;
	}

	/**
	 * Creates the top level control for this dialog page under the given parent
	 * composite.
	 * <p>
	 * Subclasses of WizardPage must implement this method.
	 */
	public void createControl(Composite parent) {
		// Setup the widget container
		Composite container = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 3;
		layout.verticalSpacing = 9;

		getClientServices();
		// Setup the configuration file name widget
		Label label = new Label(container, SWT.NULL);
		label.setText("Name of the new configuration file\n(Without extension):");

		textFile = new Text(container, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;

		textFile.setLayoutData(gd);
		textFile.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				dialogChanged();
			}
		});

		// Setup the model text field widget
		label = new Label(container, SWT.NULL);
		label.setText("Model name:\nSelect from the list below");

		textModel = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		textModel.setLayoutData(gd);
		textModel.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				dialogChanged();
			}
		});

		// Setup the model list widget
		label = new Label(container, SWT.NULL);
		label.setText("Available models on the server:");
		gd = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
		label.setLayoutData(gd);
		listModels = new List(container, SWT.SINGLE | SWT.BORDER | SWT.V_SCROLL);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.heightHint = 200;
		gd.widthHint = 100;
		listModels.setLayoutData(gd);
		refreshModelList();
		
		
		listModels.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				textModel.setText(listModels.getSelection()[0]);
				if (listModels.getSelectionCount() != 0) {
					btnRemove.setEnabled(true);
					
				}
				else {
					btnRemove.setEnabled(false);

				}
			}
			/* This is called when an item in the list is double clicked */
			public void widgetDefaultSelected(SelectionEvent e) {
				if (isPageComplete()) {
					getWizard().performFinish();
					// Is this a good way to close the wizard?
					getWizard().getContainer().getShell().close();
				}
			}
		});

		// Setup a composite for Import and Delete buttons
		Composite btnComposite = new Composite(container, SWT.NONE);
		btnComposite.setLayout(new GridLayout(1,false));
		gd = new GridData();
		gd.verticalAlignment = SWT.TOP;
		btnComposite.setLayoutData(gd);
		
		// Setup the import button
		btnImport = new Button(btnComposite, SWT.PUSH);
		btnImport.setText("Import new models to the server");
		//btnImport.setLayoutData(gd);
		btnImport.addSelectionListener(getSelectionAdapter());
                

		// Don't allow importing if the server is not available
		if (!getPlugin().getHandler().isServerAvailable()) {
			btnImport.setEnabled(false);
		}

		
		createRemoveButton(btnComposite);
		
		
		
		
		IPreferenceStore store = getPlugin().getPreferenceStore();
		// store.getString(PreferenceConstants.P_COMMUNICATIONMETHOD);
		label = new Label(container, SWT.BOLD);
		gd = new GridData();
		gd.verticalAlignment = SWT.BOTTOM;
		label.setLayoutData(gd);
		label.setText("Current communication method: "
				+ store.getString(PreferenceConstants.P_COMMUNICATIONMETHOD));
		initialize();
		dialogChanged();
		setControl(container);
	}

	protected void createRemoveButton(Composite btnComposite) {
		btnRemove = new Button(btnComposite, SWT.PUSH);
		btnRemove.setText("Remove selected model from the server");
		btnRemove.setEnabled(false);
		//btnRemove.setLayoutData(gd);
		btnRemove.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				String modelName = listModels.getSelection()[0];
				
				String message ="Are you sure you want to delete " + modelName + " from the server?";
				int numOfSessions = 1;
				try {
					numOfSessions = getPlugin().getHandler().getClientServices().numberOfSessions();
				} catch (ClientException ce)
				{}

				if (numOfSessions > 1) {
					message += "\nThere are " + (numOfSessions - 1) + " other sessions active at the moment.";
				}
				
				if (MessageDialog.openConfirm(getShell(), "Confirm removal", message)) {
					try {
                        getPlugin().getHandler().getClientServices().removeModel(modelName);
						textModel.setText("");
					} catch (ClientException e1) {
						MessageDialog.openError(getShell(), "Client Exception", "Error removing model");
						e1.printStackTrace();
					} catch (ModelException e1) {
						MessageDialog.openError(getShell(), "Model Exception", "Error removing model");
						e1.printStackTrace();
					}
					finally {
						refreshModelList();
					}

				} else {
					return; // Clicked no
				}
				
			}
		});
	}

    protected Activator getPlugin() {
        return Activator.getDefault();
    }

    protected SelectionListener getSelectionAdapter() {
        return new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {

                // Setup the FileDialog
                // TODO: use some constant instead of a hard coded absolute path
                String path = PluginHandler.getActivator()
                        .getPluginDir("kumbang.configurator");
                        //+ "build/testmodels/";

                FileDialog dlg = new FileDialog(getShell(),
                        SWT.OPEN);
                dlg.setFilterPath(path);
                dlg.setFilterExtensions(new String[] { "*.kbm" });
                dlg.setText("Select a model to import");

                // Get the file
                String filename = dlg.open();
                if (filename != null) {
                    File modelFile = new File(filename);

                    // Ask for the input name and do the importing
                    InputDialog inputDlg = new InputDialog(Display.getCurrent()
                            .getActiveShell(), "Name the model to be imported",
                            "Under which name do you want the model "
                                    + modelFile.getName() + " to be imported?",
                            modelFile.getName().substring(0,
                                    modelFile.getName().length() - 4), null);
                    int rc = inputDlg.open();
                    if (rc == Window.OK) {
                        try {
                            client.importModel(modelFile,
                                    inputDlg.getValue());
                            refreshModelList();
                        } catch (ClientException ce) {
                            MessageDialog.openError(getShell(),
                                    "Client Error importing model",
                                    "ClientException: " + ce.getMessage());
                        } catch (ModelException me) {
                            if (me.getMessage().indexOf("named") != -1) {
                                MessageDialog.openInformation(getShell(), "Model already on server", 
                                        "The model you are trying to import already exists on the server. If you want to refresh it, delete the old one and then import it again.");
                            } else {
                                MessageDialog.openError(getShell(),
                                        "Model Error importing model", me.getMessage());
                            }
                        }
                    } else {
                        System.out.println("User canceled");
                    }
                } else {
                    System.out.println("User canceled");
                }
            }
        };


    }

    protected void getClientServices() {
        client = Activator.getDefault().getHandler().getClientServices();
    }

	protected void refreshModelList() {
		listModels.setItems(client.getModelNames());
	}

	private void initialize() {
		if (selection != null && selection.isEmpty() == false
				&& selection instanceof IStructuredSelection) {
			IStructuredSelection ssel = (IStructuredSelection) selection;
			if (ssel.size() > 1)
				return;
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource) {
				IContainer container;
				if (obj instanceof IContainer)
					container = (IContainer) obj;
				else
					container = ((IResource) obj).getParent();
				textContainer.setText(container.getFullPath().toString());
			}
		}
		if (listModels.getItemCount() > 0) {
			textModel.setText(listModels.getItem(0).toString());
		}
		textFile.setText("NewConfiguration");
	}

	private void dialogChanged() {

		String fileName = getFileName();
		String modelName = getModelname();
		List listModels = getModelList();

		if (fileName.length() == 0) {
			updateStatus("File name must be specified");
			return;
		}
		if (fileName.replace('\\', '/').indexOf('/', 1) > 0) {
			updateStatus("File name must be valid");
			return;
		}

		if (fileName.lastIndexOf('.') != -1) {
			updateStatus("Configuration name mustn't contain a file extension");
			return;
		}

		if (modelName.length() == 0) {
			this.updateStatus("Model name must be specified");
			return;
		}
		if (listModels.indexOf(modelName) == -1) {
			updateStatus("The model "
					+ modelName
					+ " is not available on the server. Select one on the list or import a new one.");
			return;
		}
		if (!getPlugin().getHandler().isServerAvailable()) {
			updateStatus("The configuration server is not available!");
			btnImport.setEnabled(false);
			return;
		} else {
			btnImport.setEnabled(true);
		}
		// everything is OK
		updateStatus(null);
	}

	private List getModelList() {
		return this.listModels;
	}

	private void updateStatus(String message) {
		// setErrorMessage(message);
		setMessage(message, IMessageProvider.ERROR);
		if (message == null)
			setMessage("Create a new configuration based on this model.",
					IMessageProvider.INFORMATION);
		setPageComplete(message == null);
	}

	public String getContainerName() {
		return textContainer.getText();
	}

	public String getFileName() {
		return textFile.getText();
	}

	public String getModelname() {
		return textModel.getText();
	}

}
