/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * ServerProperties.java
 *
 * Created on 26. elokuuta 2004, 10:15
 */

package kumbang.configurator.server.core;

import java.io.File;
import java.io.IOException;

import java.util.Properties;


/**
 *
 * @author  vmyllarn
 */
public class ServerProperties {
    
	public static final String ROOTDIR = "server_rootdir";
	public static final String BIN_PATH = "server_bin_path";
	public static final String LOG_PATH = "server_log_path";
	public static final String MODEL_PATH = "server_model_path";
    public static final String MODEL_SUFFIX = "server_model_suffix";
	
	public static final String SERVER_PORT = "server_port";
	public static final String SERVER_HOST = "server_host";
	
	public static final String PARSER_CLASS = "server_parser_class";

	public static final String DEFAULT_ROOTDIR = new File(".").getAbsolutePath();
	public static final int DEFAULT_PORT = 6969;
	public static final String DEFAULT_HOST = "localhost";
	public static final String DEFAULT_BIN_PATH = "bin";
	public static final String DEFAULT_LOG_PATH = "logs";
	public static final String DEFAULT_MODEL_PATH = "models";
    public static final String DEFAULT_MODEL_SUFFIX = ".kbm";
	public static final String DEFAULT_PARSER_CLASS = "kumbang.core.language.parser.ModelParser";	

    
	
    protected Properties properties;
    private static ServerProperties serverProperties;
	
    /** Creates a new instance of ServerProperties */
    private ServerProperties(String fileName) {
    	this();
    	try {
    		properties.load(ServerProperties.class.getClassLoader()
    				.getResourceAsStream(fileName));
    	} catch (IOException ioe) {
    		setRootDir(DEFAULT_ROOTDIR);
    		setServerAddress(DEFAULT_HOST);
    		setServerPort(DEFAULT_PORT);
    		setBinPath(DEFAULT_BIN_PATH);
    		setLogPath(DEFAULT_LOG_PATH);
    		setModelPath(DEFAULT_MODEL_PATH);
            setModelSuffix(DEFAULT_MODEL_SUFFIX);
    		setParserClass(DEFAULT_PARSER_CLASS);
    	}
    }

	private ServerProperties() {
		properties = new Properties();
		setRootDir(DEFAULT_ROOTDIR);
		setServerAddress(DEFAULT_HOST);
		setServerPort(DEFAULT_PORT);
		setBinPath(DEFAULT_BIN_PATH);
		setLogPath(DEFAULT_LOG_PATH);
		setModelPath(DEFAULT_MODEL_PATH);
        setModelSuffix(DEFAULT_MODEL_SUFFIX);
		setParserClass(DEFAULT_PARSER_CLASS);		
    }


	public static ServerProperties getInstance(String fileName) {
		if (serverProperties == null) {
			serverProperties = new ServerProperties(fileName);
		}
		return serverProperties;
	}
	public static ServerProperties getInstance() {
		if (serverProperties == null) {
			serverProperties = new ServerProperties();
		}
		return serverProperties;
	}
	
	/**
	 * Set the root directory of the server, needed 
	 * especially when using a local server
	 * @param dir
	 */
	public void setRootDir(String dir) {
		properties.setProperty(ROOTDIR, dir);
	}

	public String getRootDir() {
		return properties.getProperty(ROOTDIR);
	}
	
	public void setServerPort(int port) {
		properties.setProperty(SERVER_PORT, Integer.toString(port));
	}
	
	public int getServerPort() {
		return Integer.parseInt(properties.getProperty(SERVER_PORT));
	}
    
	public void setServerAddress(String addr) {
		properties.setProperty(SERVER_HOST, addr);
	}
	
	public String getServerAddress() {
		return properties.getProperty(SERVER_HOST);
	}

	public void setBinPath(String path) {
		properties.setProperty(BIN_PATH, path);
	}
	
	public String getBinPath() {
		return properties.getProperty(BIN_PATH);
	}

	public void setLogPath(String path) {
		properties.setProperty(LOG_PATH, path);
	}	
	
	public String getLogPath() {
		return properties.getProperty(LOG_PATH);
	}
	
	public void setModelPath(String path) {
		properties.setProperty(MODEL_PATH, path);
	}
	
	public String getModelPath() {
		return properties.getProperty(MODEL_PATH);
	}
    
    public void setModelSuffix(String str) {
        String suffix = str;
        
        if (!str.startsWith("."))
            suffix = "." + suffix;
        
        properties.setProperty(MODEL_SUFFIX, suffix);
    }
    
    public String getModelSuffix() {
        return properties.getProperty(MODEL_SUFFIX);
    }
	
	public void setParserClass(String path) {
		properties.setProperty(PARSER_CLASS, path);
	}
	
	public String getParserClass() {
		return properties.getProperty(PARSER_CLASS);
	}
	
	public String currentProperties() {
		String str = "Host: " + getServerAddress() + "\n" +
        "Port: " + getServerPort() + "\n" +
		"Binpath: " + getBinPath() + "\n" +
		"Log path: " + getLogPath() + "\n" +
        "Model suffix: " + getModelSuffix() + "\n" +
		"Model path: " + getModelPath() + "\n" + 
		"Parser class: " + getParserClass() + "\n";
		return str;
	}
    
    /**
     * Resets ServerProperties.
     * NOTE: There should be no reason to call this method
     * within the KumbangTools. 
     *
     */
    public void resetProperties() {
        serverProperties = null;
    }
}
