/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * SessionManager.java
 *
 * Created on 18. huhtikuuta 2004, 18:12
 */

package kumbang.configurator.server.services;

import java.util.HashMap;

/**
 *
 * @author  Administrator
 */
public class SessionManager implements Runnable {
   
    public static final int MAX_SESSIONS = 20;
    
    private HashMap sessions;
    
    private int numberOfSessions;
    
    private int maxSessions;
    
    private int idCounter = 0;
    
    /** Creates a new instance of SessionManager */
    public SessionManager(int maxSessions) {
        this.maxSessions = maxSessions;
        this.numberOfSessions = 0;
        this.sessions = new HashMap();
    }
    
    public SessionManager() {
        this(MAX_SESSIONS);
    }

    public synchronized RemoteSession startSession() {
        if (numberOfSessions < maxSessions) {
            this.numberOfSessions++;
            Session session = new Session(idCounter++);
            RemoteSession rSession = session.getRemoteSession();
            this.sessions.put(rSession, session);
            return rSession;
        }
        else {
            return null;
        }
    }
    
    public synchronized void endSession(RemoteSession rSession) {
        Session session = (Session)sessions.get(rSession);
        if (session == null) {
            return;
        }
        sessions.remove(rSession);
        numberOfSessions--;
    }
    
    public synchronized void endAllSessions() {
    	sessions.clear();
    	numberOfSessions = 0;
    }
    
    public synchronized Session getSession(RemoteSession rSession) {
        Session session = (Session)sessions.get(rSession);
        if (session == null) {
            return null;
        }
        else {
            session.refresh();
            return session;
        }
    }
    
    public int getNumberOfSessions() {
    	return numberOfSessions;
    }
    
    public void run() {
        // here we should go through sessions and send them ping
        // or check timestamps?
    }
    
}
