/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * SmComputeStatement.java
 *
 * Created on 28. joulukuuta 2004, 11:58
 */

package kumbang.configurator.server.smodels;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 *
 * @author  vmyllarn
 */
public class ComputeStatement {
	public static final String COMPUTE_START = "compute {";
	public static final String COMPUTE_MIDDLE = ", ";
	public static final String COMPUTE_END = " }.";

	public static final String COMPUTE_NEG = "not ";

	public static final String INST_START = "in(i";
	public static final String INST_END = ")";

	public static final String BIND_START = "cn(i";
	public static final String BIND_MIDDLE = ",i";
	public static final String BIND_END = ")";

	public static final String ATTR_START = "hasattr(i";
	public static final String ATTR_MIDDLE = ",";
	public static final String ATTR_END = ")";
	
	private List<String> pos;
	private List<String> neg;

	/** Creates a new instance of ComputeStatement */
	public ComputeStatement() {
		pos = new ArrayList<String>();
		neg = new ArrayList<String>();
	}

	public void reset() {
		pos.clear();
		neg.clear();
	}

	public void addInstance(int instanceID) {
		String pred = INST_START + instanceID + INST_END;
		addPositive(pred);
	}

	public void addBinding(int sourceID, int targetID) {
		String pred = BIND_START + sourceID + BIND_MIDDLE + targetID + BIND_END;
		addPositive(pred);
	}

	public void setAttribute(int instanceID, String name, Object value) {
		String pred = ATTR_START + instanceID + ATTR_MIDDLE + NameMap.getAttributeName(name)
				+ ATTR_MIDDLE + NameMap.getAttributeValue(value.toString()) + ATTR_END;
		addPositive(pred);
	}

	public void removeInstance(int instanceID) {
		String pred = INST_START + instanceID + INST_END;
		addNegative(pred);
	}
    
	public void removeBinding(int sourceID, int targetID) {
		String pred = BIND_START + sourceID + BIND_MIDDLE + targetID + BIND_END;
		addNegative(pred);
	}

	private void addPositive(String predicate) {
		if (!pos.contains(predicate))
			pos.add(predicate);
	}
	
	private void addNegative(String predicate) {
		if (!neg.contains(predicate))
			neg.add(predicate);
	}
	
	public List<String> getPositivePart() {
		return pos;
	}
	
	public List<String> getNegativePart() {
		return neg;
	}
	
	public String toString() {
		String ret = "Positive: {";
		for (Iterator it = pos.iterator(); it.hasNext(); ) {
			ret += it.next().toString();
			if (it.hasNext()) {
				ret += ", ";
			} else {
				ret += "}";
			}
		}
		ret += " Negative: ";
		for (Iterator it = neg.iterator(); it.hasNext(); ) {
			ret += it.next().toString();
			if (it.hasNext()) {
				ret += ", ";
			} else {
				ret += "}";
			}
		}
		return ret;
	}
}
