/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * SmodelsTester.java
 *
 * Created on 13. toukokuuta 2004, 12:26
 */

package kumbang.configurator.util.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;

import kumbang.configurator.server.core.ConfigurationHandler;
import kumbang.configurator.server.core.ModelManager;
import kumbang.configurator.server.services.KumbangConfigurationServer;
import kumbang.configurator.server.smodels.Translator;
import kumbang.core.configuration.core.Configuration;
import kumbang.core.configuration.core.ConfigurationException;
import kumbang.core.configuration.task.ConfigurationResponse;
import kumbang.core.language.parser.ModelParser;
import kumbang.core.model.core.Model;
import kumbang.core.model.exception.ModelException;

/**
 *
 * @author  vmyllarn
 */
public class SmodelsTester {

	/** Creates a new instance of SmodelsTester */
	public SmodelsTester() {}

	/**
	 * @param args the command line arguments
	 */
	public static void main(String[] args) throws IOException {
		File inputFile;

		if (args.length == 0) {
			System.out.print("Give name of the input file: ");
			BufferedReader buf = new BufferedReader(new InputStreamReader(System.in));
			inputFile = new File(buf.readLine());
		} else
			inputFile = new File(args[0]);
		
		if (inputFile == null)
			return;

		try {
			System.out.println("Parsing model from file " + inputFile);
			ModelParser parser = new ModelParser();
			Model model = parser.parseModel(inputFile);

			System.out.println("Creating lparse format to file test.lp");
			String lparseContent = Translator.Translate(model);
			File lpFile = new File("test.lp");
			PrintWriter out = new PrintWriter(new FileWriter(lpFile));
			out.println(lparseContent);
			out.close();

			System.out.println("Creating grounded smodels format to file test.sm");
			File smFile = new File("test.sm");
			ModelManager.ground(lpFile, smFile);

			ConfigurationHandler ch = new ConfigurationHandler(KumbangConfigurationServer.logger);
			ch.init(smFile);

			System.out.println("Invoking smodels");
			System.out.println("\n");
			
			long start, end;
			start = System.currentTimeMillis();
			ConfigurationResponse response = ch.getConsequences(new Configuration(model));		
			end = System.currentTimeMillis();
			
			System.out.println("smodels returned: " + response.isConsistent());
			System.out.println("Invoking smodels engine took " + (end - start) + " ms.");
		} catch (ModelException me) {
			System.out.println("Model was not valid: " + me);
		} catch (IOException ioe) {
			System.out.println("Error in configuration: " + ioe);
		} catch (ConfigurationException e) {
			System.out.println("A ConfigurationException occured: " + e);
		}
	}
}
