/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on Apr 11, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.configurator.server.smodels;

import kumbang.core.configuration.task.AttributeHasValueConsequence;
import kumbang.core.configuration.task.ConfigurationConsequence;
import kumbang.core.configuration.task.InstanceExistsConsequence;
import kumbang.core.configuration.task.InterfacesBoundConsequence;

import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.StringTokenizer;
import java.util.List;
import java.util.LinkedList;
import java.util.TreeSet;

/**
 * @author toasikai
 *
 */
public class ConfigurationState {
	private boolean isConsistent;
	private boolean isComplete;
	
	/** The atoms that are known to be true in the configuration. */ 
	private SortedSet<ConfigurationConsequence> pos;
	
	/** The atoms that are known to be false in the configuration. */
	private SortedSet<ConfigurationConsequence> neg;
	
	/** A set of atoms that constitutes a valid configuration (if not null). */
	private List<ConfigurationConsequence> complete;
	
	public ConfigurationState() {}
	
	public void setInconsistent() {
		this.isConsistent = false;
		isComplete = false;
		
		pos = null;
		neg = null; 
	}
	
	/**
	 * This method is called from within the smodels module (DLL or .so). 
	 * 
	 * @param isComplete
	 * @param pos
	 * @param neg
	 * @param complete
	 */
	public void setConsistentState(boolean isComplete, String[] pos, String[] neg, String[] complete) {
		this.isConsistent = true;
		this.isComplete = isComplete;
		
		this.pos = new TreeSet<ConfigurationConsequence>();
		this.neg = new TreeSet<ConfigurationConsequence>();
		this.complete = new LinkedList<ConfigurationConsequence>();
		
		for (int i = 0; i < pos.length; i++) {
			ConfigurationConsequence cc = parseAtom(pos[i]);
			if (cc != null) {
				this.pos.add(cc);
			}
		}
		
		for (int i = 0; i < neg.length; i++) {
			ConfigurationConsequence cc = parseAtom(neg[i]);
			if (cc != null)
				this.neg.add(cc);
		}
		
		for (int i = 0; i < complete.length; i++) {
			ConfigurationConsequence cc = parseAtom(complete[i]);
			if (cc != null)
				this.complete.add(cc);
		}		
	}

	/**
	 * @return
	 */
	public boolean isComplete() {
		return isComplete;
	}
	
	public boolean isConsistent() {
		return isConsistent;
	}
	
	private ConfigurationConsequence parseAtom(String str) {
		try {
			//	An in(X) atom
			if (str.startsWith("in(i") && str.endsWith(")")) {
				int id = Integer.parseInt(str.substring(4, str.length() - 1));
				return new InstanceExistsConsequence(id);
			}
			
			//	A hasattr(X, Y, Z) atom
			if (str.startsWith("hasattr(i") && str.endsWith(")")) {
				str = str.substring(9, str.length() - 1);
				StringTokenizer tokens = new StringTokenizer(str, ",");
				//if (tokens.countTokens() != 3) {
				//    return null;
				//}
				int id = Integer.parseInt(tokens.nextToken());
				return new AttributeHasValueConsequence(
					id,
					tokens.nextToken(),
					tokens.nextToken());
			} 
			
			//	A cn(X, Y) atom
			if (str.startsWith("cn(i") && str.endsWith(")")) {
				str = str.substring(4, str.length() - 1);
				StringTokenizer tokens = new StringTokenizer(str, ",i");
				int id1 = Integer.parseInt(tokens.nextToken());
				int id2 = Integer.parseInt(tokens.nextToken());
				return new InterfacesBoundConsequence(id1, id2);
			}
		} catch (NumberFormatException nfe) {}

		return null;
	}
	
	public Iterator<ConfigurationConsequence> getPositivePart() {
		return pos.iterator(); 
	}
	
	public Iterator<ConfigurationConsequence> getNegativePart() {
		return neg.iterator(); 
	}

	public Iterator<ConfigurationConsequence> getCompleteConfiguration() {
		return complete.iterator();
	}	
}
