/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.configurator.server.smodels;

import kumbang.core.model.type.AttributeDefinition;
import kumbang.core.model.type.AttributeValueType;
import kumbang.core.model.type.ComponentType;
import kumbang.core.model.type.ElementaryType;
import kumbang.core.model.type.FeatureType;
import kumbang.core.model.type.InterfaceDefinition;
import kumbang.core.model.type.InterfaceType;
import kumbang.core.model.type.PartDefinition;

import java.util.HashMap;

/**
 * A class that maps names appearing in Kumbang models into names that can be used in lparse
 * input files.
 * 
 * @author  vmyllarn
 */
public class NameMap {
	private static final String ATTRVALUE = "av";
	private static final String OTHER = "other";
	private static final String ATTRDEF = "attrDef";
	private static final String INTDEF = "intDef";
	private static final String PARTDEF = "partDef";
	
	// prefixes for renaming
	public static final String COMPONENT = "comp";
	public static final String FEATURE = "feat";
	public static final String INTERFACE = "int";
	public static final String ATTRIBUTE = "attr";

	public static String getSmName(ElementaryType type) {
		if (type instanceof ComponentType) {
			return getSmName((ComponentType) type);
		} else if (type instanceof FeatureType) {
			return getSmName((FeatureType) type);
		} else if (type instanceof InterfaceType) {
			return getSmName((InterfaceType) type);
		} else {
			return null;
		}
	}

	public static String getSmName(ComponentType type) {
		return COMPONENT + type.getName();
	}

	public static String getSmName(FeatureType type) {
		return FEATURE + type.getName();
	}

	public static String getSmName(InterfaceType type) {
		return INTERFACE + type.getName();
	}

	public static String getSmName(AttributeValueType type) {
		return ATTRIBUTE + type.getName();
	}

	public static String getSmName(PartDefinition def) {
		return PARTDEF + def.getName();
	}

	public static String getSmName(InterfaceDefinition def) {
		return INTDEF + def.getName();
	}

	//	Attribute definitions are mapped with their names to avoid name clashes.
	public static String getSmName(AttributeDefinition def) {
		return ATTRDEF + def.getName();
	}
	
	public static String getSmName(String name) {
		return OTHER + name;
	}

	public static String getSmNameForAttributeValue(String value) {	//	Object value
		// Only prefix if the value starts with an uppercase letter. 
		return getAttributeValue(value);
	}

	public static String getAttributeName(String name) {
		return ATTRDEF + name;
	}

	/*
	 * TODO Change the methods taking types and definitions as arguments use these
	 * versions. -toasikai 7.8.2006
	 */
	public static String getInterfaceName(String name) {
		return INTDEF + name;
	}

	public static String getPartName(String name) {
		return PARTDEF + name;
	}

	public static String getAttributeValue(String value) {
		if (Character.isDigit(value.charAt(0))) 
			return value;
		
		return ATTRVALUE + value;
	}
}
