/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.core;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

/**
 * The activator class controls the plug-in life cycle
 */
public class Activator extends Plugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "kumbang.core";

	// The shared instance
	private static Activator plugin;
	
	/**
	 * The constructor
	 */
	public Activator() {
		plugin = this;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.core.runtime.Plugins#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.core.runtime.Plugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static Activator getDefault() {
		return plugin;
	}
	
	public static String getPluginDir(String pluginId) throws IOException {

		if (Platform.isRunning()) {
			URL relativeURL = Platform.getBundle(pluginId).getEntry("/");
			URL localURL;
			localURL = FileLocator.toFileURL(relativeURL);
			return localURL.getPath();
		} else {
			// If Eclipse is not running, we have to do tricks..
			// Expects that the projects are on the same directory level eg
			// -projects
			// +-kumbang.core
			// +-kumbang.configurator
			File runningDir = new File("").getAbsoluteFile();
			if (runningDir.getAbsolutePath().equals(File.separator)) {
				// Fitnesse runs with '/' as its running directory
				runningDir = new File("/usr/local/cruisecontrol-bin-2.5/projects/", pluginId);
			} else {
				// Non-fitnesse things like SmodelsTester run with the assosiated project as their
				// running directory
				runningDir = new File(runningDir.getAbsoluteFile().getParent().concat(File.separator + pluginId));
			}
			return runningDir.toString();
			
			// This also happens when running some standalone stuff like the 
			// Smodels-tester
		}

		
		
	}
	

}
