/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on 8.11.2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package kumbang.core.configuration.description;

import kumbang.core.configuration.core.Configuration;

import kumbang.core.configuration.instance.AttributeInstance;
import kumbang.core.configuration.instance.BindingInstance;
import kumbang.core.configuration.instance.ComponentInstance;
import kumbang.core.configuration.instance.ComposableInstance;
import kumbang.core.configuration.instance.FeatureInstance;
import kumbang.core.configuration.instance.Instance;
import kumbang.core.configuration.instance.InterfaceInstance;

//import kumbang.core.model.type.InterfaceType;

import java.io.FileWriter;
import java.io.IOException;
import java.io.File;
import java.io.PrintWriter;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * @author vmyllarn
 */
public class KumbangConfigurationDescription extends ConfigurationDescription {

	private PrintWriter out;

    /**
     * Creates a new KumbangConfigurationDescription
     */
    public KumbangConfigurationDescription() {
    	super();
    }
    
    /**
     * @param exportFile
     * @see kumbang.core.configuration.description.ConfigurationDescription#generate(java.io.PrintWriter)
     */
    public void describe(File exportFile, List<Instance> roots, String configurationName) throws IOException {
        this.out = new PrintWriter(new FileWriter(exportFile));
        
        for (Instance instance : roots) {
        	out.println("//-------- " + instance.getInstanceKind() +"s ---------------------------");
        	describeInstance(instance);
        }

        out.close();
        this.out = null;
    }

    private void describeInstance(Instance instance) {
    	if (instance instanceof ComponentInstance) {
    		describeComponent((ComponentInstance) instance);
    	} else if (instance instanceof FeatureInstance) {
    		describeFeature((FeatureInstance) instance);
    	} else if (instance instanceof InterfaceInstance) {
    		//describeInterface((InterfaceInstance) instance);
    	}
    }

    private void describeComponent(ComponentInstance cInstance) {
        if (cInstance == null) {
            return;
        }

        /* collect part components in a list */
        List parts = new LinkedList();
        
        /* collect bindings in a list */
        List bindings = new LinkedList();
        
        /* start description */
        out.println("component " + cInstance.getNameWithIndex() 
                + ":" + cInstance.getType().getName() + " {");

        /* describe attributes */
        describeAttributes(cInstance);

        /* describe interfaces */
        describeInterfaces(cInstance);

        /* collect part instances and bindings */
        Iterator iter = cInstance.getPartInstances(); // getInstancesAsPart();
        if (iter.hasNext()) {
            out.println(INDENT + "contains");
        }
        while (iter.hasNext()) {
            ComponentInstance c = (ComponentInstance) iter.next();
            out.println(INDENT + INDENT + c.getType().getName() + " " 
                        + c.getNameWithIndex() + ";");
            parts.add(c);
            // search for bindings
            Iterator it = c.getInterfaces();
            while (it.hasNext()) {
                InterfaceInstance i = (InterfaceInstance) it.next();
                Iterator it2 = i.getBindings();
                while (it2.hasNext()) {
                    BindingInstance bind = (BindingInstance) it2.next();
                    if (!bindings.contains(bind)) {
                        bindings.add(bind);
                    }
                }
            }
        }

        /* describe bindings */
        if (!bindings.isEmpty()) {
            out.println(INDENT + "connects");
            Iterator it = bindings.iterator();
            while (it.hasNext()) {
                BindingInstance bind = (BindingInstance) it.next();
                InterfaceInstance source = bind.getSourceInterface();
                ComponentInstance cSource = source.getComponent();
                InterfaceInstance target = bind.getTargetInterface();
                ComponentInstance cTarget = target.getComponent();
                String strS = "", strT = "";
                if (cSource != cInstance) {
                    strS += cSource.getName() + ".";
                }
                strS += source.getName();
                if (cTarget != cInstance) {
                    strT += cTarget.getName() + ".";
                }
                strT += target.getName();
                out.println(INDENT + INDENT + strS + " = " + strT + ";");
            }
        }

        /* ending brace */
        out.println("}");
        out.println();
        
        /* describe part components */
        for (Iterator it = parts.iterator(); it.hasNext(); ) {
            describeComponent((ComponentInstance) it.next());
        }
    }

    private void describeFeature(FeatureInstance fInstance) {
        if (fInstance == null) {
            return;
        }
 
        /* collect part instances in a list */
        List parts = new LinkedList();
        
        /* start description */
        out.println("feature " + fInstance.getNameWithIndex()
                + ":" + fInstance.getType().getName() + " {");

        /* describe attributes */
        describeAttributes(fInstance);

        /* collect part instances */
        Iterator iter = fInstance.getPartInstances(); // getInstancesAsPart();
        if (iter.hasNext()) {
            out.println(INDENT + "subfeatures");
        }
        while (iter.hasNext()) {
            FeatureInstance f = (FeatureInstance) iter.next();
            out.println(INDENT + INDENT + f.getType().getName() + " " 
                        + f.getNameWithIndex() + ";");
            parts.add(f);
        }

        /* ending brace */
        out.println("}");
        out.println();
        
        /* describe part features */
        for (Iterator it = parts.iterator(); it.hasNext(); ) {
            describeFeature((FeatureInstance) it.next());
        }
    }

    
    private void describeInterfaces(ComponentInstance cInstance) {
        /* provided interfaces */
        Iterator iter = cInstance.getInterfaces();
        boolean hasProvided = false;
        while (iter.hasNext()) {
            InterfaceInstance iInstance = (InterfaceInstance) iter.next();
            if (iInstance.isProvided()) {
                if (!hasProvided) {
                    out.println(INDENT + "provides");
                    hasProvided = true;
                }
                out.println(INDENT + INDENT 
                            + iInstance.getType().getName() + " " 
                            + iInstance.getName() + ";");
            }
        }

        /* required interfaces */
        iter = cInstance.getInterfaces();
        boolean hasRequired = false;
        while (iter.hasNext()) {
            InterfaceInstance iInstance = (InterfaceInstance) iter.next();
            if (!iInstance.isProvided()) {
                if (!hasRequired) {
                    out.println(INDENT + "requires");
                    hasRequired = true;
                }
                out.println(INDENT + INDENT 
                        + iInstance.getType().getName() + " " 
                        + iInstance.getName() + ";");
            }
        }
    }

    private void describeAttributes(ComposableInstance instance) {
        if (instance == null || !instance.hasAttributes()) {
            return;
        }
        out.println(INDENT + "attributes");
        Iterator iter = instance.getAttributes();
        while (iter.hasNext()) {
            AttributeInstance attr = (AttributeInstance) iter.next();
            out.println(INDENT + INDENT + attr.getName() + " = "
                    + attr.getValue() + ";");
        }
    }


}
