/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on 8.11.2005
 *
 */
package kumbang.core.configuration.description;


import kumbang.core.configuration.instance.AttributeInstance;
import kumbang.core.configuration.instance.BindingInstance;
import kumbang.core.configuration.instance.ComponentInstance;
import kumbang.core.configuration.instance.ComposableInstance;
import kumbang.core.configuration.instance.FeatureInstance;
import kumbang.core.configuration.instance.Instance;
import kumbang.core.configuration.instance.InterfaceInstance;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.StringWriter;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * @author vmyllarn
 */
public class XMLConfigurationDescription extends ConfigurationDescription {
	

	/**
	 * Creates a new XMLConfigurationDescription
	 */
	public XMLConfigurationDescription() {
		super();
	}	
	
	/**
	 * This is a function used to get String of XML-document for Fitnesse. 
	 *
	 */
	public String getXMLString(String confname, String modelName, ComponentInstance cInstance,
			FeatureInstance fInstance) throws IOException {
		
		// Luodaan root-elementti (configuraatio) ja sille lapset (komponentti ja feature) ja attribuutit:
		StringWriter str = null;
		Element root = new Element("configuration");
		Element comp = new Element("component");
		Element feat = new Element("feature");
		
		Attribute atr = new Attribute("name", confname);
		//Attribute attr = new Attribute("model", modelName);
		
		List<Attribute> attributes = new ArrayList<Attribute>();
		attributes.add(atr);
		//attributes.add(attr);
		
		root.setAttributes(attributes);

		if (fInstance != null)
			root.addContent(feat);
		
		if (cInstance != null)
			root.addContent(comp);
		

		
		// Luodaan lapset komponentti- ja feature -elementeille:
		exportFeature(feat, fInstance);		
		exportComponent(comp, cInstance);

		
		Document doc = new Document(root);
		
		str = new StringWriter();
		XMLOutputter serializer = new XMLOutputter();
		Format style = Format.getCompactFormat();
		style.setIndent(INDENT);
		serializer.setFormat(style);
		serializer.output(doc, str);

		
		return str.toString();
	}
	
	// This method is for exporting a configuration into xml
	
	public void describe(File exportFile, List<Instance> roots, String configurationName) throws IOException {
		
		// Creates root-element (configuration) and children for the root element
		// (component and feature root)
		
		Element root = new Element("configuration");
		Attribute atr = new Attribute("name", configurationName);
		List<Attribute> attributes = new ArrayList<Attribute>();
		attributes.add(atr);
		root.setAttributes(attributes);		
		
		for (Instance inst : roots) {
			Element elem = new Element(inst.getInstanceKind());
			root.addContent(elem);
			exportInstance(elem, inst);
		}
//		Element comp = new Element("Component");
//		Element feat = new Element("Feature");
//		if (cInstance != null)
//			root.addContent(comp);
//		if (fInstance != null)
//			root.addContent(feat);
//		exportComponent(comp, cInstance);
//		exportFeature(feat, fInstance);
		
		Document doc = new Document(root);
		
		// And then lets write the whole document into a file
		FileOutputStream outF = new FileOutputStream(exportFile);
		XMLOutputter serializer = new XMLOutputter();
		Format style = Format.getCompactFormat();
		style.setIndent(INDENT);
		serializer.setFormat(style);
		serializer.output(doc, outF);
		outF.close();
		
	}
	
	private void exportInstance(Element elem, Instance instance) {
		if (instance instanceof ComponentInstance) {
			exportComponent(elem, (ComponentInstance) instance);
		} else if (instance instanceof FeatureInstance) {
			exportFeature(elem, (FeatureInstance) instance);
		} else if (instance instanceof InterfaceInstance) {
			exportInterface(elem, (InterfaceInstance) instance);
		}
	}
	
	private void exportComponent(Element comp, ComponentInstance cInstance) {
		
		// Creates the component tree recursively
		
		if (cInstance == null) {
			return;
		}
		
		// Attributes:
		
		List<Attribute> lista = new ArrayList<Attribute>();
		Attribute atr = new Attribute("type", cInstance.getType().getName());
		Attribute attr = new Attribute("name", cInstance.getName());
		lista.add(atr);
		lista.add(attr);
		
		comp.setAttributes(lista);
		exportAttributes(comp, cInstance);
		
		// Interfaces:
		
		Iterator iter = cInstance.getInterfaces();
		while (iter.hasNext()) {
			InterfaceInstance iInstance = (InterfaceInstance) iter.next();
			exportInterface(comp, iInstance);
		}
		
		// Component children:
		
		iter = cInstance.getPartInstances();
		while (iter.hasNext()) {
			ComponentInstance c = (ComponentInstance) iter.next();
			Element elem = new Element("component");
			comp.addContent(elem);
			exportComponent(elem, c);
		}
	}
	
	private void exportAttributes(Element comp, ComposableInstance instance) {
		
		// Creates attribute instances for the features
		
		if (instance == null || !instance.hasAttributes()) {
			return;
		}
		Iterator iter = instance.getAttributes();
		while (iter.hasNext()) {
			AttributeInstance att = (AttributeInstance) iter.next();
			Element atrib = new Element("attribute");
			
			Attribute atr = new Attribute("name", att.getName());
			Attribute attr = new Attribute("value", att.getValue());
			List<Attribute> atributes = new ArrayList<Attribute>();
			atributes.add(atr);
			atributes.add(attr);
			atrib.setAttributes(atributes);
			
//			atr.setAttribute("value", att.getValue());
			comp.addContent(atrib);
			
		}
	}
	
	private void exportInterface(Element comp, InterfaceInstance iInstance) {
		
		// Creates interfaces
		
		if (iInstance == null) {
			return;
		}
		
		Element intr = new Element("interface");
		
		String str;
		
		if (iInstance.isProvided()) {
			str = "provided";
		} else {
			str = "required";
		}
		
		Attribute atr = new Attribute("type", iInstance.getType().getName());
		Attribute attr = new Attribute("name", iInstance.getName());
		Attribute atttr = new Attribute("direction", str);
		
		List<Attribute> atributes = new ArrayList<Attribute>();
		atributes.add(atr);
		atributes.add(attr);
		atributes.add(atttr);
		
		intr.setAttributes(atributes);
		
		comp.addContent(intr);
		
		Iterator iter = iInstance.getFunctions();
		while (iter.hasNext()) {
			Element fnct = new Element("function");
			fnct.setAttribute("name", (String) iter.next());
			intr.addContent(fnct);
		}
		
		iter = iInstance.getBindings();
		while (iter.hasNext()) {
			BindingInstance bind = (BindingInstance) iter.next();
			Element bindi = new Element("binding");
			
			atr = new Attribute("source", bind.getSourceInterface().getName());
			attr = new Attribute("target", bind.getTargetInterface().getName());
			atributes = new ArrayList<Attribute>();
			atributes.add(atr);
			atributes.add(attr);
			
			bindi.setAttributes(atributes);
			intr.addContent(bindi);
			
		}
		
	}
	
	private void exportFeature(Element feat, FeatureInstance fInstance) {
		
		// Creates features
		
		if (fInstance == null) {
			return;
		}
		
		List<Attribute> lista = new ArrayList<Attribute>();
		Attribute at = new Attribute("type", fInstance.getType().getName());
		Attribute att = new Attribute("name", fInstance.getName());
		lista.add(at);
		lista.add(att);
		
		feat.setAttributes(lista);
		
		exportAttributes(feat, fInstance);
		
		Iterator iter = fInstance.getPartInstances();
		while (iter.hasNext()) {
			FeatureInstance f = (FeatureInstance) iter.next();
			Element ff = new Element("feature");
			feat.addContent(ff);
			exportFeature(ff, f);
		}
	}
}
