/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * AttributeInstance.java
 *
 * Created on 2. maaliskuuta 2004, 12:58
 */

package kumbang.core.configuration.instance;

import java.util.Iterator;

import kumbang.core.configuration.reference.AttributeReference;
import kumbang.core.configuration.reference.ComposableReference;
import kumbang.core.configuration.reference.Reference;
import kumbang.core.model.type.AttributeDefinition;
import kumbang.core.model.type.AttributeValueType;
import kumbang.core.model.type.ElementaryDefinition;
import kumbang.core.model.type.ElementaryType;
import kumbang.core.model.type.Name;

/**
 * This class could be alright, but there's not need for it to contain anything else but the Name
 * and the value.
 * 
 * @author vmyllarn
 */
public class AttributeInstance extends Instance {
	/** Definition of this attribute instance */
	private AttributeDefinition definition;

	/** Value of this attribute instance */
	private Name value = null;

	public AttributeInstance(AttributedInstance parent, AttributeDefinition def, boolean userAdded) {
		super(parent.getAsTypedInstance(), userAdded);
		
		definition = def;
	}

	public void setValue(Name value, boolean isUserSet) {
		if (!definition.getType().includesValue(value))
			return;
		
		this.value = value;
		setUserAdded(isUserSet);
	}

	public void setValue(String value, boolean isUserSet) {
		if ((value != null && getType().includesValue(value)) || value == null)
			this.setValue(getType().getValue(value), isUserSet);
	}

	public void reset() {
		this.value = null;
	}

	/*
	 * TODO Change name to hasValue(value)
	 */
	public boolean isCurrentValue(Name value) {
		return this.value != null && this.value.equals(value);
	}

	/*
	 * TODO Change name to getStringValue. No method for getting the Value value? -toasikai
	 */
	public String getValue() {
		if (this.value != null) {
			return this.value.getName();
		} else {
			return null;
		}
	}

	public String getDisplayValue() {
		if (this.value != null) {
			return this.value.getDisplayName();
		} else {
			return null;
		}
	}

	public Iterator<Name> getPossibleValues() {
		return getType().getValues();
	}

	public boolean isPossibleValue(Name value) {
		return getType().includesValue(value);
	}

	public String getName() {
		return definition.getName();
	}

	public String getDisplayName() {
		return definition.getDisplayName();
	}
	
	public AttributeDefinition getDefinition() {
		return definition;
	}
	
	/**
	 * Compares to Attribute instances: equal if (and only if) they agree in name, value and type.
	 * 
	 */
	public boolean equals(AttributeInstance instance) {
		return instance != null && definition.equals(instance.definition)
				&& value.equals(instance.value);
	}

	/**
	 * Tests whether this instance is equal to another object
	 * 
	 * @param o
	 */
	public boolean equals(Object o) {
		if (o instanceof AttributeInstance)
			return equals((AttributeInstance) o);
		else
			return false;
	}

	public String toString() {
		if (value != null) {
			return getDisplayName() + " = " + value;
		} else {
			return getDisplayName();
		}
	}

	public String getDescription() {
		if (value != null) {
			return getName() + ":" + getType().getName() + " = " + value;
		} else {
			return getName() + ":" + getType().getName();
		}
	}

	public String getInstanceKind() {
		return "attribute";
	}

	public AttributeValueType getType() {
		return definition.getType();
	}

	public void remove() {
		/*
		 * TODO Consider restructuring the Instance class hierarchy: AttributeInstance and
		 * PartInstance objects really cannot be removed or restored.
		 */
	}

	public void restore() {
	}

	public Reference getPath() {
		ComposableReference ref = (ComposableReference) getParent().getPath();
		return new AttributeReference(ref, definition.getName());
	}
}
