/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * ComponentInstance.java
 *
 * Created on 2. maaliskuuta 2004, 11:29
 */

package kumbang.core.configuration.instance;

import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;

import kumbang.core.model.type.ComponentType;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.InterfaceDefinition;
import kumbang.core.model.type.InterfaceType;

/**
 * 
 * @author vmyllarn
 */
public class ComponentInstance extends ComposableInstance {
	private TreeSet<InterfaceInstance> interfaces;
	
	/**
	 * Creates a new instance of ComponentInstance that is an exact copy of the given instance under
	 * the given parent.
	 */
	public ComponentInstance(ComponentType type, PartInstance parent, boolean userAdded) {
		super(type, parent, userAdded);
		this.interfaces = new TreeSet();
	}

	/**
	 * Creates a new instance of ComponentInstance that is the root component and is an exact copy
	 * of the given instance.
	 */
	public ComponentInstance(ComponentType type) {
		// It doesn't matter if root instances are user-added or not. They're always there anyway.
		this(type, null, false);
	}

	public String getInstanceKind() {
		return "component";
	}

	public ComponentType getComponentType() {
		return (ComponentType) getType();
	}

	public InterfaceInstance getInterface(String name) {
		for (InterfaceInstance ii : interfaces) {
			InterfaceDefinition def = ii.getInterfaceDefinition();
			if (def.getName().equals(name))
				return ii;
		}
		
		return null;
		
//		for (Iterator it = interfaces.keySet().iterator(); it.hasNext(); ) {
//			InterfaceDefinition def = (InterfaceDefinition) it.next();
//			if (def.getName().equals(name))
//				return (InterfaceInstance) interfaces.get(def);
//		}
//		
//		return null;
	}

	public InterfaceInstance getInterface(InterfaceDefinition def) {
		for (InterfaceInstance ii : interfaces) {
			if (ii.getInterfaceDefinition().equals(def))
				return ii;
		}
		
		return null;
//		return (InterfaceInstance) interfaces.get(def);
	}

	/*
	 * TODO Refactor to hasInterface(InterfaceInstance)
	 */
	public boolean containsInterface(String name) {
		return getInterface(name) != null;
	}

	/*
	 * TODO Refactor to hasInterface(InterfaceInstance)
	 */
	public boolean containsInterface(InterfaceInstance interf) {
		return interfaces.contains(interf);
	}

	public Iterator<InterfaceInstance> getInterfaces() {
		return interfaces.iterator();
	}

	public void removeInterface(InterfaceInstance interf) {
		interfaces.remove(interf); 
	}

	public boolean hasInterfaces() {
		if (interfaces.size() > 0)
			return true;
		else
			return false;
	}

	public InterfaceInstance addInterface(InterfaceDefinition defininition, InterfaceType type,
			boolean userAdded) {
		if (getInterface(defininition) != null)
			return getInterface(defininition);

		InterfaceInstance instance = new InterfaceInstance(type, this, defininition, userAdded);
		interfaces.add(instance);

		return instance;
	}
	
	/**
	 * Used to restore interface instanes that were previously deleted.
	 * 
	 * @param instance
	 */
	protected void addInterface(InterfaceInstance instance) {
		if (!interfaces.contains(instance))
			interfaces.add(instance);
	}

	public List getInstanceList(ComposableType type, boolean recursive) {
		List ret = super.getInstanceList(type, recursive);
		
		for (Iterator<InterfaceInstance> it = interfaces.iterator(); it.hasNext(); ) {
			InterfaceInstance ii = it.next();
			if (!ret.contains(ii))
				ret.add(ii);
		}
		
		return ret;
	}
}
