/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Instance.java
 *
 * Created on 31. maaliskuuta 2004, 14:57
 */

package kumbang.core.configuration.instance;

import java.io.Serializable;

import kumbang.core.configuration.reference.Reference;

/**
 * 
 * @author vmyllarn
 */
public abstract class Instance implements Serializable {
	/**
	 * Contains the information whether the instances was added to the configuration by the user
	 * or whether it was included as a configuration consequence. -toasikai 27.7.2006
	 */
	private boolean isUserAdded = false;
		
	/**
	 * Parent instance of this instance. This instance is contained in the parent in a
	 * part. Parent is null if this is the root instance in the hierarchy.
	 */
	private TypedInstance parent;
	
	public Instance(TypedInstance parent, boolean isUserAdded) {
		this.parent = parent;
		setUserAdded(isUserAdded);
	}
	
	public boolean isUserAdded() {
		return isUserAdded;
	}
	
	public void setUserAdded(boolean isUserAdded) {
		this.isUserAdded = isUserAdded;
	}

	/**
	 * Shows a detailed description of the instance (e.g. for logging purposes)
	 * 
	 * @return String
	 */
	public abstract String getDescription();

	/**
	 * Returns an instance kind for the instance (e.g. component, binding, interface, attribute)
	 * 
	 * @return String a string containing the instance kind
	 */
	public abstract String getInstanceKind();

	public TypedInstance getParent() {
		return parent;
	}
	
	public abstract void remove();
	public abstract void restore();
	public abstract Reference getPath();
}
