/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * PartInstance.java
 *
 * Created on 3. maaliskuuta 2004, 12:53
 */

package kumbang.core.configuration.instance;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import kumbang.core.configuration.reference.ComposableReference;
import kumbang.core.configuration.reference.Reference;
import kumbang.core.configuration.reference.ReferenceItem;
import kumbang.core.model.type.ComponentType;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.FeatureType;
import kumbang.core.model.type.PartDefinition;


/**
 * This class is not needed. There is no such thing as a part instance. -toasikai
 * 
 * @author vmyllarn
 */
public class PartInstance extends Instance {
	/**
	 * The part definition this PartInstance is based on.
	 * 
	 * TODO Is this really
	 */
	protected PartDefinition def;
	
	/**
	 * List of ComposableInstance object that are parts under this part instance.
	 */
	private List instances;
	
	/**
	 * Creates a new part instance with the given parent and given part
	 * definition.
	 * 
	 * @param parent
	 *            parent instance
	 * @param partDefinition
	 *            part definition
	 */
	public PartInstance(ComposableInstance parent, PartDefinition partDefinition) {
		super(parent, false);
		
		def = partDefinition;
		instances = new ArrayList();
	}
	
	/**
	 * Returns the name of this part, specified by its part definition.
	 * 
	 * @return String name
	 */
	public String getName() {
		return def.getName();
	}

	/**
	 * Returns the display name of this part, specified by its part definition.
	 * 
	 * @return String name
	 */
	public String getDisplayName() {
		return def.getDisplayName();
	}

	/**
	 * Returns instance kind of this part instance, which is a string "part".
	 * 
	 * @return String "part"
	 */
	public String getInstanceKind() {
		return "part";
	}

	public Iterator getPossibleTypes() {
		return def.getAllPossibleTypesIterator(false);
	}

	/**
	 * Returns composed kind of this part instance, which is the instance kind
	 * of the composable instances in the same hierarchy.
	 * 
	 * @return String instance kind of the composable instances in this
	 *         hierarchy
	 */
	public String getComposedKind() {
		return getParent().getInstanceKind();
	}

	/**
	 * Returns a string that can be shown to the user directly. Uses display
	 * names.
	 * 
	 * @return String displayname[lowerbound,upperbound] if lowerbound ==
	 *         upperbound == 1, only displayname
	 */
	public String toString() {
		if (getLowerBound() == 1 && getUpperBound() == 1) {
			return getDisplayName();
		} else {
			return getDisplayName() + "[" + getLowerBound() + ","
					+ getUpperBound() + "]";
		}
	}

	/**
	 * Returns a description that contains regular names.
	 * 
	 * @return String name[lowerbound,upperbound] if lowerbound == upperbound ==
	 *         1, only name
	 */
	public String getDescription() {
		if (getLowerBound() == 1 && getUpperBound() == 1) {
			return getName();
		} else {
			return getName() + "[" + getLowerBound() + "," + getUpperBound()
					+ "]";
		}
	}

	/**
	 * Tests whether this part instance is equal to another instance. This is
	 * tested by comparing the name of this part and ID of the parent instance.
	 * 
	 * @param instance
	 *            another instance
	 * @return boolean <code>true</code> if this is equal, <code>false</code>
	 *         otherwise
	 *//*
	public boolean equals(Instance instance) {
		return instance != null
				&& instance instanceof PartInstance
				&& ((PartInstance) instance).getName().equals(getName())
				&& ((PartInstance) instance).getParent().getID() == getParent()
						.getID();
	}

	*//**
	 * Returns a hash code for this part instance. If two parts are equal, they
	 * will return the same hashcode. Otherwise the hash code is distinct from
	 * all other instances.
	 * 
	 * @return int hash code
	 *//*
	public int hashCode() {
		return (parent.getID() + "part" + getName()).hashCode();
	}

	*//**
	 * Tests whether this instance is equal to another object.
	 * 
	 * @param o
	 *//*
	public boolean equals(Object o) {
		if (o instanceof Instance) {
			return equals((Instance) o);
		} else {
			return false;
		}
	}
	*/
	
	/**
	 * Returns the parent composable instance of this part.
	 * 
	 * Implemented in Instance.
	 * 
	 * @return ComposableInstance parent instance
	 */
//	public ComposableInstance getParent() {
//		return parent;
//	}

	/**
	 * Returns the part definition that defines this part instance.
	 * 
	 * @return PartDefinition part definition
	 *//*
	public PartDefinition getPartDefinition() {
		return partDefinition;
	}

	*//**
	 * Returns the elementary definition that defines this part instance.
	 * Specified by method <code>getDefinition()</code> in interface
	 * <code>DefinedInstance</code>.
	 * 
	 * @return ElementaryDefinition part definition
	 *//*
	public ElementaryDefinition getDefinition() {
		return partDefinition;
	} */

	public int getLowerBound() {
		return def.getLowerBound();
	}

	public int getUpperBound() {
		return def.getUpperBound();
	}
	
	public void addInstance(ComposableInstance instance) {
		if (!containsInstance(instance) && !isFull())
			instances.add(instance);
	}
	
	protected void removeInstance(ComposableInstance instance) {
		if (this.containsInstance(instance))
			instances.remove(instance);
	}

	public boolean containsInstance(ComposableInstance instance) {
		return instances.contains(instance);
	}

	/**
	 * Returns the number of composable instances as part in this part instance.
	 * 
	 * @return int number of composable instances as part
	 */
	public int getNumberOfInstances() {
		return instances.size();
	}

	public boolean hasInstances() {
		return !instances.isEmpty();
	} 

	/**
	 * A method for testing whether the part instance is full, i.e., whether it contains the 
	 * maximum number of instances specified by its cardinality or not.
	 * 
	 * @return true if the part instance is full, false otherwise
	 */
	public boolean isFull() {
		return instances.size() == def.getUpperBound();
	} 
	
	public int getNumberOfPossibleTypes() {
		return def.getAllPossibleTypes(false).size();
	}
	
	/**
	 * Returns an iterator over the instances contained by this part definition.
	 * 
	 * @return see above
	 */
	public Iterator getInstances() {
		return instances.iterator();
	}

	public PartDefinition getDefinition() {
		return def;
	}

	/**
	 * Returns the 1-based index of instance within this part instance. 
	 * 
	 * @param instance
	 * @return
	 */
	public int getIndex(ComposableInstance instance) {
		if (instance == null)
			return -1;
		
		if (instances.contains(instance) && instances.size() == 1)
			return 0;
		
		/*
		 * Return the zero-based index of instance among the instances that are
		 * of the same type as instance.
		 */
		int ret = 0;
		ComposableType comp = (ComposableType) instance.getType();
		for (Iterator it = instances.iterator(); it.hasNext(); ) {
			ComposableInstance ci = (ComposableInstance) it.next();
			if (ci.equals(instance))
				return ret;
			
			if (ci.getType().equals(comp))
				ret++;
		}
		
		return ret;
//		return instances.indexOf(instance);
	}

	/**
	 * NOTE: this method is overriden in subclasses to support other
	 * instances than just component and feature instances.
	 * @param type
	 * @param userAdded
	 * @return
	 */
	public ComposableInstance addInstance(ComposableType type, boolean userAdded) {
		ComposableInstance instance = null;
		
		if (type instanceof ComponentType)
			instance = new ComponentInstance((ComponentType) type, this, userAdded);
		else if (type instanceof FeatureType)
			/*
			 * There should be a better way to handle these different kind of instances.
			 * To enable defining new type kinds etc.
			 * 
			 * TODO Do something about this.
			 */
			instance = new FeatureInstance((FeatureType) type, this, userAdded);
		
		return instance;
	}
	
	/*
	 * This may not make much sense, but at least this is an implementation. No one probably calls
	 * this method in the current implementation. -toasikai 2.8.2006
	 * 
	 * TODO Do something about this.
	 */
	public void remove() {
	}

	public void restore() {
	}

	public int getSize() {
		return instances.size();
	}

	/*
	 * This does not work, a part instance does not have a path.
	 */
	public Reference getPath() {
		return new ComposableReference((ComposableReference) getParent().getPath(),
				new ReferenceItem(def.getName()), true);
	}
}
