/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * TypedInstance.java
 *
 * Created on 3. kes�kuuta 2005, 11:46
 */

package kumbang.core.configuration.instance;

import kumbang.core.model.type.ElementaryType;

/**
 *
 * @author  vmyllarn
 */
public abstract class TypedInstance extends Instance implements Comparable {
	private static int serialNr = 0;
	
	/**
	 * An id that is used to impose a total order on instances of this class. Needed when restoring
	 * a composable instance: the id's are used to sort the parts list in a PartInstance, thus
	 * an instance that is restored can be restored in its original position.
	 */
	private int id = serialNr++;
	
    public abstract ElementaryType getType();
    
    public abstract String getName();
    
    public abstract String getDisplayName();
    
    public TypedInstance(ComposableInstance parent, boolean userAdded) {
    	super(parent, userAdded);
    }
    
	public int compareTo(Object comp) {
		if (!(comp instanceof TypedInstance))
			throw new ClassCastException();
		
		return id - ((TypedInstance) comp).id;
	}
	
	public boolean equals(Object comp) {
		if (!(comp instanceof TypedInstance))
			return false;
		
		return id == ((TypedInstance) comp).id;
	}
}
