/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.core.configuration.reference;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * 
 * @author toasikai
 */
public class ComposableReference implements Reference {
	private boolean isComponent;
	private List<ReferenceItem> items;
	private boolean isPartReference;
	
	public ComposableReference(boolean isComponent, List<ReferenceItem> items) {
		this.isComponent = isComponent;
		this.items = items;
	}
	
	public ComposableReference(ComposableReference head, ReferenceItem tail, boolean isPart) {
		this.isComponent = head.isComponent;
		this.items = new LinkedList<ReferenceItem>(head.items);
		this.items.add(tail);
		this.isPartReference = isPart;
	}
	
	public boolean isComponentReference() {
		return isComponent;
	}
	
	public Iterator<ReferenceItem> getItems() {
		return items.iterator();
	}

	public ComposableReference getComposable() {
		return this;
	}

	public boolean isPartReference() {
		return isPartReference;
	}
	
	public void setPartReference() {
		isPartReference = true;
	}
	
	public String toString() {
		String ret = "ComposableReference[";
		for (ReferenceItem refItem : items) {
			ret += " " + refItem.getName();
		}
		ret += "]";
		return ret;
	}
}
