/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * AddBindingTask.java
 *
 * Created on 19. lokakuuta 2004, 13:55
 */

package kumbang.core.configuration.task;

import kumbang.core.configuration.core.Configuration;
import kumbang.core.configuration.instance.BindingInstance;
import kumbang.core.configuration.instance.InterfaceInstance;
import kumbang.core.configuration.reference.InterfaceReference;

/**
 * 
 * @author vmyllarn
 */
public class AddBindingTask extends AddInstanceTask {
	private AddInterfaceTask targetParent;
	private AddInterfaceTask sourceParent;
	
	private InterfaceInstance source;
	private InterfaceInstance target;
	
	private InterfaceReference sourceRef;
	private InterfaceReference targetRef;

	public AddBindingTask(InterfaceInstance source, InterfaceInstance target, boolean isUserTask) {
		super(null, isUserTask);
		this.source = source;
		this.target = target;
	}

	public AddBindingTask(InterfaceReference source, InterfaceReference target,
			AddInterfaceTask sourceTask, AddInterfaceTask targetTask, boolean isUserTask) {
		super(null, isUserTask);
		this.sourceRef = source;
		this.targetRef = target;
		this.sourceParent = sourceTask;
		this.targetParent = targetTask;
	}

	public InterfaceInstance getSource() {
		if (source != null)
			return source;
		
		if (sourceParent != null)
			return (InterfaceInstance) sourceParent.getAddedInstance();
		
		return null;
	}

	public InterfaceInstance getTarget() {
		if (target != null)
			return target;
		
		if (targetParent != null)
			return (InterfaceInstance) targetParent.getAddedInstance();
		
		return null;
	}
	
	/**
	 * Returns the newly added instance
	 * 
	 * @return BindingInstance
	 */
	public BindingInstance getAddedBinding() {
		return (BindingInstance) super.getAddedInstance();
	}

	/**
	 * Sets the newly added binding
	 * 
	 * @param binding
	 */
	public void setAddedBinding(BindingInstance binding) {
		super.setAddedInstance(binding);
	}

	public boolean equals(Object o) {
		if (o instanceof AddBindingTask) {
			AddBindingTask t = (AddBindingTask) o;
//			return t.getSourceID() == getSourceID() && t.getTargetID() == getTargetID();
			return t.source.equals(source) && t.target.equals(target);
		}
		return false;
	}

	public void convertReferences(Configuration configuration) {
		if (source == null && sourceRef != null)
			source = (InterfaceInstance) configuration.getInstance(sourceRef);
		if (target == null && targetRef != null)
			target = (InterfaceInstance) configuration.getInstance(targetRef);
	}
	
	public String toString() {
		return "AddBindingTask[]";
	}	
}
