/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * AddComposableTask.java
 *
 * Created on 19. lokakuuta 2004, 12:08
 */

package kumbang.core.configuration.task;

import kumbang.core.configuration.core.Configuration;
import kumbang.core.configuration.instance.ComposableInstance;
import kumbang.core.configuration.instance.PartInstance;
import kumbang.core.configuration.reference.ComposableReference;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.InterfaceDefinition;
import kumbang.core.model.type.InterfaceType;
import kumbang.core.model.type.PartDefinition;

/**
 * 
 * @author vmyllarn
 */
public class AddComposableTask extends AddInstanceTask {
	private PartInstance parent;
	
	private ComposableReference reference;
	
	/*
	 * Either PartInstance or PartDefinition must be given, and the type.
	 */
	private PartDefinition definition;
	
	private ComposableType type; 

	public AddComposableTask(PartInstance instance, ComposableType type, boolean isUserTask) {
		super(null, isUserTask);
		this.parent = instance;
		this.type = type;
	}
	
	public AddComposableTask(ComposableReference ref, ComposableType type, boolean isUserTask) {
		super(null, isUserTask);
		this.reference = ref;
		this.type = type;
	}
	
	/**
	 * This constructor is needed if the parent has not yet been added.
	 * 
	 * @param parent
	 * @param def
	 * @param type
	 */
	public AddComposableTask(AddComposableTask parent, PartDefinition def, ComposableType type) {
		super(parent, parent.isUserTask());
		this.definition = def;
		this.type = type;
	}

	public String toString() {
		return "AddComposableTask[" + parent + "." + type + "]";
	}

	public PartInstance getParent() {
		if (parent == null) {
			AddInstanceTask ait = (AddInstanceTask) getParentTask();
			ComposableInstance ci = (ComposableInstance) ait.getAddedInstance();
			parent = ci.getPart(definition);
		}
		
		return parent;
	}
	
	public ComposableType getType() {
		return type;
	}
	
	public PartDefinition getDefinition() {
		return definition;
	}

	/*
	 * TODO Could this use better identifiers than name and value, both string? -toasikai 26.7.2006
	 */
	public void setAttributeValue(String name, String value) {
		addSubTask(new SetAttributeTask(this, name, value));
	}

	public void addInterface(InterfaceDefinition def, InterfaceType type) {
		addSubTask(new AddInterfaceTask(this, def, type));
	}
	
	public void setAddedInstance(ComposableInstance instance) {
		super.setAddedInstance(instance);
		
		parent = instance.getParentPart();
	}
	
	public ComposableInstance getAddedComposable() {
		return (ComposableInstance) getAddedInstance();
	}

	public void convertReferences(Configuration configuration) {
		if (parent == null && reference != null)
			parent = (PartInstance) configuration.getInstance(reference);
	}
}
