/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * ConfigurationAttributeConsequence.java
 *
 * Created on 31. elokuuta 2004, 5:41
 */

package kumbang.core.configuration.task;

/**
 * 
 * @author vmyllarn
 */
public class AttributeHasValueConsequence extends ConfigurationConsequence {

	private String name;

	private String value;

	/** Creates a new instance of ConfigurationAttributeStatement */
	public AttributeHasValueConsequence(int targetID, String name, String value) {
		super(targetID);
		this.name = name.substring(7);
		
		if (Character.isDigit(value.charAt(0)))
			this.value = value;
		else
			this.value = value.substring(2);
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getValue() {
		return value;
	}

	public void setValue(String value) {
		this.value = value;
	}

	public String toString() {
		return "AttributeHasValueConsequence[ID=" + getTargetID() + "." + name + "=" + value + "]";
	}

	public boolean equals(Object o) {
		if (o instanceof AttributeHasValueConsequence) {
			AttributeHasValueConsequence cnsq = (AttributeHasValueConsequence) o;
			if (cnsq.getTargetID() == getTargetID()) {
				if (cnsq.getName().equals(getName())) {
					if (value != null) {
						return value.equals(cnsq.getValue());
					} else {
						return cnsq.getValue() == null;
					}
				}
			}
		}
		return false;
	}

	/**
	 * Compares this consequence to another object. Added to implement interface Comparable 
	 * 
	 * @param comp another object
	 * @return int a comparison number
	 */
	public int compareTo(Object comp) {
		// Only ConfigurationConsequences can be compared with each other.
		if (!(comp instanceof ConfigurationConsequence))
			throw new ClassCastException();
		
		// AttributeHasValueConsequences come after InstanceExistConsequence objects.
		if (comp instanceof InstanceExistsConsequence)
			return 1;
		
		// And before InterfacesBoundConsequence objects.
		if (comp instanceof InterfacesBoundConsequence)
			return -1;
		
		// Two AttributeHasValueConsequences are ordered based on the instances they relate to.
		AttributeHasValueConsequence ahvc = (AttributeHasValueConsequence) comp;
		if (getTargetID() != ahvc.getTargetID())
			return getTargetID() - ahvc.getTargetID();
		
		if (!getName().equals(ahvc.getName()))
			return getName().compareTo(ahvc.getName());
		
		return getValue().compareTo(ahvc.getValue());
	}
}
