/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * ClientTask.java
 *
 * Created on 19. lokakuuta 2004, 13:42
 */

package kumbang.core.configuration.task;

import java.io.Serializable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import kumbang.core.configuration.core.Configuration;

/**
 * 
 * @author vmyllarn
 */
public class ClientTask implements Serializable {
	public static final int STATE_INITIALIZED = 0;
	public static final int STATE_VALIDATED = 1;
	public static final int STATE_FULFILLED = 2;
	public static final int STATE_ERROR = 3;

	private List<ClientTask> subTasks;

	private ClientTask parentTask;

	private String errorMsg;

	private int state;
	
	/**
	 * A flag that specifies whether this ClientTask was generated based on a user action (true)
	 * or as based on the server response (false).
	 */
	private boolean isUserTask;

	public ClientTask(boolean isUserTask) {
		this(null, isUserTask);
	}

	/** Creates a new instance of ClientTask */
	public ClientTask(ClientTask parent, boolean isUserTask) {
		this.parentTask = parent;
		this.state = STATE_INITIALIZED;
		this.subTasks = new LinkedList<ClientTask>();
		this.isUserTask = isUserTask;
	}

	public void addSubTask(ClientTask task) {
		subTasks.add(task);
	}

	protected boolean containsSubTask(ClientTask task) {
		return subTasks.contains(task);
	}

	protected boolean hasSubTasks() {
		return !subTasks.isEmpty();
	}

	public ClientTask getParentTask() {
		return parentTask;
	}

	public Iterator<ClientTask> getSubTasks() {
		return subTasks.iterator();
	}

	public void markValidated() {
		this.state = STATE_VALIDATED;
		for (ClientTask task : subTasks) {
			task.markValidated();
		}
	}

	public void markFulfilled() {
		this.state = STATE_FULFILLED;
		for (ClientTask task : subTasks) {
			task.markFulfilled();
		}
	}

	public void markError(String msg) {
		this.state = STATE_ERROR;
		this.errorMsg = msg;
		for (ClientTask task : subTasks) {
			task.markError(msg);
		}
	}

	public boolean isMarkedError() {
		return this.state == STATE_ERROR;
	}

	public String getErrorMsg() {
		return this.errorMsg;
	}

	public boolean isUserTask() {
		return isUserTask;
	}

	public void convertReferences(Configuration configuration) {
		for (ClientTask task : subTasks) {
			task.convertReferences(configuration);
		}
	}
	
	public String toString() {
		return "ClientTask[]";
	}
}
