/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.core.configuration.task;

import kumbang.core.configuration.core.Configuration;
import kumbang.core.configuration.instance.AttributedInstance;
import kumbang.core.configuration.reference.ComposableReference;
import kumbang.core.configuration.reference.Reference;


/**
 * 
 * @author vmyllarn
 */
public class EditAttributesTask extends ClientTask {

	private AttributedInstance instance;
	
	private Reference reference;

	/** Creates a new instance of EditComposableTask */
	public EditAttributesTask(AttributedInstance instance, boolean isUserTask) {
		super(isUserTask);
		this.instance = instance;
	}
	
	public EditAttributesTask(Reference ref, boolean isUserTask) {
		super(null, isUserTask);
		this.reference = ref;
	}
	
	/** Creates a new instance of EditComposableTask */	
	public EditAttributesTask(ClientTask parent, AttributedInstance instance) {
		super(parent, parent.isUserTask());
		this.instance = instance;
	}

	public EditAttributesTask(Reference path) {
		super(null, false);
		this.reference = path;
	}

	public void setAttribute(String name, String value) {
		addSubTask(new SetAttributeTask(this, name, value));
	}

	public AttributedInstance getInstance() {
		return this.instance;
	}

	public String toString() {
		return "EditAttributesTask[" + instance.toString() + "]";
	}
	
	public void convertReferences(Configuration configuration) {
		if (instance == null && reference != null)
			instance = (AttributedInstance) configuration.getInstance(reference);
	}
}
