/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.core.configuration.task;

import kumbang.core.configuration.core.Configuration;
import kumbang.core.configuration.instance.ComponentInstance;
import kumbang.core.configuration.instance.InterfaceInstance;
import kumbang.core.configuration.reference.ComposableReference;
import kumbang.core.configuration.reference.Reference;
import kumbang.core.model.type.InterfaceDefinition;
import kumbang.core.model.type.InterfaceType;

/**
 * 
 * @author vmyllarn
 */
public class EditInterfacesTask extends ClientTask {

	private ComponentInstance instance;
	
	private Reference reference;

	/** Creates a new instance of EditComposableTask */
	public EditInterfacesTask(ComponentInstance instance, boolean isUserTask) {
		super(isUserTask);
		this.instance = instance;
	}
	
	public EditInterfacesTask(ComposableReference ref, boolean isUserTask) {
		super(null, isUserTask);
		this.reference = ref;
	}
	
	/** Creates a new instance of EditComposableTask */	
	public EditInterfacesTask(ClientTask parent, ComponentInstance instance) {
		super(parent, parent.isUserTask());
		this.instance = instance;
	}

	public EditInterfacesTask(Reference path) {
		super(null, false);
		this.reference = path;
	}
	
	/*
	 * TODO Adding interface here is overly similar to what is found in AddComposableTask.
	 * Consider reusing the implementation. -toasikai 27.7.2006
	 * 
	 * TODO The parent-child information seems to be included twice. Do something about this.
	 */
	public void addInterface(InterfaceDefinition def, InterfaceType type) {
		addSubTask(new AddInterfaceTask(this, def, type));
	}

	public void removeInterface(InterfaceInstance iInstance) {
		if (instance instanceof ComponentInstance) {
			addSubTask(new RemoveInstanceTask(this, iInstance));
		}
	}

	public ComponentInstance getInstance() {
		return this.instance;
	}

	public String toString() {
		return "EditInterfacesTask[" + instance.toString() + "]";
	}
	
	public void convertReferences(Configuration configuration) {
		if (instance == null && reference != null)
			instance = (ComponentInstance) configuration.getInstance(reference);
	}
}

