/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * ConfigurationBindingConsequence.java
 *
 * Created on 31. elokuuta 2004, 5:43
 */

package kumbang.core.configuration.task;

/**
 * 
 * @author vmyllarn
 */
public class InterfacesBoundConsequence extends ConfigurationConsequence {

	private int sourceID;

	/** Creates a new instance of ConfigurationBindingConsequence */
	public InterfacesBoundConsequence(int sourceID, int targetID) {
		super(targetID);
		this.sourceID = sourceID;
	}

	public int getSourceID() {
		return this.sourceID;
	}

	public String toString() {
		return "InterfacesBoundConsequence[" + getSourceID() + "," + getTargetID() + "]";
	}

	public boolean equals(Object o) {
		if (o instanceof InterfacesBoundConsequence) {
			InterfacesBoundConsequence cnsq = (InterfacesBoundConsequence) o;
			return cnsq.getTargetID() == getTargetID() && cnsq.getSourceID() == getSourceID();
		}
		return false;
	}

	/*
	 * Must make sure this does not return 0, or one of the consequences gets lost.
	 */
	public int compareTo(Object comp) {
		// Only ConfigurationConsequences can be compared with each other.
		if (!(comp instanceof ConfigurationConsequence))
			throw new ClassCastException();
		
		// InterfacesBoundConsequence come after InstanceExistConsequence objects.
		if (comp instanceof InstanceExistsConsequence || comp instanceof AttributeHasValueConsequence)
			return 1;
		
		// Ordering based on the target id's is arbitrary.
		InterfacesBoundConsequence ibc = (InterfacesBoundConsequence) comp;
		if (getTargetID() != ibc.getTargetID())
			return getTargetID() - ibc.getTargetID();
		
		return getSourceID() - ibc.getSourceID();
	}
}
