/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * RemoveInstanceTask.java
 *
 * Created on 19. lokakuuta 2004, 13:57
 */

package kumbang.core.configuration.task;

import java.util.Iterator;

import kumbang.core.configuration.core.Configuration;
import kumbang.core.configuration.instance.BindingInstance;
import kumbang.core.configuration.instance.ComponentInstance;
import kumbang.core.configuration.instance.ComposableInstance;
import kumbang.core.configuration.instance.Instance;
import kumbang.core.configuration.instance.InterfaceInstance;
import kumbang.core.configuration.reference.Reference;

/**
 * 
 * @author vmyllarn
 */
public class RemoveInstanceTask extends ClientTask {
	/**
	 * The instance to be removed from the configuration.
	 */
	private Instance instance;
	
	private Reference reference;

	/** Creates a new instance of RemoveInstanceTask */
	public RemoveInstanceTask(Instance instance) {
		this(null, instance);
	}
	
	public RemoveInstanceTask(Reference path) {
//		super(false);
		super(null, false);
		this.reference = path;
	}
	
	/**
	 * Creates a new instance of RemoveInstanceTask, including subtasks for removing the possible
	 * parts, interfaces and bindings under the instance.
	 */
	public RemoveInstanceTask(ClientTask parentTask, Instance instance) {
		super(parentTask, false);
		this.instance = instance;

		generateSubTasks();
	}

	/**
	 * @param instance
	 */
	private void generateSubTasks() {
		if (instance instanceof ComposableInstance) {
			Iterator it = ((ComposableInstance) instance).getInstancesAsPartsOfType(null);
			while (it.hasNext()) {
				Instance child = (Instance) it.next();
				if (instance == child)
					continue;
				addSubTask(new RemoveInstanceTask(this, child));
			}

			if (instance instanceof ComponentInstance) {
				it = ((ComponentInstance) instance).getInterfaces();
				while (it.hasNext()) {
					InterfaceInstance intf = (InterfaceInstance) it.next();
					addSubTask(new RemoveInstanceTask(this, intf));
				}
			}
		} else if (instance instanceof InterfaceInstance) {
			Iterator it = ((InterfaceInstance) instance).getBindings();
			while (it.hasNext()) {
				BindingInstance bind = (BindingInstance) it.next();
				addSubTask(new RemoveInstanceTask(this, bind));
			}
		}
	}

	public Instance getInstance() {
		return this.instance;
	}
	
	public String toString() {
		return "RemoveInstanceTask[" + instance.toString() + "]";
	}

	public boolean equals(Object o) {
		if (o instanceof RemoveInstanceTask) {
			return ((RemoveInstanceTask) o).getInstance().equals(instance);
		}
		return false;
	}

	public void convertReferences(Configuration configuration) {
		if (instance == null && reference != null) {
			instance = configuration.getInstance(reference);
			generateSubTasks();
		}
	}
}
