/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * SetAttributeTask.java
 *
 * Created on 19. lokakuuta 2004, 13:40
 */

package kumbang.core.configuration.task;

import kumbang.core.configuration.instance.AttributeInstance;
import kumbang.core.configuration.instance.AttributedInstance;

/**
 * A task for setting an attribute. Always a child task of either an Add- or EditComposableTask?
 * -toasikai 27.7.2006
 * 
 * @author vmyllarn
 */
public class SetAttributeTask extends ClientTask {
	private AttributeInstance attribute;

	private String attributeName;

	private String attributeValue;
	
	/**
	 * The value the attribute had before the new value was set. Needed for reverting tasks.
	 */
	private String oldValue;
	
	/**
	 * The userAdded (set) status before the change made due to this task.
	 */
	private boolean wasUserSet;

	/** Creates a new instance of SetAttributeTask */
	public SetAttributeTask(ClientTask parentTask, String name, String value) {
		super(parentTask, parentTask.isUserTask());
		this.attributeName = name;
		this.attributeValue = value;
//		parentTask.addSubTask(this);
	}

	/**
	 * Returns the AttributedInstance object for which the attribute is set. 
	 * 
	 * @return see above
	 */
	public AttributedInstance getParentInstance() {
		ClientTask parent = getParentTask();

		if (parent instanceof AddComposableTask) {
			return (AttributedInstance) ((AddComposableTask) parent).getAddedInstance();
		} else if (parent instanceof AddInterfaceTask) {
			return (AttributedInstance) ((AddInterfaceTask) parent).getAddedInstance();
		} else if (parent instanceof EditAttributesTask) {
			return ((EditAttributesTask) parent).getInstance();
		} else {
			return null;
		}
	}

	public void setAttribute(AttributeInstance attribute) {
		this.attribute = attribute;
		oldValue = attribute.getValue();
		wasUserSet = attribute.isUserAdded();
	}

	public AttributeInstance getAttribute() {
		return this.attribute;
	}

	public String getName() {
		return this.attributeName;
	}

	public String getValue() {
		return this.attributeValue;
	}

	public String toString() {
		return "SetAttributeTask[" + attributeName + "=" + attributeValue + "]"; 
	}

	public String getOldValue() {
		return oldValue;
	}

	public boolean wasUserSet() {
		return wasUserSet;
	}
}
