/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


//
// Generated by JTB 1.2.2
//

package kumbang.core.language.visitor;
import kumbang.core.language.syntaxtree.*;
import java.util.*;

/**
 * Provides default methods which visit each node in the tree in depth-first
 * order.  Your visitors may extend this class.
 */
public class DepthFirstVisitor implements Visitor {
   //
   // Auto class visitors--probably don't need to be overridden.
   //
   public void visit(NodeList n) {
      for ( Enumeration e = n.elements(); e.hasMoreElements(); )
         ((Node)e.nextElement()).accept(this);
   }

   public void visit(NodeListOptional n) {
      if ( n.present() )
         for ( Enumeration e = n.elements(); e.hasMoreElements(); )
            ((Node)e.nextElement()).accept(this);
   }

   public void visit(NodeOptional n) {
      if ( n.present() )
         n.node.accept(this);
   }

   public void visit(NodeSequence n) {
      for ( Enumeration e = n.elements(); e.hasMoreElements(); )
         ((Node)e.nextElement()).accept(this);
   }

   public void visit(NodeToken n) { }

   //
   // User-generated visitor methods below
   //

   /**
    * f0 -> SyntaxModelKind()
    * f1 -> <MODEL>
    * f2 -> <IDENTIFIER>
    * f3 -> ( <ROOT> ( <FEATURE> | <COMPONENT> ) <IDENTIFIER> )+
    * f4 -> ( SyntaxTypeDefinition() )*
    */
   public void visit(SyntaxModel n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
      n.f4.accept(this);
   }

   /**
    * f0 -> <FORFAMEL>
    *       | <KOALISH>
    *       | <KUMBANG>
    */
   public void visit(SyntaxModelKind n) {
      n.f0.accept(this);
   }

   /**
    * f0 -> SyntaxAttributeTypeDefinition()
    *       | SyntaxInterfaceTypeDefinition()
    *       | SyntaxComposableTypeDefinition()
    */
   public void visit(SyntaxTypeDefinition n) {
      n.f0.accept(this);
   }

   /**
    * f0 -> [ <ABSTRACT> ]
    * f1 -> ( <FEATURE> | <COMPONENT> )
    * f2 -> <TYPE>
    * f3 -> SyntaxDisplayIdentifier()
    * f4 -> [ <EXTENDS> SyntaxIdentifierList() ]
    * f5 -> <LBRACE>
    * f6 -> ( SyntaxSection() )*
    * f7 -> <RBRACE>
    */
   public void visit(SyntaxComposableTypeDefinition n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
      n.f4.accept(this);
      n.f5.accept(this);
      n.f6.accept(this);
      n.f7.accept(this);
   }

   /**
    * f0 -> <ATTRIBUTE>
    * f1 -> <TYPE>
    * f2 -> SyntaxDisplayIdentifier()
    * f3 -> <EQUALS>
    * f4 -> <LBRACE>
    * f5 -> SyntaxValueEnumeration()
    * f6 -> <RBRACE>
    */
   public void visit(SyntaxAttributeTypeDefinition n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
      n.f4.accept(this);
      n.f5.accept(this);
      n.f6.accept(this);
   }

   /**
    * f0 -> <INTERFACE>
    * f1 -> <TYPE>
    * f2 -> SyntaxDisplayIdentifier()
    * f3 -> <LBRACE>
    * f4 -> ( <IDENTIFIER> <SEMICOLON> )*
    * f5 -> <RBRACE>
    */
   public void visit(SyntaxInterfaceTypeDefinition n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
      n.f4.accept(this);
      n.f5.accept(this);
   }

   /**
    * f0 -> SyntaxSubfeatureSection()
    *       | SyntaxAttributeSection()
    *       | SyntaxProvideSection()
    *       | SyntaxRequireSection()
    *       | SyntaxConstraintSection()
    *       | SyntaxContainSection()
    *       | SyntaxConnectSection()
    *       | SyntaxImplementationSection()
    */
   public void visit(SyntaxSection n) {
      n.f0.accept(this);
   }

   /**
    * f0 -> <SUBFEATURES>
    * f1 -> ( SyntaxPartSpecification() )*
    */
   public void visit(SyntaxSubfeatureSection n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> <ATTRIBUTES>
    * f1 -> ( SyntaxAttribute() )*
    */
   public void visit(SyntaxAttributeSection n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> <PROVIDES>
    * f1 -> ( SyntaxInterface() )*
    */
   public void visit(SyntaxProvideSection n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> <REQUIRES>
    * f1 -> ( SyntaxInterface() )*
    */
   public void visit(SyntaxRequireSection n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> SyntaxIdentifierList()
    * f1 -> SyntaxInterfaceSpecification()
    * f2 -> ( <COMMA> SyntaxInterfaceSpecification() )*
    * f3 -> <SEMICOLON>
    */
   public void visit(SyntaxInterface n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
   }

   /**
    * f0 -> SyntaxDisplayIdentifier()
    * f1 -> [ <LBRACE> ( <OPTIONAL> | <GROUNDED> ) [ <COMMA> ( <OPTIONAL> | <GROUNDED> ) ] <RBRACE> ]
    */
   public void visit(SyntaxInterfaceSpecification n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> <CONSTRAINTS>
    * f1 -> ( SyntaxConstraint() )*
    */
   public void visit(SyntaxConstraintSection n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> <CONTAINS>
    * f1 -> ( SyntaxPartSpecification() )*
    */
   public void visit(SyntaxContainSection n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> <IMPLEMENTATION>
    * f1 -> ( SyntaxConstraint() )*
    */
   public void visit(SyntaxImplementationSection n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> SyntaxIdentifierList()
    * f1 -> SyntaxDisplayIdentifier()
    * f2 -> [ SyntaxCardinality() ]
    * f3 -> [ <LBRACE> ( <NONE> | <SAME> | <DIFFERENT> ) <RBRACE> ]
    * f4 -> <SEMICOLON>
    */
   public void visit(SyntaxPartSpecification n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
      n.f4.accept(this);
   }

   /**
    * f0 -> <IDENTIFIER>
    *       | <LPAREN> <IDENTIFIER> ( SyntaxIdentifierListEnd() )* <RPAREN>
    */
   public void visit(SyntaxIdentifierList n) {
      n.f0.accept(this);
   }

   /**
    * f0 -> <COMMA>
    * f1 -> <IDENTIFIER>
    */
   public void visit(SyntaxIdentifierListEnd n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> <LBRACKET>
    * f1 -> <INTEGER_LITERAL>
    * f2 -> [ <MINUS> <INTEGER_LITERAL> ]
    * f3 -> <RBRACKET>
    */
   public void visit(SyntaxCardinality n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
   }

   /**
    * f0 -> <IDENTIFIER>
    * f1 -> SyntaxDisplayIdentifier()
    * f2 -> [ <LBRACE> <HIDDEN> <RBRACE> ]
    * f3 -> <SEMICOLON>
    */
   public void visit(SyntaxAttribute n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
   }

   /**
    * f0 -> SyntaxValueDefinition()
    * f1 -> ( SyntaxEnumerationRest() )*
    */
   public void visit(SyntaxValueEnumeration n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> ( [ <MINUS> ] <INTEGER_LITERAL> | <IDENTIFIER> )
    * f1 -> [ <STRING_LITERAL> ]
    */
   public void visit(SyntaxValueDefinition n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> [ <MINUS> ] <INTEGER_LITERAL>
    *       | <IDENTIFIER>
    */
   public void visit(SyntaxValue n) {
      n.f0.accept(this);
   }

   /**
    * f0 -> <COMMA>
    * f1 -> SyntaxValueDefinition()
    */
   public void visit(SyntaxEnumerationRest n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> <IDENTIFIER>
    */
   public void visit(SyntaxFunction n) {
      n.f0.accept(this);
   }

   /**
    * f0 -> <CONNECTS>
    * f1 -> ( SyntaxConnection() )*
    */
   public void visit(SyntaxConnectSection n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> <IDENTIFIER>
    * f1 -> [ <DOT> <IDENTIFIER> ]
    * f2 -> <EQUALS>
    * f3 -> <IDENTIFIER>
    * f4 -> [ <DOT> <IDENTIFIER> ]
    * f5 -> <SEMICOLON>
    */
   public void visit(SyntaxConnection n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
      n.f4.accept(this);
      n.f5.accept(this);
   }

   /**
    * f0 -> SyntaxEquivalenceExpression()
    * f1 -> <SEMICOLON>
    */
   public void visit(SyntaxConstraint n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> [ <NOT> ]
    * f1 -> ( <FORALL> | <EXISTS> )
    * f2 -> <LPAREN>
    * f3 -> SyntaxVariableBinding()
    * f4 -> ( <COMMA> SyntaxVariableBinding() )*
    * f5 -> <RPAREN>
    */
   public void visit(SyntaxQuantification n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
      n.f4.accept(this);
      n.f5.accept(this);
   }

   /**
    * f0 -> <IDENTIFIER>
    * f1 -> <COLON>
    * f2 -> SyntaxStructureReference()
    */
   public void visit(SyntaxVariableBinding n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
   }

   /**
    * f0 -> SyntaxImplicationExpression()
    * f1 -> [ <EQUIVALENT> SyntaxImplicationExpression() ]
    */
   public void visit(SyntaxEquivalenceExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> SyntaxAndOrExpression()
    * f1 -> [ <IMPLIES> SyntaxAndOrExpression() ]
    */
   public void visit(SyntaxImplicationExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> SyntaxQuantifiedExpression()
    * f1 -> ( ( <AND> | <OR> ) SyntaxQuantifiedExpression() )*
    */
   public void visit(SyntaxAndOrExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> ( SyntaxQuantification() )*
    * f1 -> [ <NOT> ]
    * f2 -> ( SyntaxConfigurationPredicateExpression() | SyntaxComparisonExpression() )
    */
   public void visit(SyntaxQuantifiedExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
   }

   /**
    * f0 -> SyntaxAdditiveExpression()
    * f1 -> [ SyntaxComparisonOperator() SyntaxAdditiveExpression() ]
    */
   public void visit(SyntaxComparisonExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> SyntaxMultiplicativeExpression()
    * f1 -> ( ( <PLUS> | <MINUS> ) SyntaxMultiplicativeExpression() )*
    */
   public void visit(SyntaxAdditiveExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> SyntaxPrimitiveArithmetic()
    * f1 -> ( ( <ASTERISK> | <DIV> | <MOD> ) SyntaxPrimitiveArithmetic() )*
    */
   public void visit(SyntaxMultiplicativeExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> ( [ <MINUS> ] ( SyntaxCardinalityExpression() | SyntaxAttributeValue() | <LPAREN> SyntaxEquivalenceExpression() <RPAREN> ) | SyntaxValue() )
    */
   public void visit(SyntaxPrimitiveArithmetic n) {
      n.f0.accept(this);
   }

   /**
    * f0 -> <CARDINALITY>
    * f1 -> <LPAREN>
    * f2 -> SyntaxStructureReference()
    * f3 -> <RPAREN>
    */
   public void visit(SyntaxCardinalityExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
   }

   /**
    * f0 -> SyntaxPresentExpression()
    *       | SyntaxInstanceOfExpression()
    *       | SyntaxHasInstancesExpression()
    *       | SyntaxHasPartOfTypeExpression()
    *       | SyntaxConnectedExpression()
    *       | SyntaxConnectedToExpression()
    */
   public void visit(SyntaxConfigurationPredicateExpression n) {
      n.f0.accept(this);
   }

   /**
    * f0 -> <PRESENT>
    * f1 -> <LPAREN>
    * f2 -> SyntaxStructureReference()
    * f3 -> <RPAREN>
    */
   public void visit(SyntaxPresentExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
   }

   /**
    * f0 -> <INSTANCEOF>
    * f1 -> <LPAREN>
    * f2 -> SyntaxStructureReference()
    * f3 -> <COMMA>
    * f4 -> <IDENTIFIER>
    * f5 -> <RPAREN>
    */
   public void visit(SyntaxInstanceOfExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
      n.f4.accept(this);
      n.f5.accept(this);
   }

   /**
    * f0 -> <HASINSTANCES>
    * f1 -> <LPAREN>
    * f2 -> <IDENTIFIER>
    * f3 -> <RPAREN>
    */
   public void visit(SyntaxHasInstancesExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
   }

   /**
    * f0 -> <HASPART_OFTYPE>
    * f1 -> <LPAREN>
    * f2 -> [ SyntaxStructureReference() <COMMA> ]
    * f3 -> <IDENTIFIER>
    * f4 -> <RPAREN>
    */
   public void visit(SyntaxHasPartOfTypeExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
      n.f4.accept(this);
   }

   /**
    * f0 -> <CONNECTED>
    * f1 -> <LPAREN>
    * f2 -> SyntaxStructureReference()
    * f3 -> <RPAREN>
    */
   public void visit(SyntaxConnectedExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
   }

   /**
    * f0 -> <CONNECTED_TO>
    * f1 -> <LPAREN>
    * f2 -> SyntaxStructureReference()
    * f3 -> <COMMA>
    * f4 -> SyntaxStructureReference()
    * f5 -> <RPAREN>
    */
   public void visit(SyntaxConnectedToExpression n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
      n.f4.accept(this);
      n.f5.accept(this);
   }

   /**
    * f0 -> <VALUE>
    * f1 -> <LPAREN>
    * f2 -> [ SyntaxStructureReference() <COMMA> ]
    * f3 -> <IDENTIFIER>
    * f4 -> <RPAREN>
    */
   public void visit(SyntaxAttributeValue n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
      n.f4.accept(this);
   }

   /**
    * f0 -> <IDENTIFIER>
    * f1 -> [ <STRING_LITERAL> ]
    */
   public void visit(SyntaxDisplayIdentifier n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> [ SyntaxRootSymbol() | SyntaxStructureReferenceItem() ]
    * f1 -> ( <DOT> SyntaxStructureReferenceItem() )*
    * f2 -> [ <ARROW> SyntaxStructureReferenceItem() ]
    */
   public void visit(SyntaxStructureReference n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
   }

   /**
    * f0 -> ( <ASTERISK> | <QUESTION_MARK> | <IDENTIFIER> )
    * f1 -> [ <COLON> <IDENTIFIER> ]
    */
   public void visit(SyntaxStructureReferenceItem n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> ( <FEATURE> | <COMPONENT> )
    * f1 -> <MINUS>
    * f2 -> <ROOT>
    */
   public void visit(SyntaxRootSymbol n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
   }

   /**
    * f0 -> <EQUALS>
    *       | <NE>
    *       | <LT>
    *       | <LTE>
    *       | <GT>
    *       | <GTE>
    */
   public void visit(SyntaxComparisonOperator n) {
      n.f0.accept(this);
   }

   /**
    * f0 -> SyntaxConfigurationComposableReference()
    * f1 -> [ <ARROW> <IDENTIFIER> [ <COLON> <IDENTIFIER> ] [ <EQUALS> SyntaxConfigurationComposableReference() <ARROW> <IDENTIFIER> [ <COLON> <IDENTIFIER> ] ] | <ATTR> <IDENTIFIER> [ <EQUALS> <IDENTIFIER> ] ]
    */
   public void visit(SyntaxConfigurationReference n) {
      n.f0.accept(this);
      n.f1.accept(this);
   }

   /**
    * f0 -> ( <FEATURE> | <COMPONENT> )
    * f1 -> <MINUS>
    * f2 -> <ROOT>
    * f3 -> ( SyntaxConfigurationReferenceItem() )*
    */
   public void visit(SyntaxConfigurationComposableReference n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
   }

   /**
    * f0 -> <DOT>
    * f1 -> <IDENTIFIER>
    * f2 -> [ <LBRACKET> <INTEGER_LITERAL> <RBRACKET> ]
    * f3 -> [ <COLON> <IDENTIFIER> ]
    */
   public void visit(SyntaxConfigurationReferenceItem n) {
      n.f0.accept(this);
      n.f1.accept(this);
      n.f2.accept(this);
      n.f3.accept(this);
   }

}
