/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on Jun 3, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.language.visitor;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import kumbang.core.language.syntaxtree.NodeListOptional;
import kumbang.core.language.syntaxtree.NodeSequence;
import kumbang.core.language.syntaxtree.SyntaxDisplayIdentifier;
import kumbang.core.language.syntaxtree.SyntaxIdentifierList;
import kumbang.core.language.syntaxtree.SyntaxIdentifierListEnd;
import kumbang.core.model.core.Model;
import kumbang.core.model.type.Name;
import kumbang.util.ErrorHandler;

/**
 * @author toasikai
 *
 */
public abstract class KumbangVisitor extends ObjectDepthFirst {
	protected Model model;
	protected ErrorHandler handler;
	
	/**
	 * f0 -> <IDENTIFIER>
	 * f1 -> [ <STRING_LITERAL> ]
	 */
	public Object visit(SyntaxDisplayIdentifier n, Object argu) {
		if (n.f1.present()) {
			String display = n.f1.node.toString();
			return new Name(n.f0.toString(), display.substring(1, display.length() - 1));
		}
		
		return new Name(n.f0.toString());
	}
	
	/**
	 * f0 -> <IDENTIFIER>
	 *       | <LPAREN> <IDENTIFIER> ( IdentifierListEnd() )* <RPAREN>
	 */
	public Object visit(SyntaxIdentifierList n, Object argu) {
		List values = new ArrayList();

		if (n.f0.which == 0) {
			values.add(n.f0.choice.toString());
			return values;
		}

		NodeSequence ns = (NodeSequence) n.f0.choice;
		values.add(ns.elementAt(1).toString());

		//	Are there more possible types (separated with commas)?
		NodeListOptional nlo = (NodeListOptional) ns.elementAt(2);
		if (!nlo.present())
			return values;

		for (Enumeration e = nlo.elements(); e.hasMoreElements();) {
			SyntaxIdentifierListEnd ile = (SyntaxIdentifierListEnd) e.nextElement();
			values.add(ile.f1.toString());
		}

		return values;
	}
}
