/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on Feb 13, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.language.visitor;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Vector;

import kumbang.core.language.syntaxtree.NodeChoice;
import kumbang.core.language.syntaxtree.NodeListOptional;
import kumbang.core.language.syntaxtree.NodeOptional;
import kumbang.core.language.syntaxtree.NodeSequence;
import kumbang.core.language.syntaxtree.SyntaxAttributeTypeDefinition;
import kumbang.core.language.syntaxtree.SyntaxComposableTypeDefinition;
import kumbang.core.language.syntaxtree.SyntaxEnumerationRest;
import kumbang.core.language.syntaxtree.SyntaxFunction;
import kumbang.core.language.syntaxtree.SyntaxInterfaceTypeDefinition;
import kumbang.core.language.syntaxtree.SyntaxModel;
import kumbang.core.language.syntaxtree.SyntaxModelKind;
import kumbang.core.language.syntaxtree.SyntaxTypeDefinition;
import kumbang.core.language.syntaxtree.SyntaxValueDefinition;
import kumbang.core.language.syntaxtree.SyntaxValueEnumeration;
import kumbang.core.model.core.AttributedModel;
import kumbang.core.model.core.ComponentedModel;
import kumbang.core.model.core.FeaturedModel;
import kumbang.core.model.core.ForfamelModel;
import kumbang.core.model.core.KoalishModel;
import kumbang.core.model.core.KumbangModel;
import kumbang.core.model.core.Model;
import kumbang.core.model.type.AttributeValueType;
import kumbang.core.model.type.ComponentType;
import kumbang.core.model.type.FeatureType;
import kumbang.core.model.type.InterfaceType;
import kumbang.core.model.type.Name;
import kumbang.util.ErrorHandler;

/**
 * First pass through the abstract syntax tree.
 * 
 * @author toasikai
 *
 */
public class TypeVisitor extends KumbangVisitor {
	private String modelKind;

	public TypeVisitor(SyntaxModel model, ErrorHandler handler) {
		this.handler = handler;
		model.accept(this, new Object());
	}

	public Model getModel() {
		return model;
	}

	//
	// User-generated visitor methods below
	//

	/**
	 * f0 -> ModelKind()
	 * f1 -> <MODEL>
	 * f2 -> <IDENTIFIER>
	 * f3 -> ( <ROOT> ( <FEATURE> | <COMPONENT> ) <IDENTIFIER> )+
	 * f4 -> ( TypeDefinition() )*
	 */
	public Object visit(SyntaxModel n, Object argu) {
		//	Read in the model kind.
		n.f0.accept(this, argu);

		String name = n.f2.toString();
		
		if (modelKind.equals(ForfamelModel.MODEL_KIND)) {
			model = new ForfamelModel(name);
		} else if (modelKind.equals(KoalishModel.MODEL_KIND)) {
			model = new KoalishModel(name);
		} else if (modelKind.equals(KumbangModel.MODEL_KIND)) {
			model = new KumbangModel(name);
		}

		n.f4.accept(this, argu);

		for (Enumeration e = n.f3.elements(); e.hasMoreElements();) {
			NodeSequence ns = (NodeSequence) e.nextElement();
			String rootName = ns.elementAt(2).toString();
			int which = ((NodeChoice) ns.elementAt(1)).which;
			
			if (which == 0) {
				// this is a feature root, get corresponding type
				FeatureType fRoot = (FeatureType) model.getType(rootName, FeatureType.TYPE_KIND);
				if (model instanceof FeaturedModel) {
					if (fRoot == null) {
						handler.complain(n.f2.beginLine, 
								"The feature root " + rootName + " has not been defined in the model.");
						return null;
					} else {
						try {
							((FeaturedModel) model).setFeatureRoot(fRoot);
						} catch (Exception ex) {
							handler.complain(n.f2.beginLine, ex.getMessage());
							return null;
						}
					}
				} else {
					// a non-featured model defines a feature root
					handler.complain(n.f2.beginLine, 
							"A model that does not contain features cannot define a feature root.");
					return null;
				}
			} else if (which == 1) {
				// this is a component root, get corresponding type
				ComponentType cRoot = (ComponentType) model.getType(rootName, ComponentType.TYPE_KIND);
				if (model instanceof ComponentedModel) {
					if (cRoot == null) {
						handler.complain(n.f2.beginLine, 
								"The component root " + rootName + " has not been defined in the model.");
						return null;
					} else {
						try {
							((ComponentedModel) model).setComponentRoot(cRoot);
						} catch (Exception ex) {
							handler.complain(n.f2.beginLine, ex.getMessage());
							return null;
						}
					}
				} else {
					// a non-componented model defines a component root
					handler.complain(n.f2.beginLine, 
						"A model that does not contain components cannot define a component root.");
					return null;
				}
			} else {
				// this should not happen
				handler.complain(n.f2.beginLine, 
						"Model specifies a root that is not a feature nor a component.");
				return null;
			}
		}

		return model;
	}

	/**
	 * f0 -> <FORFAMEL>
	 *       | <KOALISH>
	 *       | <KUMBANG>
	 */
	public Object visit(SyntaxModelKind n, Object argu) {
		if (n.f0.which == 0) {
			modelKind = ForfamelModel.MODEL_KIND;
		} else if (n.f0.which == 1) {
			modelKind = KoalishModel.MODEL_KIND;
		} else if (n.f0.which == 2) {
			modelKind = KumbangModel.MODEL_KIND;
		}

		return null;
	}

	/**
	 * f0 -> SyntaxAttributeTypeDefinition()
	 *       | SyntaxInterfaceTypeDefinition()
	 *       | SyntaxComposableTypeDefinition()
	 */
	public Object visit(SyntaxTypeDefinition n, Object argu) {
//		if (!(model instanceof ComponentedModel) && (n.f0.which == 2 || n.f0.which == 3)) {
//			handler.complain("Non-component models cannot define component types or interface types.");
//			return null;
//		}
//		if (!(model instanceof AttributedModel) && (n.f0.which == 1)) {
//			handler.complain("Non-attribute models cannot define attribute value types.");
//			return null;
//		}
//		if (!(model instanceof FeaturedModel) && (n.f0.which == 0)) {
//			handler.complain("Non-feature models cannot define feature types.");
//			return null;
//		}

		n.f0.accept(this, argu);
		return argu;
	}

	/**
	 * f0 -> [ <ABSTRACT> ]
	 * f1 -> ( <FEATURE> | <COMPONENT> )
	 * f2 -> <TYPE>
	 * f3 -> SyntaxDisplayIdentifier()
	 * f4 -> [ <EXTENDS> SyntaxIdentifierList() ]
	 * f5 -> <LBRACE>
	 * f6 -> ( SyntaxSection() )*
	 * f7 -> <RBRACE>
	 */
	public Object visit(SyntaxComposableTypeDefinition n, Object argu) {
		Name name = (Name) n.f3.accept(this, argu);
		
		if (n.f1.which == 0) {
			FeatureType ft = new FeatureType(name);
			FeaturedModel fm = (FeaturedModel) model;
			try {
				fm.addFeatureType(ft);
			} catch (Exception e) {
				handler.complain(n.f2.beginLine, e.getMessage());
			}
			
			//	Check the type for abstractness
			if (n.f0.present())
				ft.setAbstract();
		} else if (n.f1.which == 1) {
			/*
			 * TODO This is not good, should get rid of all these interfaces. Type etc. instead.
			 */
			ComponentType ct = new ComponentType(name);
			ComponentedModel cm = (ComponentedModel) model;
			try {
				cm.addComponentType(ct);
			} catch (Exception e) {
				handler.complain(n.f2.beginLine, e.getMessage());
			}
			
			//	Check the type for abstractness
			if (n.f0.present())
				ct.setAbstract();			
		}
		

		return argu;
	}

	/**
	 * f0 -> <ATTRIBUTE>
	 * f1 -> <TYPE>
	 * f2 -> SyntaxDisplayIdentifier()
	 * f3 -> <EQUALS>
	 * f4 -> <LBRACE>
	 * f5 -> SyntaxValueEnumeration()
	 * f6 -> <RBRACE>
	 */
	public Object visit(SyntaxAttributeTypeDefinition n, Object argu) {
		Name name = (Name) n.f2.accept(this, argu);
		List<Name> values =  (List<Name>) n.f5.accept(this, argu);

		AttributeValueType avt = new AttributeValueType(name);

		if (model instanceof AttributedModel) {
			try {
				((AttributedModel) model).addAttributeValueType(avt);
			} catch (Exception ex) {
				handler.complain(n.f0.beginLine, ex.getMessage());
				return null;
			}				
		} else {
			handler.complain(n.f0.beginLine, "Non-attributed models cannot define attribute value types.");
			return null;
		}
		
		for (Name value : values) {
			try {
				avt.addValue(value);
			} catch (Exception e) {
				handler.warn(n.f4.beginLine, e.getMessage());
			}
		}

		return argu;
	}

	/**
	 * f0 -> SyntaxValueDefinition()
	 * f1 -> ( SyntaxEnumerationRest() )*
	 */
	public Object visit(SyntaxValueEnumeration n, Object argu) {
		ArrayList<Name> values = new ArrayList<Name>();
		values.add((Name) n.f0.accept(this, argu));

		if (!n.f1.present())
			return values;

		for (Enumeration e = n.f1.elements(); e.hasMoreElements();) {
			SyntaxEnumerationRest er = (SyntaxEnumerationRest) e.nextElement();
			values.add((Name) er.f1.accept(this, argu));
		}

		return values;
	}

	/**
	 * f0 -> <IDENTIFIER>
	 */
	public Object visit(SyntaxFunction n, Object argu) {
		Object _ret = null;
		n.f0.accept(this, argu);
		return _ret;
	}

	/**
	 * f0 -> <INTERFACE>
	 * f1 -> <TYPE>
	 * f2 -> SyntaxDisplayIdentifier()
	 * f3 -> <LBRACE>
	 * f4 -> ( <IDENTIFIER> <SEMICOLON> )*
	 * f5 -> <RBRACE>
	 */
	public Object visit(SyntaxInterfaceTypeDefinition n, Object argu) {
		Name name = (Name) n.f2.accept(this, argu);
		InterfaceType it = new InterfaceType(name);

		NodeListOptional nlo = n.f4;
		for (Enumeration e = nlo.elements(); e.hasMoreElements();) {
			NodeSequence ns = (NodeSequence) e.nextElement();

			try {
				it.addFunction(ns.nodes.elementAt(0).toString());
			} catch (Exception exc) {
				handler.complain(n.f0.beginLine, exc.getMessage());
			}
		}

		ComponentedModel km = (ComponentedModel) model;

		try {
			km.addInterfaceType(it);
		} catch (Exception e) {
			handler.complain(n.f0.beginLine, e.getMessage());
		}

		return argu;
	}
	
	/**
	 * f0 -> ( [ <MINUS> ] <INTEGER_LITERAL> | <IDENTIFIER> ) 
	 * f1 -> [ <STRING_LITERAL> ]
	 */
	public Object visit(SyntaxValueDefinition n, Object argu) {
		Name name = null;
		String display = null, val;
		
		if (n.f1.present()) {
			display = n.f1.node.toString();
			display = display.substring(1, display.length() - 1);
		}

		if (n.f0.which == 0) {
			NodeSequence nl = (NodeSequence) n.f0.choice;
			NodeOptional no = (NodeOptional) nl.elementAt(0);

			Integer intVal = new Integer(nl.elementAt(1).toString());
			if (no.present())
				intVal = new Integer(-intVal.intValue());
			
			val = intVal.toString();
		} else
			val = n.f0.choice.toString();

		if (display != null)
			return new Name(val, display);
		else
			return new Name(val);
	}
}
