/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on Feb 20, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.constraint;

import kumbang.core.model.type.AttributeDefinition;
import kumbang.core.model.type.AttributeValueType;
//import kumbang.core.model.type.ComponentType;
import kumbang.core.model.type.AttributedType;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Vector;

/**
 * @author toasikai
 * 
 */
public class AttributeReference implements Serializable {
	
	private static int nextIndex = 1;

	/**
	 * The subfeature or part the attribute of which is referred to.
	 * May be null.
	 */
	private StructureReference ref;
	
	private AttributedType origin;
		
	private String name;
	
	private int index;

	/**
	 * The set of attribute value types the referred attribute may have.
	 */
	private Vector possibleTypes = new Vector();
	
	private AttributeReference()  {
		index = nextIndex++;		
	}
	
	public AttributeReference(StructureReference ref, String name) {
		this();
		this.ref = ref;
		this.name = name;

		calculatePossibleTypes();
	}
	
	public AttributeReference(AttributedType origin, String name) {
		this();
		this.origin = origin;
		this.name = name;
		
		calculatePossibleTypes();
	}

	public AttributeReference(StructureReference ref, String name, AttributedType origin) {
		this();
		this.ref = ref;
		this.origin = origin;
		this.name = name;
		AttributeValueType avt = origin.getAttributeDefinition(name).getType();

		//	What if avt == null?
		if (avt != null)
			possibleTypes.add(avt);
		
	}

	public int getIndex() {
		return index;
	}

	public StructureReference getReference() { 
		return ref;
	}
	
	public String getName() {
		return name;
	}
	
	public Iterator getPossibleTypes() {
		return possibleTypes.iterator();
	}

	public boolean appears(Variable var) {
//		return ref.getVariable() == var;
		if (ref == null)
			return false;
		
		if (ref.getVariable() == null)
			return false;
		
		return ref.getVariable() == var || ref.getVariable().containsVariable(var);
	}
	
	public boolean equals(Object comp) {
		if (!(comp instanceof AttributeReference))
			return false;
		
		AttributeReference arComp = (AttributeReference) comp;
		
		if (ref == null) {
			if (arComp.ref != null)
				return false;
		} else if (!ref.equals(arComp.ref))
			return false;
		
		return name.equals(arComp.name);		
	}
	
	/**
	 * Calculates the set of attribute value types an attribute referred to by
	 * this reference may have.
	 *
	 */
	private void calculatePossibleTypes() {
		if (ref == null) {
			AttributeValueType avt = origin.getAttributeDefinition(name).getType();

			//	What if avt == null?
			if (avt != null)
				possibleTypes.add(avt);
				
			return;
		}
		
		for (Iterator it = ref.possibleTypes(); it.hasNext(); ) {
			AttributedType cand = (AttributedType) it.next();
			AttributeDefinition ad = cand.getAttributeDefinition(name);
			
			if (ad == null)
				continue;
			
			if (!possibleTypes.contains(ad.getType()))
				possibleTypes.add(ad.getType());			
		}
	}
	
	public int possibleTypeCount() {
		return possibleTypes.size();
	}
	
	/**
	 * Returns the first possible value type, which is possibly unique.
	 * Could also be null.
	 * 
	 * TODO Warnings in case the references point to nothing.
	 * 
	 * @return
	 */
	public AttributeValueType getType() {
		return (AttributeValueType) possibleTypes.elementAt(0);
	}
	
	public String toString() {
		String ret = "";
		
		if (ref != null)
			ret += ref.toString() + ", ";
			
		ret += name;
		
		return ret;
	}

	public String getModelString(boolean implementation) {
		String ret = "";
		
		if (ref != null)
			ret += ref.getModelString(implementation) + ", ";
			
		ret += name;
		
		return ret;
	}

	public void setName(String name) {
		this.name = name;
		
	}
}
