/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on May 12, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.constraint;

import java.util.ArrayList;

/**
 * @author toasikai
 *
 */
public class BinaryExpression extends IntegerExpression {
	public static final int opNone = 0, opMult = 1, opDiv = 2, opMod = 3, opPlus = 4, opMinus = 5;
	private static final String[] operators = { "*", "/", "mod", "+", "-" };
	
	private IntegerExpression left, right;
	private int operator;
	
	public BinaryExpression(IntegerExpression left) {
		this.left = left;
		operator = opNone;
	}

	
	public BinaryExpression(int op, IntegerExpression left, IntegerExpression right) {
		operator = op;
		this.left = left;
		this.right = right;
	}

	public boolean appears(Variable var) {
		return left.appears(var) || right.appears(var);
	}

	public void setOperator(int operator) {
		if (this.operator == opNone)
			this.operator = operator;
	}

	public void setRight(IntegerExpression expression) {
		if (right == null)
			right = expression;
	}

	public void computePrimitives(ArrayList primitives) {
		left.computePrimitives(primitives);
		right.computePrimitives(primitives);
	}
	
	public String getOperator() {
		return operators[operator-1];
	}
	
	public IntegerExpression getLeft() {
		return left;
	}
	
	public IntegerExpression getRight() {
		return right;
	}


	@Override
	public String getModelString(boolean implementation) {
		return super.toString() + left.getModelString(implementation) + getOperator() + right.getModelString(implementation);
	}
}
