/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on May 17, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.constraint;

import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

/**
 * @author toasikai
 *
 */
public abstract class BooleanExpression extends Expression {
	/**
	 * true if the expression has been negated with the keyword 'not'.
	 */
	private boolean isNegated;
	
	protected BooleanExpression parent;
	
	public void setNegated(boolean value) {
		if (value)
			isNegated = !isNegated;
	}
	
	public boolean isNegated() {
		return isNegated;
	}

	public boolean isBoolean() {
		return true;
	}
	
	public abstract boolean appears(Variable var);
	
	public QuantificationStack addedVariables() {
		if (parent != null)
			return parent.addedVariables();
			
		return null;
	}
	
	public QuantificationStack indirectlyAdded() {
		return parent.addedVariables();
	}

	public QuantificationStack incomingVariables() {
		if (parent != null)
			return parent.incomingVariables();
		
		return null;
	}
	
	/**
	 * Compared with appearingVariables(QuantificationStack remaining, Quantification previous),
	 * this method returns all the variables, whereas remaining and previous are use to 
	 * restrict the set of variables included.
	 * 
	 * @return the set of variables referred to in the constraint
	 */ 	
	public Collection appearingVariables() {
		Vector v = new Vector();
		
		for (Iterator it = variables().iterator(); it.hasNext(); ) {
			Variable var = (Variable) it.next();
			if (appears(var))
				v.add(var);
		}
		
		return v;
	}

	/**
	 * See above: appearingVariables()
	 *
	 * @param remaining
	 * @param previous
	 */
	public Collection appearingVariables(QuantificationStack remaining, Quantification previous) {
		Vector vars = new Vector();
		
		if (previous != null)
			vars.addAll(previous.variables());
		
		if (remaining != null)
			vars.addAll(remaining.variables());
		
		if (incomingVariables() != null) {
			Collection c = incomingVariables().variables();
			for (Iterator it = c.iterator(); it.hasNext(); ) {
				Object o = it.next();
				if (!vars.contains(o))
					vars.add(o);
			}
		}
		
		Vector v = new Vector();
		
		for (Iterator it = vars.iterator(); it.hasNext(); ) {
			Variable var = (Variable) it.next();
			if (appears(var))
				v.add(var);
		}
		
		return v;		
	}
	
	/**
	 * 
	 * @return the set of variables in the quantifications covering the constraint
	 */
	public Collection variables() {
		Vector v = new Vector();
		
		if (addedVariables() != null)
			v.addAll(addedVariables().variables());
			
		if (incomingVariables() != null)
			v.addAll(incomingVariables().variables());
		
		return v;
	}
	
	/**
	 * Are parents indeed needed for all kinds of expressions?
	 * 
	 * @param parent
	 */	
	public void setParent(BooleanExpression parent) {
		if (this.parent == null)
			this.parent = parent;
	}
	
	public String toString() {
		return isNegated() ? "not " : "";
	}


}
