/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on Jul 6, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.constraint;

import java.util.ArrayList;
import java.util.Collection;

/**
 * A comparison between an attribute reference and a constant value or 
 * another attribute reference.
 * 
 * @author toasikai
 *
 */
public class ComparisonExpression extends BooleanExpression {
	public static final int opEquals = 0, opNotEquals = 1, opLessThan = 2, opLessThanOrEqual = 3, opGreaterThan = 4, opGreaterThanOrEqual = 5;
	public static final String[] ops = { "==", "!=", "<", "<=", ">", ">=" } ;
	
	private int operator;
	private IntegerExpression left, right;
	
	private ArrayList primitives;
	
	public ComparisonExpression() {
		super();
	}

	public ComparisonExpression(IntegerExpression left, IntegerExpression right, int op) {
		this.left = left;
		this.right = right;
		operator = op;
	
		//	Compute IndexedValues	
		primitives = new ArrayList();
		left.computePrimitives(primitives);
		right.computePrimitives(primitives);
	}
		
	public boolean appears(Variable var) {
		return left.appears(var) || right.appears(var);		
	}
	
	public IntegerExpression getLeft() {
		return left;
	}
	
	public IntegerExpression getRight() {
		return right;
	}
	
	public Collection getPrimitives() {
		return primitives;
	}
	
	public String getOperator() {
		return ops[operator];  
	}
	
	public String toString() {
		return super.toString() + left.toString() + " " + getOperator() + " " + right.toString();
	}

	@Override
	public String getModelString(boolean implementation) {
		return (super.toString() + left.getModelString(implementation) + " " + getOperator() + " " + right.getModelString(implementation)).replace("==", "=");
	}
}
