/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on Jul 6, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.constraint;

import java.io.Serializable;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

/**
 * A series of primitive expressions concatenated using AND or OR.
 * 
 * @author toasikai
 *
 */
public class ConjunctiveExpression extends BooleanExpression implements Serializable {
	public static final int opAnd = 0, opOr = 1, opUnavail = 2;
	
	private int operator = 2;
	private List elems = new ArrayList();
	
	public ConjunctiveExpression() {}
	
	/**
	 * There seems to be no use for this constructor, after all.
	 * 
	 * @param op
	 */
	public ConjunctiveExpression(int op) {
		operator = op;
		
		if (op != opAnd && op != opOr)
			throw new Error("Invalid operator: must be either and or or.");
	}
	
	public void addPrimitive(BooleanExpression cp) {
		elems.add(cp);
		cp.setParent(this);
	}
	
	public void setOperator(int op) {
		if (operator != op && operator != opUnavail) {
			throw new Error("The operator cannot be changed.");
		}
		
		if (op != opAnd && op != opOr)
			throw new Error("Invalid operator: must be either and or or.");
			
		operator = op;
	}
	
	public Iterator getPrimitives() {
		return elems.iterator();
	}
	
	public int getOperator() {
		return operator;
	}
	
	public String toString() {
		String ret = super.toString();
		String oper = "";
		
		switch (operator) {
		case opAnd:
			oper = "and";
			break;
		case opOr:
			oper = "or";
			break;
		}
		
		Iterator it = elems.iterator();
		while (it.hasNext()) {
			ret += it.next().toString();
			
			if (it.hasNext())
				ret += " " + oper + " "; 
		}
		
		return ret;
	}

	public void setParent(BooleanExpression parent) {
		if (this.parent == null)
			this.parent = parent;
	}

	public boolean appears(Variable var) {
		for (Iterator it = elems.iterator(); it.hasNext(); ) {
			BooleanExpression cp = (BooleanExpression) it.next();
			if (cp.appears(var))
				return true;
		}
		
		return false;
	}

	@Override
	public String getModelString(boolean implementation) {
		String ret = super.toString();
		String oper = "";
		
		switch (operator) {
		case opAnd:
			oper = "and";
			break;
		case opOr:
			oper = "or";
			break;
		}
		
		Iterator<BooleanExpression> it = elems.iterator();
		while (it.hasNext()) {
			ret += it.next().getModelString(implementation);
			
			if (it.hasNext())
				ret += " " + oper + " "; 
		}
		
		return ret;
	}
}
