/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on Feb 10, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.constraint;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
 * @author toasikai
 *
 */
public class Quantification implements Serializable {
	private boolean universal;
	private ArrayList variables;
	private boolean negated;
	
	public Quantification(boolean universal, boolean negated, ArrayList variables) {
		this.universal = universal;
		this.negated = negated;
		this.variables = variables;
	}
	
	public boolean isUniversal() {
		return universal;
	}
	
	private boolean containsVariable(String name) {
		for (Iterator it = variables.iterator(); it.hasNext(); ) {
			Variable var = (Variable) it.next();
			if (var.getName().equals(name))
				return true;
		}
		
		return false;
	}
	
	/**
	 * @param compare the Quantification object to which this Quantification is compared
	 * @return null if the two Quantification's don't overlap; otherwise an overlapping variable name 
	 */
	public String overlaps(Quantification compare) {
		for (Iterator it = variables.iterator(); it.hasNext(); ) {
			String name = ((Variable) it.next()).getName();
			if (compare.containsVariable(name))
				return name;
		}
		
		return null;
	}

	public Variable getVariable(String element) {
		for (Iterator it = variables.iterator(); it.hasNext(); ) {
			Variable var = (Variable) it.next(); 
			if (var.getName().equals(element))
				return var; 
		}
		
		return null;
	}
	
	public int size() {
		return variables.size(); 
	}

	public Collection variables() {
		return variables;
	}
	
	public boolean isNegated() {
		return negated;
	}
}
