/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on Feb 10, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.constraint;

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

/**
 * @author toasikai
 *
 */
public class QuantificationStack implements Serializable {
	private Vector stack;
	
	public QuantificationStack() {
		stack = new Vector();
	}
	
	public QuantificationStack(QuantificationStack base) {
		stack = new Vector(base.stack);
	}
	
	public void push(Quantification q) {
		for (Iterator it = stack.iterator(); it.hasNext(); ) {
			Quantification qit = (Quantification) it.next();
			String conflict = qit.overlaps(q);
			if (conflict != null) {
				throw new Error();
			}
		}
		
		stack.add(0, q);
	}
	
	public Quantification pop() {
		Quantification q = (Quantification) stack.elementAt(0);
		stack.removeElementAt(0);
		
		return q; 
	}
	
	public void pop(int n) {
		for (int i = 0; i < n; i++)
			pop();
	}
	
	public Variable getVariable(String element) {
		for (Iterator it = stack.iterator(); it.hasNext(); ) {
			Quantification q = (Quantification) it.next();
			Variable v = q.getVariable(element);
			if (v != null)
				return v;
		}
		
		return null;
	}
	
	public int size() {
		return stack.size();
	}

	public Collection variables() {
		Vector ret = new Vector();
		for (Iterator it = stack.iterator(); it.hasNext(); ) {
			Quantification q = (Quantification) it.next();
			ret.addAll(q.variables());
		}
		
		return ret;
	}
	
	public boolean isEmpty() {
		return stack.isEmpty();
	}

	public Iterator iterator() {
		return stack.iterator();
	}
}
