/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on May 20, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.constraint;

import java.util.Iterator;
import java.util.Stack;

/**
 * @author toasikai
 *
 */
public class QuantifiedExpression extends BooleanExpression {
	private BooleanExpression exp;

	private QuantificationStack incoming;
	private QuantificationStack addedVariables;

	public QuantifiedExpression(BooleanExpression exp) {
		exp.setParent(this);
		this.exp = exp;
	}

	public boolean appears(Variable var) {
		return exp.appears(var);
	}

	public BooleanExpression getExpression() {
		return exp;
	}

	public void setContext(QuantificationStack incoming, QuantificationStack added) {
		this.incoming = incoming;
		this.addedVariables = added;
	} 

	public QuantificationStack addedVariables() {
		return addedVariables;
	}
	
	public QuantificationStack incomingVariables() {
		return incoming;
	}
	
	public String toString() {
		String ret = super.toString();
		
		Stack<Quantification> temp = new Stack<Quantification>();
		for (Iterator<Quantification> it = addedVariables.iterator(); it.hasNext(); )
			temp.push(it.next());
		
		while (!temp.isEmpty()) {
//		for (Iterator it = temp.iterator(); it.hasNext(); ) {
//			Quantification q = (Quantification) it.next();
			Quantification q = temp.pop();
		
			if (q.isUniversal())
				ret += "for_all(";
			else
				ret += "exists(";
			
			for (Iterator vit = q.variables().iterator(); vit.hasNext(); ) {
				Variable var = (Variable) vit.next();
				ret += var.getName() + ":" + var.getReference();
				
				if (vit.hasNext())
					ret += ", "; 
			}
			
			ret += ")";
		}
		
		return ret + " (" + exp.toString() + ")";
	}

	@Override
	public String getModelString(boolean implementation) {
		return toString();
	}
}
