/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.core.model.constraint;

import java.io.Serializable;
import kumbang.core.model.type.InstantiableType;


public class ReferenceItem implements Serializable {
	private String part;
	private InstantiableType type;
	
	public static final String WILDCARD_PART = "?";
	public static final String WILDCARD_TRANSITIVE_PART = "*";
	
	/**
	 * True if this ReferenceItem does not point to any instances.
	 */
	private boolean isVoid;
	private Variable variable;
	
	public ReferenceItem(String part) {
		this.part = part;
	}
	
	public ReferenceItem(String part, boolean isVoid) {
		this(part);
		this.isVoid = isVoid;
	}
	
	public ReferenceItem(String part, InstantiableType type) {
		this(part);
		this.type = type;
	}
	
	public String getPartName() {
		return part;
	}
	
	public InstantiableType getType() {
		return type;
	}
	
	public boolean equals(Object o) {
		if (!(o instanceof ReferenceItem))
			return false;
		
		ReferenceItem comp = (ReferenceItem) o;
		
		if ((variable != null && comp.variable == null)
				|| (variable == null && comp.variable != null))
			return false;
		
		if ((type != null && comp.type == null) || (type == null && comp.type != null))
			return false;
		
		return part.equals(comp.part) && ((variable == null && comp.variable == null) || variable.equals(comp.type)) 
				&& ((type == null && comp.type == null) || type.equals(comp.type));
	}
	
	public String toString() {
		String ret = part;
		if (type != null)
			ret += ":" + type.toString();
		
		return ret;
	}
	
	public boolean isVoid() {
		return isVoid;
	}
	
	public void setVoid() {
		this.isVoid = true;
	}

	public void setVariable(Variable var) {
		this.variable = var;
	}

	public boolean isVariable() {
		return variable != null;
	}
	
	public Variable getVariable() {
		return variable;
	}
}
