/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on Feb 10, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.constraint;

import java.io.Serializable;
import java.util.Iterator;

/**
 * @author toasikai
 *
 */
public class Variable implements Serializable {
	//	The symbol of the variable used in the input Kumbang program	
	private String name;
	
	//	The number of the variable (within the constraint)
	private int index; 
	
	/**
	 * The StructureReference that defines the path of the variable.
	 * For example, if variable is defined as X : Y.a, this StructureReference
	 * links Y to Y.a?
	 */
	private StructureReference ref;

	public Variable(String name, int id, StructureReference reference) {
		this.name = name;
		index = id;
		this.ref = reference;
	}
	
	public String getName() {
		return name;
	}

	public StructureReference getReference() {
		return ref;
	}
	
	public String toString() {
		String ret = name + " : (";
		
		for (Iterator it = ref.getPath().iterator(); it.hasNext(); ) {
			ret += it.next().toString();
			if (it.hasNext())
				ret += " -> ";
		}
		
		ret += ")";
		
		return ret;
	}

	public int getIndex() {
		return index;
	}

	public boolean containsVariable(Variable var) {
		Variable nested = ref.getVariable();
		boolean ret;
		
		ret = nested == var;
		if (nested != null)
			ret |= nested.containsVariable(var);
			
		return ret;
	}
	
	public int getCardinality() {
		return ref.getCardinality();
	}
}
