/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/**
 * 
 */
package kumbang.core.model.core;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import kumbang.core.model.exception.RootAlreadyDefinedException;
import kumbang.core.model.exception.TypeAlreadyDefinedException;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.FeatureType;

/**
 * @author vmyllarn
 *
 */
public class FeaturedModelFactory {

	public static FeaturedModel getFeaturedModel() {
		return new FeaturedModelImpl();
	}
	
	private static class FeaturedModelImpl implements FeaturedModel {
		
		private FeatureType root;
		
		private Map<String, FeatureType> featureTypes;
		
		public FeaturedModelImpl() {
			featureTypes = new HashMap<String, FeatureType>();
		}
		
		public FeatureType getFeatureType(String name) {
			return featureTypes.get(name);
		}

		public void addFeatureType(FeatureType type) throws TypeAlreadyDefinedException {
			String typeName = type.getName();
			if (featureTypes.containsKey(typeName)) {
				throw new TypeAlreadyDefinedException(type);
			}		
			featureTypes.put(type.getName(), type);
		}

		public Iterator<FeatureType> getFeatureTypes() {
			return featureTypes.values().iterator();
		}		

		public int getFeatureCount() {
			return featureTypes.size();
		}
	
		public void removeFeatureType(String name) {
			featureTypes.remove(name);
		}
		
		public FeatureType getFeatureRoot() {
			return root;
		}		
		
		public void setFeatureRoot(FeatureType type) throws RootAlreadyDefinedException {
			if (root != null) {
				throw new RootAlreadyDefinedException(type);
			}
			this.root = type;
		}

		public void removeFeatureRoot() {
			root = null;
			
		}

		public void updateTypeName(FeatureType type, String newName) {
			featureTypes.remove(type.getName());
			type.setName(newName);
			featureTypes.put(newName, type);
			
			
		}		
	}

}
