/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on Feb 12, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.core;

import kumbang.core.model.exception.RootAlreadyDefinedException;
import kumbang.core.model.exception.TypeAlreadyDefinedException;

import kumbang.core.model.type.AttributeValueType;
import kumbang.core.model.type.ComposableType;
import kumbang.core.model.type.ElementaryType;
import kumbang.core.model.type.FeatureType;

import java.util.Iterator;

/**
 * @author toasikai
 * @author vmyllarn
 *
 */
public class ForfamelModel extends Model implements FeaturedModel, AttributedModel {
	
	public static final String MODEL_KIND = "Forfamel";
	
	private FeaturedModel featuredModelImpl;
	
	private AttributedModel attributedModelImpl;	
	
	public ForfamelModel(String name) {
		super(name);
		featuredModelImpl = FeaturedModelFactory.getFeaturedModel();
		attributedModelImpl = AttributedModelFactory.getAttributedModel();
	}
	
	/**
	 * Returns an upper-case string representation of this
	 * model and its kind, i.e., "Forfamel".
	 * @return String "Forfamel"
	 */		
	public String getModelKind() {
		return ForfamelModel.MODEL_KIND;
	}

	/**
	 * Returns a type with the given name and given type kind.
	 * Returns null if no such type exists, or the model does not
	 * support this particular type kind.
	 * @param name
	 * @param typeKind
	 * @return ElementaryType
	 */	
	public ElementaryType getType(String name, String typeKind) {
		if (typeKind.equals(FeatureType.TYPE_KIND)) {
			return featuredModelImpl.getFeatureType(name);
		} else if (typeKind.equals(AttributeValueType.TYPE_KIND)) {
			return attributedModelImpl.getAttributeValueType(name);
		} else {
			return null;
		}
	}

	public void addFeatureType(FeatureType type) throws TypeAlreadyDefinedException {
		featuredModelImpl.addFeatureType(type);
	}

	public int getFeatureCount() {
		return featuredModelImpl.getFeatureCount();
	}

	public FeatureType getFeatureRoot() {
		return featuredModelImpl.getFeatureRoot();
	}

	public FeatureType getFeatureType(String name) {
		return featuredModelImpl.getFeatureType(name);
	}

	public Iterator<FeatureType> getFeatureTypes() {
		return featuredModelImpl.getFeatureTypes();
	}

	public void removeFeatureType(String name) {
		featuredModelImpl.removeFeatureType(name);
	}

	public void setFeatureRoot(FeatureType type) throws RootAlreadyDefinedException {
		featuredModelImpl.setFeatureRoot(type);
	}

	public void addAttributeValueType(AttributeValueType type) throws TypeAlreadyDefinedException {
		attributedModelImpl.addAttributeValueType(type);
	}

	public int getAttributeValueCount() {
		return attributedModelImpl.getAttributeValueCount();
	}

	public AttributeValueType getAttributeValueType(String name) {
		return attributedModelImpl.getAttributeValueType(name);
	}

	public Iterator<AttributeValueType> getAttributeValueTypes() {
		return attributedModelImpl.getAttributeValueTypes();
	}

	public void removeAttributeValueType(String name) {
		attributedModelImpl.removeAttributeValueType(name);
	}

	public void removeFeatureRoot() {
		featuredModelImpl.removeFeatureRoot();
		
	}

	public void updateTypeName(FeatureType type, String newName) {
		featuredModelImpl.updateTypeName(type, newName);
		
	}
	
	public void updateTypeName(AttributeValueType type, String newName) {
		attributedModelImpl.updateTypeName(type, newName);
		
	}



}
