/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on Feb 26, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.core;

import kumbang.core.model.exception.RootAlreadyDefinedException;
import kumbang.core.model.exception.TypeAlreadyDefinedException;

import kumbang.core.model.type.AttributeValueType;
import kumbang.core.model.type.ComponentType;
import kumbang.core.model.type.ElementaryType;
import kumbang.core.model.type.InterfaceType;

import java.util.Iterator;

/**
 * @author toasikai
 * @author vmyllarn
 *
 */
public class KoalishModel extends Model implements ComponentedModel, AttributedModel {

	public static final String MODEL_KIND = "Koalish";
	
	private ComponentedModel componentedModelImpl;
	
	private AttributedModel attributedModelImpl;
	
	public KoalishModel(String name) {
		super(name);
		componentedModelImpl = ComponentedModelFactory.getComponentedModel();
		attributedModelImpl = AttributedModelFactory.getAttributedModel();
	}
	
	/**
	 * Returns an upper-case string representation of this
	 * model and its kind, i.e., "Koalish".
	 * @return String "Koalish"
	 */	
	public String getModelKind() {
		return KoalishModel.MODEL_KIND;
	}

	/**
	 * Returns a type with the given name and given type kind.
	 * Returns null if no such type exists, or the model does not
	 * support this particular type kind.
	 * @param name
	 * @param typeKind
	 * @return ElementaryType
	 */
	public ElementaryType getType(String name, String typeKind) {
		if (typeKind.equals(ComponentType.TYPE_KIND)) {
			return componentedModelImpl.getComponentType(name);
		} else if (typeKind.equals(InterfaceType.TYPE_KIND)) {
			return componentedModelImpl.getInterfaceType(name);
		} else if (typeKind.equals(AttributeValueType.TYPE_KIND)) {
			return attributedModelImpl.getAttributeValueType(name);
		} else {
			return null;
		}
	}


	public void addComponentType(ComponentType type) throws TypeAlreadyDefinedException {
		componentedModelImpl.addComponentType(type);
	}

	public void addInterfaceType(InterfaceType type) throws TypeAlreadyDefinedException {
		componentedModelImpl.addInterfaceType(type);
	}

	public int getComponentCount() {
		return componentedModelImpl.getComponentCount();
	}

	public ComponentType getComponentRoot() {
		return componentedModelImpl.getComponentRoot();
	}

	public ComponentType getComponentType(String name) {
		return componentedModelImpl.getComponentType(name);
	}

	public Iterator<ComponentType> getComponentTypes() {
		return componentedModelImpl.getComponentTypes();
	}

	public int getInterfaceCount() {
		return componentedModelImpl.getInterfaceCount();
	}

	public InterfaceType getInterfaceType(String name) {
		return componentedModelImpl.getInterfaceType(name);
	}

	public Iterator<InterfaceType> getInterfaceTypes() {
		return componentedModelImpl.getInterfaceTypes();
	}

	public void removeComponentType(String name) {
		componentedModelImpl.removeComponentType(name);
		
	}

	public void removeInterfaceType(String name) {
		componentedModelImpl.removeInterfaceType(name);
	}

	public void setComponentRoot(ComponentType type) throws RootAlreadyDefinedException {
		componentedModelImpl.setComponentRoot(type);
	}

	public void addAttributeValueType(AttributeValueType type) throws TypeAlreadyDefinedException {
		attributedModelImpl.addAttributeValueType(type);
	}

	public int getAttributeValueCount() {
		return attributedModelImpl.getAttributeValueCount();
	}

	public AttributeValueType getAttributeValueType(String name) {
		return attributedModelImpl.getAttributeValueType(name);
	}

	public Iterator<AttributeValueType> getAttributeValueTypes() {
		return attributedModelImpl.getAttributeValueTypes();
	}

	public void removeAttributeValueType(String name) {
		attributedModelImpl.removeAttributeValueType(name);
	}

	public void removeComponentRoot() {
		componentedModelImpl.removeComponentRoot();
		
	}

	public void updateTypeName(ComponentType type, String newName) {
		componentedModelImpl.updateTypeName(type, newName);
		
		
	}

	public void updateTypeName(InterfaceType type, String newName) {
		componentedModelImpl.updateTypeName(type, newName);
		
	}
	
	public void updateTypeName(AttributeValueType type, String newName) {
		attributedModelImpl.updateTypeName(type, newName);
		
	}

//	/* (non-Javadoc)
//	 * @see kumbang.model.type.Model#hasConstraints()
//	 */
//	public boolean hasConstraints() {
//		for (Iterator it = componentTypes.values().iterator(); it.hasNext(); ) {
//			ComponentType ct = (ComponentType) it.next();
//			if (ct.getConstraints().hasNext())
//				return true;
//		}
//		
//		return false;	
//	}	
//
//	/* (non-Javadoc)
//	 * @see kumbang.model.type.Model#hasConnections()
//	 */
//	public boolean hasConnections() {
//		for (Iterator it = componentTypes.values().iterator(); it.hasNext(); ) {
//			ComponentType ct = (ComponentType) it.next();
//			if (ct.getConnections().hasNext())
//				return true;
//		}
//		
//		return false;
//	}
//
//	/* (non-Javadoc)
//	 * @see kumbang.model.type.Model#hasInterfaces()
//	 */
//	public boolean hasInterfaces() {
//		for (Iterator it = componentTypes.values().iterator(); it.hasNext(); ) {
//			ComponentType ct = (ComponentType) it.next();
//			if (ct.getInterfaces().hasNext())
//				return true;
//		}
//		
//		return false;
//	}
	
}
