/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/**
 * 
 */
package kumbang.core.model.exception;

import kumbang.core.model.type.ElementaryType;

/**
 * @author vmyllarn
 *
 */
public class PropertyAlreadyDefinedException extends Exception {


	private String propertyName;
	
	/**
	 * @param enclosingType type that tries to define the property
	 * @param propertyKind for example, "function" or "attribute value"
	 * @param propertyName name of the particular property we are trying to re-define
	 */
	public PropertyAlreadyDefinedException(ElementaryType enclosingType, String propertyKind, String propertyName) {
		super("The " + enclosingType.getTypeKind() + " type named " + enclosingType.getName() 
				+ " already defines a " + propertyKind + " " + propertyName);
		this.propertyName = propertyName;
	}

	public String getPropertyName() {
		return this.propertyName;
	}
}
