/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on Jan 20, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.type;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import kumbang.core.model.exception.PropertyAlreadyDefinedException;

/**
 * @author toasikai
 *
 */
public class AttributeValueType extends ElementaryType {
	public static final String TYPE_KIND = "attribute";
	
	private List<Name> values;
    
	// is hashmap an overkill, should one just iterate through values?
    private HashMap<String, Name> valueMap; 
    
    public AttributeValueType(String name) {
        super(name);
        this.values = new ArrayList<Name>();
        this.valueMap = new HashMap<String, Name>();
    }
    
    public AttributeValueType(Name name) {
        super(name);
        this.values = new ArrayList<Name>();
        this.valueMap = new HashMap<String, Name>();
    }

    public void addValue(String value) throws PropertyAlreadyDefinedException {
    	this.addValue(new Name(value));
    }    
    
    public void addValue(String value, String displayValue) throws PropertyAlreadyDefinedException {
    	this.addValue(new Name(value, displayValue));
    }      
    
    public void addValue(Name value) throws PropertyAlreadyDefinedException {
    	for (Name v : values) {
    		if (v.getName().equals(value.getName())) {
    			throw new PropertyAlreadyDefinedException(this, "attribute value", value.getName());
    		} else if (v.getDisplayName().equals(value.getDisplayName())) {
    			throw new PropertyAlreadyDefinedException(this, "attribute value", value.getDisplayName());
    		}
    	}
        values.add(value);
        valueMap.put(value.getName(), value);
    }
    
	public boolean includesValue(Name value) {
		if (value == null)
			return true;
		
		return valueMap.containsKey(value.getName());
	}
    
    public boolean includesValue(String value) {
        return valueMap.containsKey(value);
    }
    
    public Name getValue(String value) {
        return valueMap.get(value);
    }
    
	public Iterator<Name> getValues() {
		return values.iterator();
	}

	public String getTypeKind() {
		return TYPE_KIND;
	}
	
	public void setValues(Iterator<Name> names) throws PropertyAlreadyDefinedException {
		values = new ArrayList<Name>();
		while (names.hasNext())
			addValue(names.next());
	}

	/**
	 * @param index
	 * @return
	 */
	public String getValue(int index) {
		try {
			return values.get(index).getName();
		} catch (IndexOutOfBoundsException e) {
			return null;
		}
	}
}
