/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on Feb 25, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.type;

import kumbang.core.model.constraint.OldInterfaceReference;

import kumbang.core.model.exception.PropertyAlreadyDefinedException;

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;

/**
 * @author toasikai
 *
 */
public class ComponentType extends ComposableType {
	
	public static final String TYPE_KIND = "component";
	
	private List<InterfaceDefinition> directInterfaces = new ArrayList<InterfaceDefinition>();
	private List<InterfaceDefinition> interfaces = new ArrayList<InterfaceDefinition>();
	private List connections = new ArrayList();

	public ComponentType(String name) {
		super(name);
	}

	public ComponentType(Name name) {
		super(name);
	}

	public void addInterface(InterfaceDefinition def) throws PropertyAlreadyDefinedException {
		addInterface(def, true);
	}
	
	private void addInterface(InterfaceDefinition def, boolean direct) throws PropertyAlreadyDefinedException {
		String interfaceName = def.getName();
		if (interfaces.contains(def)) 
			throw new PropertyAlreadyDefinedException(this, "interface", interfaceName);
		
		interfaces.add(def);
		for (ComposableType subtype : subtypes)
			((ComponentType) subtype).addInterface(def, false);
	}

	/**
	 * Returns a lower-case string that represents name of the entities
	 * of the type.
	 * 
	 * Could also suplement functions for part of and
	 * possible part of kind of predicates, and the initial
	 * letter used in variable names in smodels; on the other hand,
	 * this class should not know anything about smodels, so the translate
	 * package could make its own decisions.
	 * 
	 * @return string "component"
	 */
	public String getTypeKind() {
		return TYPE_KIND;
	}	

    public boolean hasInterfaces() {
        return !interfaces.isEmpty();
    }
    
	public Iterator getInterfaces() {
		return interfaces.iterator();
	}

	public void addConnection(Connection connection) throws PropertyAlreadyDefinedException {
		for (Iterator iter = connections.iterator(); iter.hasNext(); ) {
			Connection c = (Connection) iter.next();
			if (c.getSourceInterface().equals(connection.getSourceInterface()) 
					&& c.getTargetInterface().equals(connection.getSourceInterface())) {
				throw new PropertyAlreadyDefinedException(this, "connection", 
						"from " + c.getSourceInterface().getName() + " to " + c.getTargetInterface().getName());
			}
		}
		connections.add(connection);
	}

	public InterfaceDefinition getInterface(String name) {
		for (Iterator it = interfaces.iterator(); it.hasNext(); ) {
			InterfaceDefinition id = (InterfaceDefinition) it.next();
			if (id.getName().equals(name))
				return id;
		}

		// Not found.
		return null;
	}

	public Iterator getConnections() {
		return connections.iterator();
	}

	public void removeInterfaceDefinition(String definitionName) {
		interfaces.remove(getInterface(definitionName));
		
	}

	public int getSize() {
		int ret = 0;
		
		ret = super.getSize();
		
		for (Iterator it = getInterfaces(); it.hasNext(); ) {
			InterfaceDefinition def = (InterfaceDefinition) it.next();
			ret += def.getPossibleTypesArray().length;
		}
		
		return ret;
	}
	
	public int getInterfaceDefinitionOffset(InterfaceDefinition def) {
		// The size of all part definitions.
		int ret = super.getSize();
		
		for (Iterator it = getInterfaces(); it.hasNext(); ) {
			InterfaceDefinition comp = (InterfaceDefinition) it.next();
			if (comp.equals(def))
				return ret;
			
			ret += comp.getSize();
		}
		
		// Error, definition was not found.
		return -1;		
	}

	public void removeConnection(Connection connection) {
		connections.remove(connection);
	}

	public boolean hasConnections() {
		return connections.size() > 0;
	}
}
