/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on Feb 26, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.type;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 * @author toasikai
 *
 * Implements comparable, as we want to preserve the order of the interfaces in a ComponentInstance
 * object after remove and restore operations. 
 */
public class InterfaceDefinition extends ElementaryDefinition implements Comparable {
	private List types;
	
	private boolean optional;
	private boolean grounded;
	private boolean provides;

	public InterfaceDefinition(Name name, Collection types, boolean groud, boolean opt, boolean prov) {
		super(name);
		
		this.types = new ArrayList(types);
		this.optional = opt;
		this.grounded = groud;
		this.provides = prov;
	}
	
	public InterfaceDefinition(String name, Collection types, boolean groud, boolean opt, boolean prov) {
		super(name);
		
		this.types = new ArrayList(types);
		this.optional = opt;
		this.grounded = groud;
		this.provides = prov;
	}	
	
	public Iterator getPossibleTypes() {
		return types.iterator();
	}
	
	public List getPossibleTypesList() {
		return types;
	}	

	public boolean isOptional() {
		return optional;
	}

	public boolean isGrounded() {
		return grounded;
	}

	public boolean isProvides() {
		return provides;
	}
	
	public Object[] getPossibleTypesArray() {
		return types.toArray();
	}

	public void setGrounded(boolean b) {
		grounded = b;
		
	}
	
	public void setOptional(boolean b) {
		optional = b;
		
	}

	public void addPossibleType(InterfaceType interfaceType) {
		if (!types.contains(interfaceType))
			types.add(interfaceType);
		
	}

	public void removePossibleType(InterfaceType interfaceType) {
		types.remove(interfaceType);
	}
	
	public int getSize() {
		return types.size();
	}

	public int getTypeOffset(InterfaceType type) {
		return types.indexOf(type);
	}
	
	public InterfaceType getInterfaceTypeAtIndex(int index) {
		return (InterfaceType) types.get(index);
	}

	public int compareTo(Object comp) {
		if (!(comp instanceof InterfaceDefinition))
			throw new ClassCastException();
		
		return getName().compareTo(((InterfaceDefinition) comp).getName());
	}
	
	public void setProvides(boolean provides) {
		this.provides = provides;
	}
	
	public InterfaceType getType(String type) {
		for (Iterator it = types.iterator(); it.hasNext(); ) {
			InterfaceType comp = (InterfaceType) it.next();
			if (comp.getName().toString().equals(type))
				return comp;
		}
		
		// Not found.
		return null;
	}
	
	public boolean equals(Object o) {
		if (!(o instanceof InterfaceDefinition))
			return false;
		
		return ((InterfaceDefinition) o).getName().equals(getName());
	}
}
