/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on Feb 26, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.type;

import kumbang.core.model.exception.PropertyAlreadyDefinedException;

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;

/**
 * @author toasikai
 *
 */
public class InterfaceType extends InstantiableType implements AttributedType {
	
	public static final String TYPE_KIND = "interface";
	
	private List<String> functions;
    private List<InterfaceType> subtypes;
    
    private AttributedType attributeImpl;
	
	public InterfaceType(String name) {
		this(new Name(name));
	}

    public InterfaceType(Name name) {
        super(name);
        functions = new ArrayList<String>();
        subtypes = new ArrayList<InterfaceType>();
        attributeImpl = AttributedTypeFactory.getAttributedTypeImplementation(this);
    }
    
    /**
     * Returns a lower-case string that represents name of the entities
     * of the type.
     * @return String "interface"
     */
    public String getTypeKind() {
        return TYPE_KIND;
    }
    
    /**
     * 
     * @param f a string describing the function to be added
     */
    public void addFunction(String f) throws PropertyAlreadyDefinedException {
        if (functions.contains(f)) {
            throw new PropertyAlreadyDefinedException(this, "function", f);
        }
        functions.add(f);
    }
	
    /**
     * 
     * @param type
     */
    public void addSubtype(InterfaceType type) {
        if (!subtypes.contains(type) && isSubtype(type))
            subtypes.add(type);
    }
	
    /**
     * Returns true if test is a subtype (in Koala sense) of this the interface type
     * represented by this; otherwise false.
     * 
     * @param test
     * @return
     */
    public boolean isSubtype(InterfaceType test) {
        return test.functions.containsAll(functions);
    }

    public Iterator getFunctions() {
        return functions.iterator();
    }

    public boolean hasFunction(String function) {
        return functions.contains(function);
    }

    /**
     * Returns true if this InterfaceType and test have disjoint sets of
     * functions, i.e., there is no function f that is included both in 
     * this InterfaceType and in test  
     * 
     * @param test an InterfaceType to be tested
     * @return true if the interfaces are disjoint, otherwise false
     */
    public boolean isDisjointFrom(InterfaceType test) {
        for (Iterator it = functions.iterator(); it.hasNext(); ) {
            if (test.functions.contains(it.next()))
                return false;
        }

        return true;
    }
    

	public void setFunctions(String[] items) throws PropertyAlreadyDefinedException {
		functions = new ArrayList(items.length);
		for (int i = 0; i < items.length; i++)
			addFunction(items[i]);
		
	}

	public int getSize() {
		return 1;
	}

	public void addAttribute(AttributeDefinition attr) throws PropertyAlreadyDefinedException {
		attributeImpl.addAttribute(attr);
	}
	
	public void addInheritedAttribute(AttributeDefinition attr) throws PropertyAlreadyDefinedException {
		attributeImpl.addInheritedAttribute(attr);		
	}

	public AttributeDefinition getAttributeDefinition(String name) {
		return attributeImpl.getAttributeDefinition(name);
	}

	public Iterator<AttributeDefinition> getAttributes() {
		return attributeImpl.getAttributes();
	}

	public Iterator<AttributeDefinition> getDirectAttributes() {
		return attributeImpl.getDirectAttributes();
	}
	
	public void removeAttribute(AttributeDefinition attr) {
		attributeImpl.removeAttribute(attr);
	}	

}
