/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * Created on May 27, 2005
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package kumbang.core.model.type;

import java.io.Serializable;

/**
 * A name of an entity, includes the name used for the entity in a model file
 * and a display name used in user interfaces.
 * 
 * @author toasikai
 */
public class Name implements Serializable {
	private String name;
	private String displayName;
	
	public Name(String name) {
		this.name = new String(name);
	}
	
	public Name(String name, String displayName) {
		this.name = new String(name);
		this.displayName = new String(displayName);
	}
	
	public String getName() {
		return name;
	}
	
	
	public String getDisplayName() {
        if (displayName != null && displayName.length() > 0) {
            return displayName;
        } else {
            return name;
        }
    }
    
    public boolean hasDisplayName() {
        return (displayName != null && displayName.length() > 0);
    }

	public void setName(String name) {
		this.name = name;		
	}
	
	public void setDisplayName(String name) {
		this.displayName = name;
	}
    
    /* should this be changed to show both names? like <name> "<displayname>" */
    public String toString() {
        return name;
    }
    
    /**
     * Overriden to compare to name objects. Checks whether name fields
     * in these to names are equal.
     * @param o
     * @return boolean
     */
    public boolean equals(Object o) {
        return o instanceof Name
            && ((Name) o).getName().equals(name);
            // && ((Name) o).getDisplayName().equals(displayName)
            /* 
             * Should not compare display names, since one cannot have
             * two name objects that have equal name but different
             * display names.
             * 
             * - This is not the case, e.g., part definitions of different types
             * may have the same name but different display name. -toasikai 13.9.2006 
             */
    }
}
