/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


package kumbang.util;


import java.io.PrintStream;

import java.util.LinkedList;
import java.util.Iterator;
import java.util.List;



public class ErrorHandler {

    private List<String> errors;
    private List<String> warnings;

	
    public ErrorHandler() {
        errors = new LinkedList<String>();
        warnings = new LinkedList<String>();
    }

	
    public void complain(String errorMsg) {
        errors.add("Error: " + errorMsg);
    }

	

    public void complain(int lineNumber, String msg) {
        String line = "Line " + lineNumber + ": ";
        complain(line + msg);
    }

	

    public void warn(String warning) {
        warnings.add("Warning: " + warning);
    }

	public void warn(int lineNumber, String msg) {
		String line = "Line " + lineNumber + ": ";
		warn(line + msg);
	}

    public boolean hasAnyErrors() {
    	return !errors.isEmpty();
    }

    public boolean hasAnyWarnings() {
    	return !warnings.isEmpty();
    }
    
    public boolean hasAnyNotices() {
        return hasAnyErrors() || hasAnyWarnings();
    }
	
    public void report() {
        report(System.out);
    }
    
    
    public void report(PrintStream out) {
        StringBuffer buffer = new StringBuffer();
        report(buffer);
        out.print(buffer.toString());
    }
    
    public void report(StringBuffer buffer) {
        if (buffer == null) {
            buffer = new StringBuffer();
        }
        if (errors.isEmpty()) {
            return;
        }

        Iterator iter = errors.iterator();
        while (iter.hasNext()) {
            buffer.append(iter.next()+"\n");
        }
        iter = warnings.iterator();
        while (iter.hasNext()) {
            buffer.append(iter.next()+"\n");
        }
        
        buffer.append(errors.size() + " error");
        if (errors.size() != 1) {
            buffer.append("s");
        }
        buffer.append(", " + warnings.size() + " warning");
        if (warnings.size() != 1) {
            buffer.append("s");
        }
        buffer.append("."+"\n");
    }

    public void reset() {
        errors.clear();
        warnings.clear();
    }

}
