/*
This file is part of KumbangTools

    KumbangTools is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    KumbangTools is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with KumbangTools; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/


/*
 * KumbangFileChooser.java
 *
 * Created on 26. huhtikuuta 2004, 10:45
 */

package kumbang.util.gui;

import java.awt.Component;

import java.io.File;

import javax.swing.JFileChooser;

import javax.swing.filechooser.FileFilter;

/**
 *
 * @author  vmyllarn
 */
public class KumbangFileChooser extends JFileChooser {
    
    /** file filter for different files */
    private KumbangFileFilter fileFilter;
    
    
    /** Creates a new instance of KumbangFileChooser */
    public KumbangFileChooser() {
        // init filechooser
        super(".");
        //System.out.println("KumbangFileChooser: Init file filter");
        fileFilter = new KumbangFileFilter();
        //System.out.println("KumbangFileChooser: Set file filter");
        setFileFilter(fileFilter);
    }
    
    public boolean showChooseModelFileDialog(Component parent) {
        setFileSelectionMode(JFileChooser.FILES_ONLY);
        fileFilter.setModelFilter();
        return showOpenDialog(parent) == JFileChooser.APPROVE_OPTION;
    }
    
    public boolean showChooseConfigurationFileDialog(Component parent) {
        setFileSelectionMode(JFileChooser.FILES_ONLY);
        fileFilter.setConfigurationFilter();
        return showOpenDialog(parent) == JFileChooser.APPROVE_OPTION;
    }
    
    public boolean showChooseDirectoryDialog(Component parent) {
        setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        fileFilter.setDirectoryFilter();
        return showOpenDialog(parent) == JFileChooser.APPROVE_OPTION;
    }
    
    class KumbangFileFilter extends FileFilter {
        
        private boolean directoriesOnly;
        private String suffix;
        private String description;
        
        public KumbangFileFilter() {
            setModelFilter();
        }
        
        public void setModelFilter() {
            directoriesOnly = false;
            suffix = ".kbm";
            description = "Kumbang model files (\".kbm\")";
        }
        
        public void setConfigurationFilter() {
            directoriesOnly = false;
            suffix = ".kbc";
            description = "Kumbang configuration files (\".kbc\")";
        }
        
        public void setDirectoryFilter() {
            directoriesOnly = true;
            description = "Directories";
        }
        
        public boolean accept(File file) {
            if (file.isDirectory()) {
                return true;
            }
            if (directoriesOnly) {
                return false;
            }
            if (file.getName().endsWith(suffix) && file.canRead()) {
                return true;
            }
            return false;
        }
        
        public String getDescription() {
            return description;
        }
    }
}
